@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -14,16 +14,16 @@
 //
 //*************************************************************************
 
-#ifndef _V3ASTCONSTONLY_H_
-#define _V3ASTCONSTONLY_H_ 1
+#ifndef VERILATOR_V3ASTCONSTONLY_H_
+#define VERILATOR_V3ASTCONSTONLY_H_
 
 // Include only in visitors that do not not edit nodes, so should use constant iterators
-#define iterateAndNext   error_use_iterateAndNextConst
-#define iterateChildren  error_use_iterateChildrenConst
+#define iterateAndNext error_use_iterateAndNextConst
+#define iterateChildren error_use_iterateChildrenConst
 
-#define addNext          error_no_addNext_in_ConstOnlyVisitor
-#define replaceWith      error_no_replaceWith_in_ConstOnlyVisitor
-#define deleteTree       error_no_deleteTree_in_ConstOnlyVisitor
-#define unlinkFrBack     error_no_unlinkFrBack_in_ConstOnlyVisitor
+#define addNext error_no_addNext_in_ConstOnlyVisitor
+#define replaceWith error_no_replaceWith_in_ConstOnlyVisitor
+#define deleteTree error_no_deleteTree_in_ConstOnlyVisitor
+#define unlinkFrBack error_no_unlinkFrBack_in_ConstOnlyVisitor
 
 #endif  // Guard
