@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -50,11 +50,11 @@ const char* AstNodeVarRef::broken() const {
 }
 
 void AstNodeVarRef::cloneRelink() {
-    if (m_varp && m_varp->clonep()) { m_varp = m_varp->clonep(); }
+    if (m_varp && m_varp->clonep()) m_varp = m_varp->clonep();
 }
 
 string AstNodeVarRef::hiernameProtect() const {
-    return VIdProtect::protectWordsIf(hiername(), protect());
+    return hiernameToUnprot() + VIdProtect::protectWordsIf(hiernameToProt(), protect());
 }
 
 int AstNodeSel::bitConst() const {
@@ -68,7 +68,7 @@ void AstNodeUOrStructDType::repairMemberCache() {
         if (m_members.find(itemp->name()) != m_members.end()) {
             itemp->v3error("Duplicate declaration of member name: " << itemp->prettyNameQ());
         } else {
-            m_members.insert(make_pair(itemp->name(), itemp));
+            m_members.emplace(itemp->name(), itemp);
         }
     }
 }
@@ -99,7 +99,7 @@ void AstNodeCCall::dump(std::ostream& str) const {
     }
 }
 void AstNodeCCall::cloneRelink() {
-    if (m_funcp && m_funcp->clonep()) { m_funcp = m_funcp->clonep(); }
+    if (m_funcp && m_funcp->clonep()) m_funcp = m_funcp->clonep();
 }
 const char* AstNodeCCall::broken() const {
     BROKEN_RTN(m_funcp && !m_funcp->brokeExists());
@@ -107,7 +107,7 @@ const char* AstNodeCCall::broken() const {
 }
 bool AstNodeCCall::isPure() const { return funcp()->pure(); }
 string AstNodeCCall::hiernameProtect() const {
-    return VIdProtect::protectWordsIf(hiername(), protect());
+    return hiernameToUnprot() + VIdProtect::protectWordsIf(hiernameToProt(), protect());
 }
 
 void AstNodeCond::numberOperate(V3Number& out, const V3Number& lhs, const V3Number& rhs,
@@ -455,38 +455,100 @@ string AstVar::cPubArgType(bool named, bool forReturn) const {
     return arg;
 }
 
-string AstVar::dpiArgType(bool named, bool forReturn) const {
-    if (forReturn) named = false;
-    string arg;
-    if (isDpiOpenArray()) {
-        arg = "const svOpenArrayHandle";
-    } else if (!basicp()) {
-        arg = "UNKNOWN";
-    } else if (basicp()->isDpiBitVec()) {
-        if (forReturn) {
-            arg = "svBitVecVal";
-        } else if (isReadOnly()) {
-            arg = "const svBitVecVal*";
-        } else {
-            arg = "svBitVecVal*";
-        }
-    } else if (basicp()->isDpiLogicVec()) {
-        if (forReturn) {
-            arg = "svLogicVecVal";
-        } else if (isReadOnly()) {
-            arg = "const svLogicVecVal*";
-        } else {
-            arg = "svLogicVecVal*";
-        }
-    } else {
-        arg = basicp()->keyword().dpiType();
-        if (basicp()->keyword().isDpiUnsignable() && !basicp()->isSigned()) {
-            arg = "unsigned " + arg;
-        }
-        if (!forReturn && isWritable()) arg += "*";
+class dpiTypesToStringConverter VL_NOT_FINAL {
+public:
+    virtual string openArray(const AstVar*) const { return "const svOpenArrayHandle"; }
+    virtual string bitLogicVector(const AstVar* varp, bool isBit) const {
+        return isBit ? "svBitVecVal" : "svLogicVecVal";
     }
-    if (named) arg += " " + name();
-    return arg;
+    virtual string primitive(const AstVar* varp) const {
+        string type;
+        const AstBasicDTypeKwd keyword = varp->basicp()->keyword();
+        if (keyword.isDpiUnsignable() && !varp->basicp()->isSigned()) type = "unsigned ";
+        type += keyword.dpiType();
+        return type;
+    }
+    string convert(const AstVar* varp) const {
+        if (varp->isDpiOpenArray()) {
+            return openArray(varp);
+        } else if (const AstBasicDType* basicp = varp->basicp()) {
+            if (basicp->isDpiBitVec() || basicp->isDpiLogicVec()) {
+                return bitLogicVector(varp, basicp->isDpiBitVec());
+            } else {
+                return primitive(varp);
+            }
+        } else {
+            return "UNKNOWN";
+        }
+    }
+};
+
+string AstVar::dpiArgType(bool named, bool forReturn) const {
+    if (forReturn) {
+        return dpiTypesToStringConverter{}.convert(this);
+    } else {
+        class converter final : public dpiTypesToStringConverter {
+            virtual string bitLogicVector(const AstVar* varp, bool isBit) const override {
+                return string(varp->isReadOnly() ? "const " : "")
+                       + dpiTypesToStringConverter::bitLogicVector(varp, isBit) + '*';
+            }
+            virtual string primitive(const AstVar* varp) const override {
+                string type = dpiTypesToStringConverter::primitive(varp);
+                if (varp->isWritable() || VN_IS(varp->dtypep()->skipRefp(), UnpackArrayDType)) {
+                    if (!varp->isWritable()
+                        && varp->basicp()->keyword() != AstBasicDTypeKwd::STRING)
+                        type = "const " + type;
+                    type += "*";
+                }
+                return type;
+            }
+        };
+        string arg = converter{}.convert(this);
+        if (named) arg += " " + name();
+        return arg;
+    }
+}
+
+string AstVar::dpiTmpVarType(const string& varName) const {
+    class converter final : public dpiTypesToStringConverter {
+        string m_name;
+        string arraySuffix(const AstVar* varp, size_t n) const {
+            if (AstUnpackArrayDType* unpackp
+                = VN_CAST(varp->dtypep()->skipRefp(), UnpackArrayDType)) {
+                // Convert multi dimensional unpacked array to 1D array
+                if (n == 0) n = 1;
+                n *= unpackp->arrayUnpackedElements();
+                return '[' + cvtToStr(n) + ']';
+            } else if (n > 0) {
+                return '[' + cvtToStr(n) + ']';
+            } else {
+                return "";
+            }
+        }
+        virtual string openArray(const AstVar* varp) const override {
+            return dpiTypesToStringConverter::openArray(varp) + ' ' + m_name
+                   + arraySuffix(varp, 0);
+        }
+        virtual string bitLogicVector(const AstVar* varp, bool isBit) const override {
+            string type = dpiTypesToStringConverter::bitLogicVector(varp, isBit);
+            type += ' ' + m_name + arraySuffix(varp, varp->widthWords());
+            return type;
+        }
+        virtual string primitive(const AstVar* varp) const override {
+            string type = dpiTypesToStringConverter::primitive(varp);
+            if (varp->isWritable() || VN_IS(varp->dtypep()->skipRefp(), UnpackArrayDType)) {
+                if (!varp->isWritable() && varp->basicp()->keyword() == AstBasicDTypeKwd::CHANDLE)
+                    type = "const " + type;
+            }
+            type += ' ' + m_name + arraySuffix(varp, 0);
+            return type;
+        }
+
+    public:
+        explicit converter(const string& name)
+            : m_name(name) {}
+    };
+    return converter{varName}.convert(this);
 }
 
 string AstVar::scType() const {
@@ -553,7 +615,7 @@ string AstVar::mtasksString() const {
     return os.str();
 }
 
-class AstNodeDType::CTypeRecursed {
+class AstNodeDType::CTypeRecursed final {
 public:
     string m_type;  // The base type, e.g.: "Foo_t"s
     string m_dims;  // Array dimensions, e.g.: "[3][2][1]"
@@ -573,6 +635,7 @@ string AstNodeDType::cType(const string& name, bool forFunc, bool isRef) const {
 }
 
 AstNodeDType::CTypeRecursed AstNodeDType::cTypeRecurse(bool compound) const {
+    // Legacy compound argument currently just passed through and unused
     CTypeRecursed info;
 
     const AstNodeDType* dtypep = this->skipRefp();
@@ -592,12 +655,11 @@ AstNodeDType::CTypeRecursed AstNodeDType::cTypeRecurse(bool compound) const {
     } else if (const auto* adtypep = VN_CAST_CONST(dtypep, ClassRefDType)) {
         info.m_type = "VlClassRef<" + EmitCBaseVisitor::prefixNameProtect(adtypep) + ">";
     } else if (const auto* adtypep = VN_CAST_CONST(dtypep, UnpackArrayDType)) {
-        if (compound) {
-            v3fatalSrc("Dynamic arrays or queues with unpacked elements are not yet supported");
-        }
+        if (adtypep->isCompound()) compound = true;
         const CTypeRecursed sub = adtypep->subDTypep()->cTypeRecurse(compound);
-        info.m_type = sub.m_type;
-        info.m_dims = "[" + cvtToStr(adtypep->declRange().elements()) + "]" + sub.m_dims;
+        info.m_type = "VlUnpacked<" + sub.m_type;
+        info.m_type += ", " + cvtToStr(adtypep->declRange().elements());
+        info.m_type += ">";
     } else if (const AstBasicDType* bdtypep = dtypep->basicp()) {
         // We don't print msb()/lsb() as multidim packed would require recursion,
         // and may confuse users as C++ data is stored always with bit 0 used
@@ -610,8 +672,6 @@ AstNodeDType::CTypeRecursed AstNodeDType::cTypeRecurse(bool compound) const {
             info.m_type = "const VerilatedScope*";
         } else if (bdtypep->keyword() == AstBasicDTypeKwd::DOUBLE) {
             info.m_type = "double";
-        } else if (bdtypep->keyword() == AstBasicDTypeKwd::FLOAT) {
-            info.m_type = "float";
         } else if (bdtypep->keyword() == AstBasicDTypeKwd::STRING) {
             info.m_type = "std::string";
         } else if (dtypep->widthMin() <= 8) {  // Handle unpacked arrays; not bdtypep->width
@@ -623,12 +683,7 @@ AstNodeDType::CTypeRecursed AstNodeDType::cTypeRecurse(bool compound) const {
         } else if (dtypep->isQuad()) {
             info.m_type = "QData" + bitvec;
         } else if (dtypep->isWide()) {
-            if (compound) {
-                info.m_type = "VlWide<" + cvtToStr(dtypep->widthWords()) + "> ";
-            } else {
-                info.m_type += "WData" + bitvec;  // []'s added later
-                info.m_dims = "[" + cvtToStr(dtypep->widthWords()) + "]";
-            }
+            info.m_type = "VlWide<" + cvtToStr(dtypep->widthWords()) + ">" + bitvec;
         }
     } else {
         v3fatalSrc("Unknown data type in var type emitter: " << dtypep->prettyName());
@@ -639,45 +694,6 @@ AstNodeDType::CTypeRecursed AstNodeDType::cTypeRecurse(bool compound) const {
     return info;
 }
 
-AstNodeDType* AstNodeDType::dtypeDimensionp(int dimension) {
-    // dimension passed from AstArraySel::dimension
-    // Dimension 0 means the VAR itself, 1 is the closest SEL to the AstVar,
-    // which is the lowest in the dtype list.
-    //     ref order:   a[1][2][3][4]
-    //     Created as:  reg [4] a [1][2][3];
-    //        *or*      reg a [1][2][3][4];
-    //                  // The bit select is optional; used only if "leftover" []'s
-    //     SEL:         SEL4(SEL3(SEL2(SEL1(VARREF0 a))))
-    //     DECL:        VAR a (ARRAYSEL0 (ARRAYSEL1 (ARRAYSEL2 (DT RANGE3))))
-    //        *or*      VAR a (ARRAYSEL0 (ARRAYSEL1 (ARRAYSEL2 (ARRAYSEL3 (DT))))
-    //     SEL1 needs to select from entire variable which is a pointer to ARRAYSEL0
-    // TODO this function should be removed in favor of recursing the dtype(),
-    // as that allows for more complicated data types.
-    int dim = 0;
-    for (AstNodeDType* dtypep = this; dtypep;) {
-        dtypep = dtypep->skipRefp();  // Skip AstRefDType/AstTypedef, or return same node
-        if (AstNodeArrayDType* adtypep = VN_CAST(dtypep, NodeArrayDType)) {
-            if ((dim++) == dimension) return dtypep;
-            dtypep = adtypep->subDTypep();
-            continue;
-        } else if (AstBasicDType* adtypep = VN_CAST(dtypep, BasicDType)) {
-            // AstBasicDType - nothing below, return null
-            if (adtypep->isRanged()) {
-                if ((dim++) == dimension) return adtypep;
-            }
-            return nullptr;
-        } else if (AstNodeUOrStructDType* adtypep = VN_CAST(dtypep, NodeUOrStructDType)) {
-            if (adtypep->packed()) {
-                if ((dim++) == dimension) return adtypep;
-            }
-            return nullptr;
-        }
-        // Node no ->next in loop; use continue where necessary
-        break;
-    }
-    return nullptr;
-}
-
 uint32_t AstNodeDType::arrayUnpackedElements() {
     uint32_t entries = 1;
     for (AstNodeDType* dtypep = this; dtypep;) {
@@ -718,7 +734,7 @@ std::pair<uint32_t, uint32_t> AstNodeDType::dimensions(bool includeBasic) {
         }
         break;
     }
-    return make_pair(packed, unpacked);
+    return std::make_pair(packed, unpacked);
 }
 
 int AstNodeDType::widthPow2() const {
@@ -731,7 +747,7 @@ int AstNodeDType::widthPow2() const {
 }
 
 /// What is the base variable (or const) this dereferences?
-AstNode* AstArraySel::baseFromp(AstNode* nodep) {
+AstNode* AstArraySel::baseFromp(AstNode* nodep, bool overMembers) {
     // Else AstArraySel etc; search for the base
     while (nodep) {
         if (VN_IS(nodep, ArraySel)) {
@@ -740,6 +756,9 @@ AstNode* AstArraySel::baseFromp(AstNode* nodep) {
         } else if (VN_IS(nodep, Sel)) {
             nodep = VN_CAST(nodep, Sel)->fromp();
             continue;
+        } else if (overMembers && VN_IS(nodep, MemberSel)) {
+            nodep = VN_CAST(nodep, MemberSel)->fromp();
+            continue;
         }
         // AstNodeSelPre stashes the associated variable under an ATTROF
         // of AstAttrType::VAR_BASE/MEMBER_BASE so it isn't constified
@@ -750,7 +769,7 @@ AstNode* AstArraySel::baseFromp(AstNode* nodep) {
             if (VN_CAST(nodep, NodePreSel)->attrp()) {
                 nodep = VN_CAST(nodep, NodePreSel)->attrp();
             } else {
-                nodep = VN_CAST(nodep, NodePreSel)->lhsp();
+                nodep = VN_CAST(nodep, NodePreSel)->fromp();
             }
             continue;
         } else {
@@ -931,7 +950,7 @@ AstBasicDType* AstTypeTable::findInsertSameDType(AstBasicDType* nodep) {
     DetailedMap& mapr = m_detailedMap;
     const auto it = mapr.find(key);
     if (it != mapr.end()) return it->second;
-    mapr.insert(make_pair(key, nodep));
+    mapr.emplace(key, nodep);
     nodep->generic(true);
     // No addTypesp; the upper function that called new() is responsible for adding
     return nodep;
@@ -1032,7 +1051,7 @@ void AstNode::dump(std::ostream& str) const {
         } else {
             str << " @dt=" << nodeAddr(dtypep()) << "@";
         }
-        if (AstNodeDType* dtp = dtypep()) { dtp->dumpSmall(str); }
+        if (AstNodeDType* dtp = dtypep()) dtp->dumpSmall(str);
     } else {  // V3Broken will throw an error
         if (dtypep()) str << " %Error-dtype-exp=null,got=" << nodeAddr(dtypep());
     }
@@ -1104,7 +1123,7 @@ void AstClass::insertCache(AstNode* nodep) {
         if (m_members.find(nodep->name()) != m_members.end()) {
             nodep->v3error("Duplicate declaration of member name: " << nodep->prettyNameQ());
         } else {
-            m_members.insert(make_pair(nodep->name(), nodep));
+            m_members.emplace(nodep->name(), nodep);
         }
     }
 }
@@ -1112,6 +1131,13 @@ void AstClass::repairCache() {
     clearCache();
     for (AstNode* itemp = membersp(); itemp; itemp = itemp->nextp()) { insertCache(itemp); }
 }
+bool AstClass::isClassExtendedFrom(const AstClass* refClassp, const AstClass* baseClassp) {
+    // TAIL RECURSIVE
+    if (!refClassp || !baseClassp) return false;
+    if (refClassp == baseClassp) return true;
+    if (!refClassp->extendsp()) return false;
+    return isClassExtendedFrom(refClassp->extendsp()->classp(), baseClassp);
+}
 void AstClass::dump(std::ostream& str) const {
     this->AstNode::dump(str);
     if (isExtended()) str << " [EXT]";
@@ -1119,6 +1145,9 @@ void AstClass::dump(std::ostream& str) const {
 }
 AstClass* AstClassExtends::classp() const {
     AstClassRefDType* refp = VN_CAST(dtypep(), ClassRefDType);
+    if (VL_UNLIKELY(!refp)) {  // LinkDot uses this for 'super.'
+        refp = VN_CAST(childDTypep(), ClassRefDType);
+    }
     UASSERT_OBJ(refp, this, "class extends non-ref");
     return refp->classp();
 }
@@ -1154,9 +1183,9 @@ void AstEnumItemRef::dump(std::ostream& str) const {
 }
 void AstIfaceRefDType::dump(std::ostream& str) const {
     this->AstNode::dump(str);
-    if (cellName() != "") { str << " cell=" << cellName(); }
-    if (ifaceName() != "") { str << " if=" << ifaceName(); }
-    if (modportName() != "") { str << " mp=" << modportName(); }
+    if (cellName() != "") str << " cell=" << cellName();
+    if (ifaceName() != "") str << " if=" << ifaceName();
+    if (modportName() != "") str << " mp=" << modportName();
     if (cellp()) {
         str << " -> ";
         cellp()->dump(str);
@@ -1312,21 +1341,23 @@ void AstNodeDType::dump(std::ostream& str) const {
 void AstNodeDType::dumpSmall(std::ostream& str) const {
     str << "(" << (generic() ? "G/" : "") << ((isSigned() && !isDouble()) ? "s" : "")
         << (isNosign() ? "n" : "") << (isDouble() ? "d" : "") << (isString() ? "str" : "");
-    if (!isDouble() && !isString()) { str << "w" << (widthSized() ? "" : "u") << width(); }
+    if (!isDouble() && !isString()) str << "w" << (widthSized() ? "" : "u") << width();
     if (!widthSized()) str << "/" << widthMin();
     str << ")";
 }
 void AstNodeArrayDType::dumpSmall(std::ostream& str) const {
     this->AstNodeDType::dumpSmall(str);
-    if (VN_IS(this, PackArrayDType)) {
-        str << "p";
+    if (auto* adtypep = VN_CAST_CONST(this, UnpackArrayDType)) {
+        // uc = packed compound object, u = unpacked POD
+        str << (adtypep->isCompound() ? "uc" : "u");
     } else {
-        str << "u";
+        str << "p";
     }
     str << declRange();
 }
 void AstNodeArrayDType::dump(std::ostream& str) const {
     this->AstNodeDType::dump(str);
+    if (isCompound()) str << " [COMPOUND]";
     str << " " << declRange();
 }
 string AstPackArrayDType::prettyDTypeName() const {
@@ -1347,6 +1378,18 @@ string AstUnpackArrayDType::prettyDTypeName() const {
     os << subp->prettyDTypeName() << "$" << ranges;
     return os.str();
 }
+std::vector<AstUnpackArrayDType*> AstUnpackArrayDType::unpackDimensions() {
+    std::vector<AstUnpackArrayDType*> dims;
+    for (AstUnpackArrayDType* unpackp = this; unpackp;) {
+        dims.push_back(unpackp);
+        if (AstNodeDType* subp = unpackp->subDTypep()) {
+            unpackp = VN_CAST(subp, UnpackArrayDType);
+        } else {
+            unpackp = nullptr;
+        }
+    }
+    return dims;
+}
 void AstNetlist::dump(std::ostream& str) const {
     this->AstNode::dump(str);
     str << " [" << timeunit() << "/" << timeprecision() << "]";
@@ -1397,7 +1440,7 @@ void AstTypeTable::dump(std::ostream& str) const {
     this->AstNode::dump(str);
     for (int i = 0; i < static_cast<int>(AstBasicDTypeKwd::_ENUM_MAX); ++i) {
         if (AstBasicDType* subnodep = m_basicps[i]) {
-            str << endl;  // Newline from caller, so newline first
+            str << '\n';  // Newline from caller, so newline first
             str << "\t\t" << std::setw(8) << AstBasicDTypeKwd(i).ascii();
             str << "  -> ";
             subnodep->dump(str);
@@ -1407,7 +1450,7 @@ void AstTypeTable::dump(std::ostream& str) const {
         const DetailedMap& mapr = m_detailedMap;
         for (const auto& itr : mapr) {
             AstBasicDType* dtypep = itr.second;
-            str << endl;  // Newline from caller, so newline first
+            str << '\n';  // Newline from caller, so newline first
             str << "\t\tdetailed  ->  ";
             dtypep->dump(str);
         }
@@ -1453,11 +1496,13 @@ void AstVarScope::dump(std::ostream& str) const {
         str << " ->UNLINKED";
     }
 }
-void AstNodeVarRef::dump(std::ostream& str) const { this->AstNodeMath::dump(str); }
+void AstNodeVarRef::dump(std::ostream& str) const {
+    this->AstNodeMath::dump(str);
+    if (classOrPackagep()) str << " pkg=" << nodeAddr(classOrPackagep());
+    str << " " << access().arrow() << " ";
+}
 void AstVarXRef::dump(std::ostream& str) const {
     this->AstNodeVarRef::dump(str);
-    if (packagep()) str << " pkg=" << nodeAddr(packagep());
-    str << " " << access().arrow() << " ";
     str << ".=" << dotted() << " ";
     if (inlinedDots() != "") str << " inline.=" << inlinedDots() << " - ";
     if (varScopep()) {
@@ -1470,8 +1515,6 @@ void AstVarXRef::dump(std::ostream& str) const {
 }
 void AstVarRef::dump(std::ostream& str) const {
     this->AstNodeVarRef::dump(str);
-    if (packagep()) str << " pkg=" << nodeAddr(packagep());
-    str << " " << access().arrow() << " ";
     if (varScopep()) {
         varScopep()->dump(str);
     } else if (varp()) {
@@ -1490,6 +1533,7 @@ void AstVar::dump(std::ostream& str) const {
     if (isPulldown()) str << " [PULLDOWN]";
     if (isUsedClock()) str << " [CLK]";
     if (isSigPublic()) str << " [P]";
+    if (isLatched()) str << " [LATCHED]";
     if (isUsedLoopIdx()) str << " [LOOP]";
     if (attrClockEn()) str << " [aCLKEN]";
     if (attrIsolateAssign()) str << " [aISO]";
@@ -1524,10 +1568,10 @@ void AstParseRef::dump(std::ostream& str) const {
 }
 void AstClassOrPackageRef::dump(std::ostream& str) const {
     this->AstNode::dump(str);
-    if (classOrPackagep()) { str << " cpkg=" << nodeAddr(classOrPackagep()); }
+    if (classOrPackageNodep()) str << " cpkg=" << nodeAddr(classOrPackageNodep());
     str << " -> ";
-    if (classOrPackagep()) {
-        classOrPackagep()->dump(str);
+    if (classOrPackageNodep()) {
+        classOrPackageNodep()->dump(str);
     } else {
         str << "UNLINKED";
     }
@@ -1547,9 +1591,9 @@ void AstActive::dump(std::ostream& str) const {
 }
 void AstNodeFTaskRef::dump(std::ostream& str) const {
     this->AstNodeStmt::dump(str);
-    if (packagep()) { str << " pkg=" << nodeAddr(packagep()); }
+    if (classOrPackagep()) str << " pkg=" << nodeAddr(classOrPackagep());
     str << " -> ";
-    if (dotted() != "") { str << ".=" << dotted() << " "; }
+    if (dotted() != "") str << ".=" << dotted() << " ";
     if (taskp()) {
         taskp()->dump(str);
     } else {
@@ -1585,7 +1629,7 @@ void AstCoverDecl::dump(std::ostream& str) const {
         str << " -> ";
         this->dataDeclNullp()->dump(str);
     } else {
-        if (binNum()) { str << " bin" << std::dec << binNum(); }
+        if (binNum()) str << " bin" << std::dec << binNum();
     }
 }
 void AstCoverInc::dump(std::ostream& str) const {
