@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -35,20 +35,17 @@
 //######################################################################
 // Branch state, as a visitor of each AstNode
 
-class BranchVisitor : public AstNVisitor {
+class BranchVisitor final : public AstNVisitor {
 private:
     // NODE STATE
     // Entire netlist:
     //  AstFTask::user1()       -> int.  Number of references
     AstUser1InUse m_inuser1;
 
-    // TYPES
-    typedef std::vector<AstCFunc*> CFuncVec;
-
     // STATE
     int m_likely;  // Excuses for branch likely taken
     int m_unlikely;  // Excuses for branch likely not taken
-    CFuncVec m_cfuncsp;  // List of all tasks
+    std::vector<AstCFunc*> m_cfuncsp;  // List of all tasks
 
     // METHODS
     VL_DEBUG_FUNC;  // Declare debug()
@@ -118,7 +115,7 @@ public:
         iterateChildren(nodep);
         calc_tasks();
     }
-    virtual ~BranchVisitor() override {}
+    virtual ~BranchVisitor() override = default;
 };
 
 //######################################################################
