@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -36,12 +36,12 @@
 
 //######################################################################
 
-class BrokenTable : public AstNVisitor {
+class BrokenTable VL_NOT_FINAL : public AstNVisitor {
     // Table of brokenExists node pointers
 private:
     // MEMBERS
     //   For each node, we keep if it exists or not.
-    typedef std::unordered_map<const AstNode*, int> NodeMap;  // Performance matters (when --debug)
+    using NodeMap = std::unordered_map<const AstNode*, int>;  // Performance matters (when --debug)
     static NodeMap s_nodes;  // Set of all nodes that exist
     // BITMASK
     enum { FLAG_ALLOCATED = 0x01 };  // new() and not delete()ed
@@ -73,7 +73,7 @@ public:
                     "Newing AstNode object that is already allocated");
         if (iter == s_nodes.end()) {
             int flags = FLAG_ALLOCATED;  // This int needed to appease GCC 4.1.2
-            s_nodes.insert(make_pair(nodep, flags));
+            s_nodes.emplace(nodep, flags);
         }
     }
     static void setUnder(const AstNode* nodep, bool flag) {
@@ -105,7 +105,7 @@ public:
         }
         int or_flags = FLAG_IN_TREE | (linkable ? FLAG_LINKABLE : 0);
         if (iter == s_nodes.end()) {
-            s_nodes.insert(make_pair(nodep, or_flags));
+            s_nodes.emplace(nodep, or_flags);
         } else {
             iter->second |= or_flags;
         }
@@ -190,8 +190,8 @@ public:
     }
 
     // CONSTRUCTORS
-    BrokenTable() {}
-    virtual ~BrokenTable() override {}
+    BrokenTable() = default;
+    virtual ~BrokenTable() override = default;
 };
 
 BrokenTable::NodeMap BrokenTable::s_nodes;
@@ -211,7 +211,7 @@ bool AstNode::brokeExistsBelow() const {
 
 //######################################################################
 
-class BrokenMarkVisitor : public AstNVisitor {
+class BrokenMarkVisitor final : public AstNVisitor {
     // Mark every node in the tree
 private:
     // NODE STATE
@@ -231,13 +231,15 @@ private:
 public:
     // CONSTRUCTORS
     explicit BrokenMarkVisitor(AstNetlist* nodep) { iterate(nodep); }
-    virtual ~BrokenMarkVisitor() override {}
+    virtual ~BrokenMarkVisitor() override = default;
 };
 
 //######################################################################
 // Broken state, as a visitor of each AstNode
 
-class BrokenCheckVisitor : public AstNVisitor {
+class BrokenCheckVisitor final : public AstNVisitor {
+    bool m_inScope = false;  // Under AstScope
+
 private:
     static void checkWidthMin(const AstNode* nodep) {
         UASSERT_OBJ(nodep->width() == nodep->widthMin()
@@ -278,6 +280,22 @@ private:
                       && !VN_CAST(nodep->lhsp(), NodeVarRef)->access().isWriteOrRW()),
                     nodep, "Assignment LHS is not an lvalue");
     }
+    virtual void visit(AstScope* nodep) override {
+        VL_RESTORER(m_inScope);
+        {
+            m_inScope = true;
+            processAndIterate(nodep);
+        }
+    }
+    virtual void visit(AstNodeVarRef* nodep) override {
+        processAndIterate(nodep);
+        // m_inScope because some Vars have initial variable references without scopes
+        // This might false fire with some debug flags, as not certain we don't have temporary
+        // clear varScopep's during some an infrequent dump just before we re-LinkDot.
+        UASSERT_OBJ(
+            !(v3Global.assertScoped() && m_inScope && nodep->varp() && !nodep->varScopep()), nodep,
+            "VarRef missing VarScope pointer");
+    }
     virtual void visit(AstNode* nodep) override {
         // Process not just iterate
         processAndIterate(nodep);
@@ -286,7 +304,7 @@ private:
 public:
     // CONSTRUCTORS
     explicit BrokenCheckVisitor(AstNetlist* nodep) { iterate(nodep); }
-    virtual ~BrokenCheckVisitor() override {}
+    virtual ~BrokenCheckVisitor() override = default;
 };
 
 //######################################################################
@@ -311,3 +329,13 @@ void V3Broken::brokenAll(AstNetlist* nodep) {
 void V3Broken::addNewed(AstNode* nodep) { BrokenTable::addNewed(nodep); }
 void V3Broken::deleted(AstNode* nodep) { BrokenTable::deleted(nodep); }
 bool V3Broken::isAllocated(AstNode* nodep) { return BrokenTable::isAllocated(nodep); }
+void V3Broken::selfTest() {
+    // Warmup addNewed and deleted for coverage, as otherwise only with VL_LEAK_CHECKS
+    FileLine* fl = new FileLine(FileLine::commandLineFilename());
+    auto* newp = new AstBegin(fl, "[EditWrapper]", nullptr);
+#ifndef VL_LEAK_CHECKS
+    addNewed(newp);
+    deleted(newp);
+#endif
+    VL_DO_DANGLING(delete newp, newp);
+}
