@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -34,7 +34,7 @@
 #include <algorithm>
 #include <map>
 
-class V3CCtorsVisitor {
+class V3CCtorsVisitor final {
 private:
     string m_basename;
     string m_argsp;
@@ -42,8 +42,8 @@ private:
     AstNodeModule* m_modp;  // Current module
     AstCFunc* m_tlFuncp;  // Top level function being built
     AstCFunc* m_funcp;  // Current function
-    int m_numStmts;  // Number of statements output
-    int m_funcNum;  // Function number being built
+    int m_numStmts = 0;  // Number of statements output
+    int m_funcNum = 0;  // Function number being built
 
 public:
     void add(AstNode* nodep) {
@@ -76,18 +76,16 @@ public:
         m_argsp = argsp;
         m_callargsp = callargsp;
         m_modp = nodep;
-        m_numStmts = 0;
-        m_funcNum = 0;
         m_tlFuncp = new AstCFunc(nodep->fileline(), basename, nullptr, "void");
         m_tlFuncp->declPrivate(true);
         m_tlFuncp->isStatic(false);
         m_tlFuncp->slow(!VN_IS(m_modp, Class));  // Only classes construct on fast path
         m_tlFuncp->argTypes(m_argsp);
-        if (stmt != "") { m_tlFuncp->addStmtsp(new AstCStmt(nodep->fileline(), stmt)); }
+        if (stmt != "") m_tlFuncp->addStmtsp(new AstCStmt(nodep->fileline(), stmt));
         m_funcp = m_tlFuncp;
         m_modp->addStmtp(m_tlFuncp);
     }
-    ~V3CCtorsVisitor() {}
+    ~V3CCtorsVisitor() = default;
 
 private:
     VL_UNCOPYABLE(V3CCtorsVisitor);
