@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2004-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2004-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -49,7 +49,7 @@
 //######################################################################
 // Cast state, as a visitor of each AstNode
 
-class CastVisitor : public AstNVisitor {
+class CastVisitor final : public AstNVisitor {
 private:
     // NODE STATE
     // Entire netlist:
@@ -193,7 +193,7 @@ private:
 public:
     // CONSTRUCTORS
     explicit CastVisitor(AstNetlist* nodep) { iterate(nodep); }
-    virtual ~CastVisitor() override {}
+    virtual ~CastVisitor() override = default;
 };
 
 //######################################################################
