@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -29,7 +29,7 @@
 
 //######################################################################
 
-class ClassVisitor : public AstNVisitor {
+class ClassVisitor final : public AstNVisitor {
 private:
     // MEMBERS
     AstUser1InUse m_inuser1;
@@ -37,8 +37,7 @@ private:
     AstScope* m_classScopep = nullptr;  // Package moving scopes into
     AstScope* m_packageScopep = nullptr;  // Class package scope
     AstNodeFTask* m_ftaskp = nullptr;  // Current task
-    typedef std::vector<std::pair<AstNode*, AstScope*>> MoveVector;
-    MoveVector m_moves;
+    std::vector<std::pair<AstNode*, AstScope*>> m_moves;
 
     // NODE STATE
     //  AstClass::user1()       -> bool.  True if iterated already
@@ -56,7 +55,7 @@ private:
         // Note origName is the same as the class origName so errors look correct
         AstClassPackage* packagep = new AstClassPackage(nodep->fileline(), nodep->origName());
         packagep->name(nodep->name() + "__Vclpkg");
-        nodep->packagep(packagep);
+        nodep->classOrPackagep(packagep);
         packagep->classp(nodep);
         v3Global.rootp()->addModulep(packagep);
         // Add package to hierarchy
@@ -87,7 +86,8 @@ private:
             iterateChildren(nodep);
         }
     }
-    virtual void visit(AstPackage* nodep) override {
+    virtual void visit(AstNodeModule* nodep) override {
+        // Visit for NodeModules that are not AstClass (AstClass is-a AstNodeModule)
         VL_RESTORER(m_prefix);
         {
             m_prefix = nodep->name() + "__03a__03a";  // ::
@@ -99,8 +99,8 @@ private:
         iterateChildren(nodep);
         // Don't move now, or wouldn't keep interating the class
         // TODO move class statics too
-        if (m_ftaskp && m_ftaskp->lifetime().isStatic()) {
-            m_moves.push_back(make_pair(nodep, m_packageScopep));
+        if (m_packageScopep && m_ftaskp && m_ftaskp->lifetime().isStatic()) {
+            m_moves.push_back(std::make_pair(nodep, m_packageScopep));
         }
     }
 
@@ -114,8 +114,8 @@ private:
         {
             m_ftaskp = nodep;
             iterateChildren(nodep);
-            if (nodep->lifetime().isStatic()) {
-                m_moves.push_back(make_pair(nodep, m_packageScopep));
+            if (m_packageScopep && nodep->lifetime().isStatic()) {
+                m_moves.push_back(std::make_pair(nodep, m_packageScopep));
             }
         }
     }
@@ -125,7 +125,7 @@ private:
         // Don't move now, or wouldn't keep interating the class
         // TODO move function statics only
         // if (m_classScopep) {
-        //    m_moves.push_back(make_pair(nodep, m_classScopep));
+        //    m_moves.push_back(std::make_pair(nodep, m_classScopep));
         //}
     }
 
