@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -40,7 +40,7 @@
 //######################################################################
 // Clock state, as a visitor of each AstNode
 
-class ClockVisitor : public AstNVisitor {
+class ClockVisitor final : public AstNVisitor {
 private:
     // NODE STATE
     // Cleared each Module:
@@ -101,6 +101,7 @@ private:
         // BOTHEDGE:  var ^ var_last
         // HIGHEDGE:  var
         // LOWEDGE:  ~var
+        // ANYEDGE:   var ^ var_last
         AstNode* newp = nullptr;
         if (nodep->edgeType() == VEdgeType::ET_ILLEGAL) {
             nodep->v3warn(E_UNSUPPORTED,
@@ -293,23 +294,26 @@ private:
         }
         m_scopep = nullptr;
     }
-    virtual void visit(AstAlways* nodep) override {
-        AstNode* cmtp = new AstComment(nodep->fileline(), nodep->typeName(), true);
-        nodep->replaceWith(cmtp);
+    virtual void visit(AstNodeProcedure* nodep) override {
         if (AstNode* stmtsp = nodep->bodysp()) {
             stmtsp->unlinkFrBackWithNext();
-            cmtp->addNextHere(stmtsp);
+            nodep->addNextHere(stmtsp);
         }
-        VL_DO_DANGLING(nodep->deleteTree(), nodep);
+        VL_DO_DANGLING(nodep->unlinkFrBack()->deleteTree(), nodep);
     }
     virtual void visit(AstAlwaysPost* nodep) override {
-        AstNode* cmtp = new AstComment(nodep->fileline(), nodep->typeName(), true);
-        nodep->replaceWith(cmtp);
         if (AstNode* stmtsp = nodep->bodysp()) {
             stmtsp->unlinkFrBackWithNext();
-            cmtp->addNextHere(stmtsp);
+            nodep->addNextHere(stmtsp);
         }
-        VL_DO_DANGLING(nodep->deleteTree(), nodep);
+        VL_DO_DANGLING(nodep->unlinkFrBack()->deleteTree(), nodep);
+    }
+    virtual void visit(AstAlwaysPostponed* nodep) override {
+        if (AstNode* stmtsp = nodep->bodysp()) {
+            stmtsp->unlinkFrBackWithNext();
+            nodep->addNextHere(stmtsp);
+        }
+        VL_DO_DANGLING(nodep->unlinkFrBack()->deleteTree(), nodep);
     }
     virtual void visit(AstCoverToggle* nodep) override {
         // nodep->dumpTree(cout, "ct:");
@@ -328,15 +332,6 @@ private:
         nodep->replaceWith(newp);
         VL_DO_DANGLING(nodep->deleteTree(), nodep);
     }
-    virtual void visit(AstInitial* nodep) override {
-        AstNode* cmtp = new AstComment(nodep->fileline(), nodep->typeName(), true);
-        nodep->replaceWith(cmtp);
-        if (AstNode* stmtsp = nodep->bodysp()) {
-            stmtsp->unlinkFrBackWithNext();
-            cmtp->addNextHere(stmtsp);
-        }
-        VL_DO_DANGLING(nodep->deleteTree(), nodep);
-    }
     virtual void visit(AstCFunc* nodep) override {
         iterateChildren(nodep);
         // Link to global function
@@ -454,7 +449,7 @@ public:
         // easily without iterating through the tree.
         nodep->evalp(m_evalFuncp);
     }
-    virtual ~ClockVisitor() override {}
+    virtual ~ClockVisitor() override = default;
 };
 
 //######################################################################
