@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -51,23 +51,23 @@ constexpr int COMBINE_MIN_STATEMENTS = 50;  // Min # of statements to be worth m
 
 //######################################################################
 
-class CombBaseVisitor : public AstNVisitor {
+class CombBaseVisitor VL_NOT_FINAL : public AstNVisitor {
 protected:
     // STATE
 
     // METHODS
-    virtual ~CombBaseVisitor() override {}
+    virtual ~CombBaseVisitor() override = default;
     VL_DEBUG_FUNC;  // Declare debug()
 };
 
 //######################################################################
 // Combine replacement function
 
-class CombCallVisitor : CombBaseVisitor {
+class CombCallVisitor final : CombBaseVisitor {
     // Find all CCALLS of each CFUNC, so that we can later rename them
 private:
     // NODE STATE
-    typedef std::multimap<AstCFunc*, AstCCall*> CallMmap;
+    using CallMmap = std::multimap<AstCFunc*, AstCCall*>;
     CallMmap m_callMmap;  // Associative array of {function}{call}
     // METHODS
 public:
@@ -79,7 +79,7 @@ public:
             UINFO(4, "   Remove " << oldfuncp << endl);
         }
         // Note: m_callMmap modified in loop, so not using equal_range.
-        for (CallMmap::iterator it = m_callMmap.find(oldfuncp); it != m_callMmap.end();
+        for (auto it = m_callMmap.find(oldfuncp); it != m_callMmap.end();
              it = m_callMmap.find(oldfuncp)) {
             AstCCall* callp = it->second;
             if (!callp->user3()) {  // !already done
@@ -106,7 +106,7 @@ public:
         }
     }
     // METHODS
-    void addCall(AstCCall* nodep) { m_callMmap.insert(make_pair(nodep->funcp(), nodep)); }
+    void addCall(AstCCall* nodep) { m_callMmap.emplace(nodep->funcp(), nodep); }
     void deleteCall(AstCCall* nodep) {
         std::pair<CallMmap::iterator, CallMmap::iterator> eqrange
             = m_callMmap.equal_range(nodep->funcp());
@@ -131,15 +131,15 @@ private:
 
 public:
     // CONSTRUCTORS
-    CombCallVisitor() {}
-    virtual ~CombCallVisitor() override {}
+    CombCallVisitor() = default;
+    virtual ~CombCallVisitor() override = default;
     void main(AstNetlist* nodep) { iterate(nodep); }
 };
 
 //######################################################################
 // Combine marking function
 
-class CombMarkVisitor : CombBaseVisitor {
+class CombMarkVisitor final : CombBaseVisitor {
     // Mark all nodes under specified one.
 private:
     // OUTPUT:
@@ -153,13 +153,13 @@ private:
 public:
     // CONSTRUCTORS
     explicit CombMarkVisitor(AstNode* nodep) { iterate(nodep); }
-    virtual ~CombMarkVisitor() override {}
+    virtual ~CombMarkVisitor() override = default;
 };
 
 //######################################################################
 // Combine state, as a visitor of each AstNode
 
-class CombineVisitor : CombBaseVisitor {
+class CombineVisitor final : CombBaseVisitor {
 private:
     // NODE STATE
     // Entire netlist:
@@ -173,7 +173,7 @@ private:
     // AstUser4InUse     part of V3Hashed
 
     // STATE
-    typedef enum : uint8_t { STATE_IDLE, STATE_HASH, STATE_DUP } CombineState;
+    enum CombineState : uint8_t { STATE_IDLE, STATE_HASH, STATE_DUP };
     VDouble0 m_statCombs;  // Statistic tracking
     CombineState m_state = STATE_IDLE;  // Major state
     AstNodeModule* m_modp = nullptr;  // Current module
@@ -218,21 +218,27 @@ private:
         }
     }
     void walkDupFuncs() {
-        for (V3Hashed::iterator it = m_hashed.begin(); it != m_hashed.end(); ++it) {
-            V3Hash hashval = it->first;
-            AstNode* node1p = it->second;
-            if (!VN_IS(node1p, CFunc)) continue;
-            UASSERT_OBJ(!hashval.isIllegal(), node1p, "Illegal (unhashed) nodes");
-            for (V3Hashed::iterator eqit = it; eqit != m_hashed.end(); ++eqit) {
-                AstNode* node2p = eqit->second;
-                if (!(eqit->first == hashval)) break;
-                if (node1p == node2p) continue;  // Identical iterator
-                if (node1p->user3p() || node2p->user3p()) continue;  // Already merged
-                if (node1p->sameTree(node2p)) {  // walk of tree has same comparison
-                    // Replace AstCCall's that point here
-                    replaceFuncWFunc(VN_CAST(node2p, CFunc), VN_CAST(node1p, CFunc));
-                    // Replacement may promote a slow routine to fast path
-                    if (!VN_CAST(node2p, CFunc)->slow()) VN_CAST(node1p, CFunc)->slow(false);
+        // Do non-slow first as then favors naming functions based on fast name
+        for (int slow = 0; slow < 2; ++slow) {
+            for (V3Hashed::iterator it = m_hashed.begin(); it != m_hashed.end(); ++it) {
+                AstNode* node1p = it->second;
+                AstCFunc* cfunc1p = VN_CAST(node1p, CFunc);
+                if (!cfunc1p) continue;
+                // cppcheck-suppress compareBoolExpressionWithInt
+                if (cfunc1p->slow() != slow) continue;
+                V3Hash hashval = it->first;
+                UASSERT_OBJ(!hashval.isIllegal(), node1p, "Illegal (unhashed) nodes");
+                for (V3Hashed::iterator eqit = it; eqit != m_hashed.end(); ++eqit) {
+                    AstNode* node2p = eqit->second;
+                    if (!(eqit->first == hashval)) break;
+                    if (node1p == node2p) continue;  // Identical iterator
+                    if (node1p->user3p() || node2p->user3p()) continue;  // Already merged
+                    if (node1p->sameTree(node2p)) {  // walk of tree has same comparison
+                        // Replace AstCCall's that point here
+                        replaceFuncWFunc(VN_CAST(node2p, CFunc), cfunc1p);
+                        // Replacement may promote a slow routine to fast path
+                        if (!VN_CAST(node2p, CFunc)->slow()) cfunc1p->slow(false);
+                    }
                 }
             }
         }
