@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2010-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2010-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -33,13 +33,13 @@
 // cache of resolved entities. Entities stored in this container need an update
 // function that takes a reference of this type to join multiple entities into one.
 template <typename T> class V3ConfigWildcardResolver {
-    typedef std::map<const string, T> Map;
+    using Map = std::map<const std::string, T>;
 
     Map m_mapWildcard;  // Wildcard strings to entities
     Map m_mapResolved;  // Resolved strings to converged entities
 public:
-    V3ConfigWildcardResolver() {}
-    ~V3ConfigWildcardResolver() {}
+    V3ConfigWildcardResolver() = default;
+    ~V3ConfigWildcardResolver() = default;
 
     /// Update into maps from other
     void update(const V3ConfigWildcardResolver& other) {
@@ -61,7 +61,7 @@ public:
     T* resolve(const string& name) {
         // Lookup if it was resolved before, typically not
         auto it = m_mapResolved.find(name);
-        if (VL_UNLIKELY(it != m_mapResolved.end())) { return &it->second; }
+        if (VL_UNLIKELY(it != m_mapResolved.end())) return &it->second;
 
         T* newp = nullptr;
         // Cannot be resolved, create if matched
@@ -82,7 +82,7 @@ public:
 };
 
 // Only public_flat_rw has the sensitity tree
-class V3ConfigVarAttr {
+class V3ConfigVarAttr final {
 public:
     AstAttrType m_type;  // Type of attribute
     AstSenTree* m_sentreep;  // Sensitivity tree for public_flat_rw
@@ -92,7 +92,7 @@ public:
 };
 
 // Overload vector with the required update function and to apply all entries
-class V3ConfigVar : public std::vector<V3ConfigVarAttr> {
+class V3ConfigVar final : public std::vector<V3ConfigVarAttr> {
 public:
     // Update from other by copying all attributes
     void update(const V3ConfigVar& node) {
@@ -111,19 +111,19 @@ public:
     }
 };
 
-typedef V3ConfigWildcardResolver<V3ConfigVar> V3ConfigVarResolver;
+using V3ConfigVarResolver = V3ConfigWildcardResolver<V3ConfigVar>;
 
 //######################################################################
 // Function or task: Have variables and properties
 
-class V3ConfigFTask {
+class V3ConfigFTask final {
     V3ConfigVarResolver m_vars;  // Variables in function/task
     bool m_isolate = false;  // Isolate function return
     bool m_noinline = false;  // Don't inline function/task
     bool m_public = false;  // Public function/task
 
 public:
-    V3ConfigFTask() {}
+    V3ConfigFTask() = default;
     void update(const V3ConfigFTask& f) {
         // Don't overwrite true with false
         if (f.m_isolate) m_isolate = true;
@@ -148,24 +148,21 @@ public:
     }
 };
 
-typedef V3ConfigWildcardResolver<V3ConfigFTask> V3ConfigFTaskResolver;
+using V3ConfigFTaskResolver = V3ConfigWildcardResolver<V3ConfigFTask>;
 
 //######################################################################
 // Modules have tasks, variables, named blocks and properties
 
-class V3ConfigModule {
-    typedef std::unordered_set<string> StringSet;
-    typedef std::set<AstPragmaType> PragmaSet;
-
+class V3ConfigModule final {
     V3ConfigFTaskResolver m_tasks;  // Functions/tasks in module
     V3ConfigVarResolver m_vars;  // Variables in module
-    StringSet m_coverageOffBlocks;  // List of block names for coverage_off
-    PragmaSet m_modPragmas;  // List of Pragmas for modules
+    std::unordered_set<std::string> m_coverageOffBlocks;  // List of block names for coverage_off
+    std::set<AstPragmaType> m_modPragmas;  // List of Pragmas for modules
     bool m_inline = false;  // Whether to force the inline
     bool m_inlineValue = false;  // The inline value (on/off)
 
 public:
-    V3ConfigModule() {}
+    V3ConfigModule() = default;
 
     void update(const V3ConfigModule& m) {
         m_tasks.update(m.m_tasks);
@@ -175,8 +172,7 @@ public:
             m_inline = m.m_inline;
             m_inlineValue = m.m_inlineValue;
         }
-        for (PragmaSet::const_iterator it = m.m_modPragmas.begin(); it != m.m_modPragmas.end();
-             ++it) {
+        for (auto it = m.m_modPragmas.cbegin(); it != m.m_modPragmas.cend(); ++it) {
             m_modPragmas.insert(*it);
         }
     }
@@ -198,7 +194,7 @@ public:
             AstNode* nodep = new AstPragma(modp->fileline(), type);
             modp->addStmtp(nodep);
         }
-        for (PragmaSet::const_iterator it = m_modPragmas.begin(); it != m_modPragmas.end(); ++it) {
+        for (auto it = m_modPragmas.cbegin(); it != m_modPragmas.cend(); ++it) {
             AstNode* nodep = new AstPragma(modp->fileline(), *it);
             modp->addStmtp(nodep);
         }
@@ -216,7 +212,7 @@ public:
     }
 };
 
-typedef V3ConfigWildcardResolver<V3ConfigModule> V3ConfigModuleResolver;
+using V3ConfigModuleResolver = V3ConfigWildcardResolver<V3ConfigModule>;
 
 //######################################################################
 // Files have:
@@ -224,7 +220,7 @@ typedef V3ConfigWildcardResolver<V3ConfigModule> V3ConfigModuleResolver;
 //  - Line attributes: Attributes attached to lines
 
 // lint/coverage/tracing on/off
-class V3ConfigIgnoresLine {
+class V3ConfigIgnoresLine final {
 public:
     int m_lineno;  // Line number to make change at
     V3ErrorCode m_code;  // Error code
@@ -233,8 +229,8 @@ public:
         : m_lineno{lineno}
         , m_code{code}
         , m_on{on} {}
-    ~V3ConfigIgnoresLine() {}
-    inline bool operator<(const V3ConfigIgnoresLine& rh) const {
+    ~V3ConfigIgnoresLine() = default;
+    bool operator<(const V3ConfigIgnoresLine& rh) const {
         if (m_lineno < rh.m_lineno) return true;
         if (m_lineno > rh.m_lineno) return false;
         if (m_code < rh.m_code) return true;
@@ -250,14 +246,14 @@ std::ostream& operator<<(std::ostream& os, const V3ConfigIgnoresLine& rhs) {
 
 // Some attributes are attached to entities of the occur on a fileline
 // and multiple attributes can be attached to a line
-typedef std::bitset<AstPragmaType::ENUM_SIZE> V3ConfigLineAttribute;
+using V3ConfigLineAttribute = std::bitset<AstPragmaType::ENUM_SIZE>;
 
 // File entity
-class V3ConfigFile {
-    typedef std::map<int, V3ConfigLineAttribute> LineAttrMap;  // Map line->bitset of attributes
-    typedef std::multiset<V3ConfigIgnoresLine> IgnLines;  // list of {line,code,on}
-    typedef std::pair<V3ErrorCode, string> WaiverSetting;  // Waive code if string matches
-    typedef std::vector<WaiverSetting> Waivers;  // List of {code,wildcard string}
+class V3ConfigFile final {
+    using LineAttrMap = std::map<int, V3ConfigLineAttribute>;  // Map line->bitset of attributes
+    using IgnLines = std::multiset<V3ConfigIgnoresLine>;  // list of {line,code,on}
+    using WaiverSetting = std::pair<V3ErrorCode, std::string>;  // Waive code if string matches
+    using Waivers = std::vector<WaiverSetting>;  // List of {code,wildcard string}
 
     LineAttrMap m_lineAttrs;  // Atributes to line mapping
     IgnLines m_ignLines;  // Ignore line settings
@@ -296,7 +292,7 @@ public:
         m_lastIgnore.it = m_ignLines.begin();
     }
     void addWaiver(V3ErrorCode code, const string& match) {
-        m_waivers.push_back(make_pair(code, match));
+        m_waivers.push_back(std::make_pair(code, match));
     }
 
     void applyBlock(AstNodeBlock* nodep) {
@@ -342,21 +338,21 @@ public:
     }
 };
 
-typedef V3ConfigWildcardResolver<V3ConfigFile> V3ConfigFileResolver;
+using V3ConfigFileResolver = V3ConfigWildcardResolver<V3ConfigFile>;
 
 //######################################################################
 // Resolve modules and files in the design
 
-class V3ConfigResolver {
+class V3ConfigResolver final {
     V3ConfigModuleResolver m_modules;  // Access to module names (with wildcards)
     V3ConfigFileResolver m_files;  // Access to file names (with wildcards)
 
     static V3ConfigResolver s_singleton;  // Singleton (not via local static, as that's slow)
-    V3ConfigResolver() {}
-    ~V3ConfigResolver() {}
+    V3ConfigResolver() = default;
+    ~V3ConfigResolver() = default;
 
 public:
-    inline static V3ConfigResolver& s() { return s_singleton; }
+    static V3ConfigResolver& s() { return s_singleton; }
 
     V3ConfigModuleResolver& modules() { return m_modules; }
     V3ConfigFileResolver& files() { return m_files; }
@@ -405,7 +401,7 @@ void V3Config::addInline(FileLine* fl, const string& module, const string& ftask
         V3ConfigResolver::s().modules().at(module).setInline(on);
     } else {
         if (!on) {
-            fl->v3error("no_inline not supported for tasks" << endl);
+            fl->v3error("no_inline not supported for tasks");
         } else {
             V3ConfigResolver::s().modules().at(module).ftasks().at(ftask).setNoInline(on);
         }
@@ -416,15 +412,14 @@ void V3Config::addVarAttr(FileLine* fl, const string& module, const string& ftas
                           const string& var, AstAttrType attr, AstSenTree* sensep) {
     // Semantics: sensep only if public_flat_rw
     if ((attr != AstAttrType::VAR_PUBLIC_FLAT_RW) && sensep) {
-        sensep->v3error("sensitivity not expected for attribute" << endl);
+        sensep->v3error("sensitivity not expected for attribute");
         return;
     }
     // Semantics: Most of the attributes operate on signals
     if (var.empty()) {
         if (attr == AstAttrType::VAR_ISOLATE_ASSIGNMENTS) {
             if (ftask.empty()) {
-                fl->v3error("isolate_assignments only applies to signals or functions/tasks"
-                            << endl);
+                fl->v3error("isolate_assignments only applies to signals or functions/tasks");
             } else {
                 V3ConfigResolver::s().modules().at(module).ftasks().at(ftask).setIsolate(true);
             }
@@ -437,7 +432,7 @@ void V3Config::addVarAttr(FileLine* fl, const string& module, const string& ftas
                 V3ConfigResolver::s().modules().at(module).ftasks().at(ftask).setPublic(true);
             }
         } else {
-            fl->v3error("missing -signal" << endl);
+            fl->v3error("missing -signal");
         }
     } else {
         V3ConfigModule& mod = V3ConfigResolver::s().modules().at(module);
