@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -37,12 +37,10 @@
 //######################################################################
 // Coverage state, as a visitor of each AstNode
 
-class CoverageVisitor : public AstNVisitor {
+class CoverageVisitor final : public AstNVisitor {
 private:
     // TYPES
-    typedef std::unordered_map<string, int> VarNameMap;
-    typedef std::set<int> LinenoSet;
-    typedef std::unordered_map<int, LinenoSet> HandleLines;
+    using LinenoSet = std::set<int>;
 
     struct ToggleEnt {
         string m_comment;  // Comment for coverage dump
@@ -52,7 +50,7 @@ private:
             : m_comment{comment}
             , m_varRefp{vp}
             , m_chgRefp{cp} {}
-        ~ToggleEnt() {}
+        ~ToggleEnt() = default;
         void cleanup() {
             VL_DO_CLEAR(m_varRefp->deleteTree(), m_varRefp = nullptr);
             VL_DO_CLEAR(m_chgRefp->deleteTree(), m_chgRefp = nullptr);
@@ -64,7 +62,7 @@ private:
         bool m_inModOff = false;  // In module with no coverage
         int m_handle = 0;  // Opaque handle for index into line tracking
         const AstNode* m_nodep = nullptr;  // Node establishing this state
-        CheckState() {}
+        CheckState() = default;
         bool lineCoverageOn(const AstNode* nodep) const {
             return m_on && !m_inModOff && nodep->fileline()->coverageOn()
                    && v3Global.opt.coverageLine();
@@ -81,9 +79,10 @@ private:
     CheckState m_state;  // State save-restored on each new coverage scope/block
     AstNodeModule* m_modp = nullptr;  // Current module to add statement to
     bool m_inToggleOff = false;  // In function/task etc
-    VarNameMap m_varnames;  // Uniquification of inserted variable names
+    std::unordered_map<std::string, int> m_varnames;  // Uniquification of inserted variable names
     string m_beginHier;  // AstBegin hier name for user coverage points
-    HandleLines m_handleLines;  // All line numbers for a given m_stateHandle
+    std::unordered_map<int, LinenoSet>
+        m_handleLines;  // All line numbers for a given m_stateHandle
 
     // METHODS
     VL_DEBUG_FUNC;  // Declare debug()
@@ -97,8 +96,9 @@ private:
             if (prettyName[0] == '_') return "Leading underscore";
             if (prettyName.find("._") != string::npos) return "Inlined leading underscore";
         }
-        if ((nodep->width() * nodep->dtypep()->arrayUnpackedElements()) > 256) {
-            return "Wide bus/array > 256 bits";
+        if ((nodep->width() * nodep->dtypep()->arrayUnpackedElements())
+            > static_cast<uint32_t>(v3Global.opt.coverageMaxWidth())) {
+            return "Wide bus/array > --coverage-max-width setting's bits";
         }
         // We allow this, though tracing doesn't
         // if (nodep->arrayp(1)) return "Unsupported: Multi-dimensional array";
@@ -142,7 +142,7 @@ private:
                       + cvtToStr(nodep->fileline()->lineno()) + "_" + type;
         const auto it = m_varnames.find(name);
         if (it == m_varnames.end()) {
-            m_varnames.insert(make_pair(name, 1));
+            m_varnames.emplace(name, 1);
         } else {
             int suffix = (it->second)++;
             name += "_" + cvtToStr(suffix);
@@ -210,6 +210,7 @@ private:
     // VISITORS - BOTH
     virtual void visit(AstNodeModule* nodep) override {
         AstNodeModule* origModp = m_modp;
+        VL_RESTORER(m_modp);
         VL_RESTORER(m_state);
         {
             createHandle(nodep);
@@ -223,7 +224,6 @@ private:
             }
             iterateChildren(nodep);
         }
-        m_modp = origModp;
     }
 
     virtual void visit(AstNodeProcedure* nodep) override { iterateProcedure(nodep); }
@@ -310,9 +310,9 @@ private:
                           const ToggleEnt& above, AstVar* varp, AstVar* chgVarp) {  // Constant
         if (const AstBasicDType* bdtypep = VN_CAST(dtypep, BasicDType)) {
             if (bdtypep->isRanged()) {
-                for (int index_docs = bdtypep->lsb(); index_docs < bdtypep->msb() + 1;
-                     index_docs++) {
-                    int index_code = index_docs - bdtypep->lsb();
+                for (int index_docs = bdtypep->lo(); index_docs < bdtypep->hi() + 1;
+                     ++index_docs) {
+                    int index_code = index_docs - bdtypep->lo();
                     ToggleEnt newent(above.m_comment + string("[") + cvtToStr(index_docs) + "]",
                                      new AstSel(varp->fileline(), above.m_varRefp->cloneTree(true),
                                                 index_code, 1),
@@ -325,8 +325,8 @@ private:
                 toggleVarBottom(above, varp);
             }
         } else if (AstUnpackArrayDType* adtypep = VN_CAST(dtypep, UnpackArrayDType)) {
-            for (int index_docs = adtypep->lsb(); index_docs <= adtypep->msb(); ++index_docs) {
-                int index_code = index_docs - adtypep->lsb();
+            for (int index_docs = adtypep->lo(); index_docs <= adtypep->hi(); ++index_docs) {
+                int index_code = index_docs - adtypep->lo();
                 ToggleEnt newent(above.m_comment + string("[") + cvtToStr(index_docs) + "]",
                                  new AstArraySel(varp->fileline(),
                                                  above.m_varRefp->cloneTree(true), index_code),
@@ -337,9 +337,9 @@ private:
                 newent.cleanup();
             }
         } else if (AstPackArrayDType* adtypep = VN_CAST(dtypep, PackArrayDType)) {
-            for (int index_docs = adtypep->lsb(); index_docs <= adtypep->msb(); ++index_docs) {
+            for (int index_docs = adtypep->lo(); index_docs <= adtypep->hi(); ++index_docs) {
                 AstNodeDType* subtypep = adtypep->subDTypep()->skipRefp();
-                int index_code = index_docs - adtypep->lsb();
+                int index_code = index_docs - adtypep->lo();
                 ToggleEnt newent(above.m_comment + string("[") + cvtToStr(index_docs) + "]",
                                  new AstSel(varp->fileline(), above.m_varRefp->cloneTree(true),
                                             index_code * subtypep->width(), subtypep->width()),
@@ -534,7 +534,7 @@ private:
 public:
     // CONSTRUCTORS
     explicit CoverageVisitor(AstNetlist* rootp) { iterateChildren(rootp); }
-    virtual ~CoverageVisitor() override {}
+    virtual ~CoverageVisitor() override = default;
 };
 
 //######################################################################
