@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -30,7 +30,7 @@
 //######################################################################
 // CoverageJoin state, as a visitor of each AstNode
 
-class CoverageJoinVisitor : public AstNVisitor {
+class CoverageJoinVisitor final : public AstNVisitor {
 private:
     // NODE STATE
     // V3Hashed
@@ -38,11 +38,8 @@ private:
 
     // AstUser4InUse     In V3Hashed
 
-    // TYPES
-    typedef std::vector<AstCoverToggle*> ToggleList;
-
     // STATE
-    ToggleList m_toggleps;  // List of of all AstCoverToggle's
+    std::vector<AstCoverToggle*> m_toggleps;  // List of of all AstCoverToggle's
 
     VDouble0 m_statToggleJoins;  // Statistic tracking
 
