@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -63,7 +63,7 @@
 //######################################################################
 // Delayed state, as a visitor of each AstNode
 
-class DelayedVisitor : public AstNVisitor {
+class DelayedVisitor final : public AstNVisitor {
 private:
     // NODE STATE
     // Cleared each module:
@@ -71,9 +71,10 @@ private:
     //  AstVarScope::user2p()   -> AstActive*.  Points to activity block of signal
     //                                       (valid when AstVarScope::user1p is valid)
     //  AstVarScope::user4p()   -> AstAlwaysPost*.  Post block for this variable
-    //  AstVarScope::user5()    -> VarUsage. Tracks delayed vs non-delayed usage
+    //  AstVarScope::user5p()   -> AstVarRef*. Last blocking or non-blocking reference
     //  AstVar::user2()         -> bool.  Set true if already made warning
     //  AstVarRef::user2()      -> bool.  Set true if already processed
+    //  AstVarRef::user5()      -> bool.  Set true if was blocking reference
     //  AstAlwaysPost::user2()  -> ActActive*.  Points to activity block of signal
     //                                      (valid when AstAlwaysPost::user4p is valid)
     //  AstAlwaysPost::user4()  -> AstIf*.  Last IF (__Vdlyvset__) created under this AlwaysPost
@@ -86,8 +87,6 @@ private:
     AstUser4InUse m_inuser4;
     AstUser5InUse m_inuser5;
 
-    enum VarUsage : uint8_t { VU_NONE = 0, VU_DLY = 1, VU_NONDLY = 2 };
-
     // STATE
     AstActive* m_activep = nullptr;  // Current activate
     AstCFunc* m_cfuncp = nullptr;  // Current public C Function
@@ -95,22 +94,36 @@ private:
     bool m_inDly = false;  // True in delayed assignments
     bool m_inLoop = false;  // True in for loops
     bool m_inInitial = false;  // True in initial blocks
-    typedef std::map<const std::pair<AstNodeModule*, string>, AstVar*> VarMap;
+    using VarMap = std::map<const std::pair<AstNodeModule*, std::string>, AstVar*>;
     VarMap m_modVarMap;  // Table of new var names created under module
     VDouble0 m_statSharedSet;  // Statistic tracking
-    typedef std::map<const AstVarScope*, int> ScopeVecMap;
-    ScopeVecMap m_scopeVecMap;  // Next var number for each scope
+    std::unordered_map<const AstVarScope*, int> m_scopeVecMap;  // Next var number for each scope
 
     // METHODS
     VL_DEBUG_FUNC;  // Declare debug()
 
-    void markVarUsage(AstVarScope* nodep, uint32_t flags) {
-        // UINFO(4, " MVU " << flags << " " << nodep << endl);
-        nodep->user5(nodep->user5() | flags);
-        if ((nodep->user5() & VU_DLY) && (nodep->user5() & VU_NONDLY)) {
-            nodep->v3warn(BLKANDNBLK,
-                          "Unsupported: Blocked and non-blocking assignments to same variable: "
-                              << nodep->varp()->prettyNameQ());
+    void markVarUsage(AstNodeVarRef* nodep, bool blocking) {
+        if (blocking) nodep->user5(true);
+        AstVarScope* vscp = nodep->varScopep();
+        // UINFO(4, " MVU " << blocking << " " << nodep << endl);
+        AstNode* lastrefp = vscp->user5p();
+        if (!lastrefp) {
+            vscp->user5p(nodep);
+        } else {
+            bool last_was_blocking = lastrefp->user5();
+            if (last_was_blocking != blocking) {
+                AstNode* nonblockingp = blocking ? nodep : lastrefp;
+                AstNode* blockingp = blocking ? lastrefp : nodep;
+                vscp->v3warn(
+                    BLKANDNBLK,
+                    "Unsupported: Blocked and non-blocking assignments to same variable: "
+                        << vscp->varp()->prettyNameQ() << '\n'
+                        << vscp->warnContextPrimary() << '\n'
+                        << blockingp->warnOther() << "... Location of blocking assignment\n"
+                        << blockingp->warnContextSecondary() << '\n'
+                        << nonblockingp->warnOther() << "... Location of nonblocking assignment\n"
+                        << nonblockingp->warnContextSecondary());
+            }
         }
     }
     AstVarScope* createVarSc(AstVarScope* oldvarscp, const string& name,
@@ -120,7 +133,7 @@ private:
         AstVar* varp;
         AstNodeModule* addmodp = oldvarscp->scopep()->modp();
         // We need a new AstVar, but only one for all scopes, to match the new AstVarScope
-        const auto it = m_modVarMap.find(make_pair(addmodp, name));
+        const auto it = m_modVarMap.find(std::make_pair(addmodp, name));
         if (it != m_modVarMap.end()) {
             // Created module's AstVar earlier under some other scope
             varp = it->second;
@@ -136,7 +149,7 @@ private:
                                   VFlagBitPacked(), width);
             }
             addmodp->addStmtp(varp);
-            m_modVarMap.insert(make_pair(make_pair(addmodp, name), varp));
+            m_modVarMap.emplace(std::make_pair(addmodp, name), varp);
         }
 
         AstVarScope* varscp = new AstVarScope(oldvarscp->fileline(), oldvarscp->scopep(), varp);
@@ -160,10 +173,10 @@ private:
                 varrefp->varp()->v3warn(
                     MULTIDRIVEN,
                     "Signal has multiple driving blocks with different clocking: "
-                        << varrefp->varp()->prettyNameQ() << endl
-                        << varrefp->warnOther() << "... Location of first driving block" << endl
-                        << varrefp->warnContextPrimary() << endl
-                        << oldactivep->warnOther() << "... Location of other driving block" << endl
+                        << varrefp->varp()->prettyNameQ() << '\n'
+                        << varrefp->warnOther() << "... Location of first driving block\n"
+                        << varrefp->warnContextPrimary() << '\n'
+                        << oldactivep->warnOther() << "... Location of other driving block\n"
                         << oldactivep->warnContextSecondary());
                 varrefp->varp()->user2(true);
             }
@@ -289,7 +302,7 @@ private:
                                         new AstConst(nodep->fileline(), 0));
             AstAssign* setassignp = new AstAssign(
                 nodep->fileline(), new AstVarRef(nodep->fileline(), setvscp, VAccess::WRITE),
-                new AstConst(nodep->fileline(), AstConst::LogicTrue()));
+                new AstConst(nodep->fileline(), AstConst::BitTrue()));
             nodep->addNextHere(setassignp);
         }
         if (m_nextDlyp) {  // Tell next assigndly it can share the variable
@@ -398,9 +411,9 @@ private:
             if (newlhsp) {
                 nodep->lhsp(newlhsp);
             } else {
-                VL_DO_DANGLING(nodep->unlinkFrBack()->deleteTree(), nodep);
+                VL_DO_DANGLING(pushDeletep(nodep->unlinkFrBack()), nodep);
             }
-            VL_DO_DANGLING(lhsp->deleteTree(), lhsp);
+            VL_DO_DANGLING(pushDeletep(lhsp), lhsp);
         } else {
             iterateChildren(nodep);
         }
@@ -412,7 +425,7 @@ private:
         if (!nodep->user2Inc()) {  // Not done yet
             if (m_inDly && nodep->access().isWriteOrRW()) {
                 UINFO(4, "AssignDlyVar: " << nodep << endl);
-                markVarUsage(nodep->varScopep(), VU_DLY);
+                markVarUsage(nodep, true);
                 UASSERT_OBJ(m_activep, nodep, "<= not under sensitivity block");
                 UASSERT_OBJ(!nodep->access().isRW(), nodep, "<= on read+write method");
                 if (!m_activep->hasClocked()) {
@@ -436,7 +449,7 @@ private:
                         prep = new AstAssignPre(
                             nodep->fileline(),
                             new AstVarRef(nodep->fileline(), dlyvscp, VAccess::WRITE),
-                            new AstConst(nodep->fileline(), AstConst::LogicFalse()));
+                            new AstConst(nodep->fileline(), AstConst::BitFalse()));
                     } else {
                         prep = new AstAssignPre(
                             nodep->fileline(),
@@ -458,12 +471,12 @@ private:
                 AstVarRef* newrefp = new AstVarRef(nodep->fileline(), dlyvscp, VAccess::WRITE);
                 newrefp->user2(true);  // No reason to do it again
                 nodep->replaceWith(newrefp);
-                VL_DO_DANGLING(nodep->deleteTree(), nodep);
+                VL_DO_DANGLING(pushDeletep(nodep), nodep);
             } else if (!m_inDly && nodep->access().isWriteOrRW()) {
                 // UINFO(9, "NBA " << nodep << endl);
                 if (!m_inInitial) {
                     UINFO(4, "AssignNDlyVar: " << nodep << endl);
-                    markVarUsage(nodep->varScopep(), VU_NONDLY);
+                    markVarUsage(nodep, false);
                 }
             }
         }
