@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -34,7 +34,7 @@
 
 //######################################################################
 
-class DepthVisitor : public AstNVisitor {
+class DepthVisitor final : public AstNVisitor {
 private:
     // NODE STATE
 
@@ -53,14 +53,8 @@ private:
         // if (debug() >= 9) nodep->dumpTree(cout, "deep:");
 
         string newvarname = (string("__Vdeeptemp") + cvtToStr(m_modp->varNumGetInc()));
-        AstVar* varp = new AstVar(nodep->fileline(), AstVarType::STMTTEMP, newvarname,
-                                  // Width, not widthMin, as we may be in
-                                  // middle of BITSEL expression which though
-                                  // it's one bit wide, needs the mask in the
-                                  // upper bits.  (Someday we'll have a valid
-                                  // bitmask instead of widths....)
-                                  // See t_func_crc for an example test that requires this
-                                  VFlagLogicPacked(), nodep->width());
+        AstVar* varp
+            = new AstVar(nodep->fileline(), AstVarType::STMTTEMP, newvarname, nodep->dtypep());
         UASSERT_OBJ(m_cfuncp, nodep, "Deep expression not under a function");
         m_cfuncp->addInitsp(varp);
         // Replace node tree with reference to var
@@ -114,11 +108,12 @@ private:
     virtual void visit(AstNodeTermop* nodep) override {}
     virtual void visit(AstNodeMath* nodep) override {
         // We have some operator defines that use 2 parens, so += 2.
-        m_depth += 2;
-        if (m_depth > m_maxdepth) m_maxdepth = m_depth;
-        iterateChildren(nodep);
-        m_depth -= 2;
-
+        {
+            VL_RESTORER(m_depth);
+            m_depth += 2;
+            if (m_depth > m_maxdepth) m_maxdepth = m_depth;
+            iterateChildren(nodep);
+        }
         if (m_stmtp && (v3Global.opt.compLimitParens() >= 1)  // Else compiler doesn't need it
             && (m_maxdepth - m_depth) > v3Global.opt.compLimitParens()
             && !VN_IS(nodep->backp(), NodeStmt)  // Not much point if we're about to use it
@@ -155,7 +150,7 @@ private:
 public:
     // CONSTRUCTORS
     explicit DepthVisitor(AstNetlist* nodep) { iterate(nodep); }
-    virtual ~DepthVisitor() override {}
+    virtual ~DepthVisitor() override = default;
 };
 
 //######################################################################
