@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -32,7 +32,7 @@
 
 //######################################################################
 
-class DepthBlockVisitor : public AstNVisitor {
+class DepthBlockVisitor final : public AstNVisitor {
 private:
     // NODE STATE
 
@@ -117,7 +117,7 @@ private:
 public:
     // CONSTRUCTORS
     explicit DepthBlockVisitor(AstNetlist* nodep) { iterate(nodep); }
-    virtual ~DepthBlockVisitor() override {}
+    virtual ~DepthBlockVisitor() override = default;
 };
 
 //######################################################################
