@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -34,7 +34,7 @@
 
 //######################################################################
 
-class DescopeVisitor : public AstNVisitor {
+class DescopeVisitor final : public AstNVisitor {
 private:
     // NODE STATE
     //  Cleared entire netlist
@@ -42,7 +42,7 @@ private:
     AstUser1InUse m_inuser1;
 
     // TYPES
-    typedef std::multimap<string, AstCFunc*> FuncMmap;
+    using FuncMmap = std::multimap<std::string, AstCFunc*>;
 
     // STATE
     AstNodeModule* m_modp = nullptr;  // Current module
@@ -60,7 +60,7 @@ private:
         int instances = 0;
         for (AstNode* stmtp = modp->stmtsp(); stmtp; stmtp = stmtp->nextp()) {
             if (VN_IS(stmtp, Scope)) {
-                if (++instances > 1) { return false; }
+                if (++instances > 1) return false;
             }
         }
         return (instances == 1);
@@ -76,7 +76,8 @@ private:
     // Sets 'hierThisr' true if the object is local to this scope
     // (and could be made into a function-local later in V3Localize),
     // false if the object is in another scope.
-    string descopedName(const AstScope* scopep, bool& hierThisr, const AstVar* varp = nullptr) {
+    string descopedName(bool& hierThisr, string& hierUnprot, const AstScope* scopep,
+                        const AstVar* varp) {
         UASSERT(scopep, "Var/Func not scoped");
         hierThisr = (scopep == m_scopep);
 
@@ -118,8 +119,11 @@ private:
         } else if (relativeRefOk && scopep == m_scopep) {
             m_needThis = true;
             return "this->";
+        } else if (VN_IS(scopep->modp(), Class)) {
+            hierUnprot = v3Global.opt.modPrefix() + "_";  // Prefix before protected part
+            return scopep->modp()->name() + "::";
         } else if (relativeRefOk && scopep->aboveScopep() && scopep->aboveScopep() == m_scopep) {
-            // Reference to scope of cell directly under this module, can just "cell->"
+            // Reference to scope of instance directly under this module, can just "cell->"
             string name = scopep->name();
             string::size_type pos;
             if ((pos = name.rfind('.')) != string::npos) name.erase(0, pos + 1);
@@ -249,12 +253,17 @@ private:
     virtual void visit(AstNodeVarRef* nodep) override {
         iterateChildren(nodep);
         // Convert the hierch name
+        UINFO(9, "  ref-in " << nodep << endl);
         UASSERT_OBJ(m_scopep, nodep, "Node not under scope");
         bool hierThis;
-        nodep->hiername(descopedName(nodep->varScopep()->scopep(), hierThis /*ref*/,
-                                     nodep->varScopep()->varp()));
+        string hierUnprot;
+        nodep->hiernameToProt(descopedName(hierThis /*ref*/, hierUnprot /*ref*/,
+                                           nodep->varScopep()->scopep(),
+                                           nodep->varScopep()->varp()));
+        nodep->hiernameToUnprot(hierUnprot);
         nodep->hierThis(hierThis);
         nodep->varScopep(nullptr);
+        UINFO(9, "  refout " << nodep << endl);
     }
     virtual void visit(AstNodeCCall* nodep) override {
         // UINFO(9, "       " << nodep << endl);
@@ -263,7 +272,10 @@ private:
         UASSERT_OBJ(m_scopep, nodep, "Node not under scope");
         UASSERT_OBJ(nodep->funcp()->scopep(), nodep, "CFunc not under scope");
         bool hierThis;
-        nodep->hiername(descopedName(nodep->funcp()->scopep(), hierThis /*ref*/));
+        string hierUnprot;
+        nodep->hiernameToProt(
+            descopedName(hierThis /*ref*/, hierUnprot /*ref*/, nodep->funcp()->scopep(), nullptr));
+        nodep->hiernameToUnprot(hierUnprot);
         // Can't do this, as we may have more calls later
         // nodep->funcp()->scopep(nullptr);
     }
@@ -284,7 +296,7 @@ private:
                 if (nodep->funcPublic()) {
                     // There may be multiple public functions by the same name;
                     // record for later correction or making of shells
-                    m_modFuncs.insert(make_pair(nodep->name(), nodep));
+                    m_modFuncs.emplace(nodep->name(), nodep);
                     nodep->name(m_scopep->nameDotless() + "__" + nodep->name());
                 }
             }
@@ -296,7 +308,7 @@ private:
 public:
     // CONSTRUCTORS
     explicit DescopeVisitor(AstNetlist* nodep) { iterate(nodep); }
-    virtual ~DescopeVisitor() override {}
+    virtual ~DescopeVisitor() override = default;
 };
 
 //######################################################################
@@ -304,6 +316,7 @@ public:
 
 void V3Descope::descopeAll(AstNetlist* nodep) {
     UINFO(2, __FUNCTION__ << ": " << endl);
+    v3Global.assertScoped(false);
     { DescopeVisitor visitor(nodep); }  // Destruct before checking
     V3Global::dumpCheckGlobalTree("descope", 0, v3Global.opt.dumpTreeLevel(__FILE__) >= 3);
 }
