@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -14,8 +14,8 @@
 //
 //*************************************************************************
 
-#ifndef _V3EMITCBASE_H_
-#define _V3EMITCBASE_H_ 1
+#ifndef VERILATOR_V3EMITCBASE_H_
+#define VERILATOR_V3EMITCBASE_H_
 
 #include "config_build.h"
 #include "verilatedos.h"
@@ -30,7 +30,7 @@
 //######################################################################
 // Base Visitor class -- holds output file pointer
 
-class EmitCBaseVisitor : public AstNVisitor {
+class EmitCBaseVisitor VL_NOT_FINAL : public AstNVisitor {
 public:
     // STATE
     V3OutCFile* m_ofp = nullptr;
@@ -106,14 +106,14 @@ public:
     }
 
     // CONSTRUCTORS
-    EmitCBaseVisitor() {}
-    virtual ~EmitCBaseVisitor() override {}
+    EmitCBaseVisitor() = default;
+    virtual ~EmitCBaseVisitor() override = default;
 };
 
 //######################################################################
 // Count operations under the given node, as a visitor of each AstNode
 
-class EmitCBaseCounterVisitor : public AstNVisitor {
+class EmitCBaseCounterVisitor final : public AstNVisitor {
 private:
     // MEMBERS
     int m_count = 0;  // Number of statements
@@ -126,7 +126,7 @@ private:
 public:
     // CONSTRUCTORS
     explicit EmitCBaseCounterVisitor(AstNode* nodep) { iterate(nodep); }
-    virtual ~EmitCBaseCounterVisitor() override {}
+    virtual ~EmitCBaseCounterVisitor() override = default;
     int count() const { return m_count; }
 };
 
