@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2004-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2004-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -28,7 +28,7 @@
 //######################################################################
 // Emit statements
 
-class CMakeEmitter {
+class CMakeEmitter final {
 
     // METHODS
     VL_DEBUG_FUNC;  // Declare debug()
@@ -58,7 +58,7 @@ class CMakeEmitter {
     static void cmake_set_raw(std::ofstream& of, const string& name, const string& raw_value,
                               const string& cache_type = "", const string& docstring = "") {
         of << "set(" << name << " " << raw_value;
-        if (!cache_type.empty()) { of << " CACHE " << cache_type << " \"" << docstring << "\""; }
+        if (!cache_type.empty()) of << " CACHE " << cache_type << " \"" << docstring << "\"";
         of << ")\n";
     }
 
@@ -256,7 +256,7 @@ class CMakeEmitter {
 
 public:
     explicit CMakeEmitter() { emitOverallCMake(); }
-    virtual ~CMakeEmitter() {}
+    virtual ~CMakeEmitter() = default;
 };
 
 void V3EmitCMake::emit() {
