@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -29,7 +29,7 @@
 //######################################################################
 // Symbol table emitting
 
-class EmitCSyms : EmitCBaseVisitor {
+class EmitCSyms final : EmitCBaseVisitor {
     // NODE STATE
     // Cleared on Netlist
     //  AstNodeModule::user1()  -> bool.  Set true __Vconfigure called
@@ -71,20 +71,18 @@ class EmitCSyms : EmitCBaseVisitor {
             , m_modp{modp}
             , m_scopep{scopep} {}
     };
-    typedef std::map<const string, ScopeFuncData> ScopeFuncs;
-    typedef std::map<const string, ScopeVarData> ScopeVars;
-    typedef std::map<const string, ScopeData> ScopeNames;
-    typedef std::pair<AstScope*, AstNodeModule*> ScopeModPair;
-    typedef std::pair<AstNodeModule*, AstVar*> ModVarPair;
-    typedef std::vector<string> ScopeNameList;
-    typedef std::map<const string, ScopeNameList> ScopeNameHierarchy;
+    using ScopeNames = std::map<const std::string, ScopeData>;
+    using ScopeModPair = std::pair<AstScope*, AstNodeModule*>;
+    using ModVarPair = std::pair<AstNodeModule*, AstVar*>;
+    using ScopeNameList = std::vector<std::string>;
+    using ScopeNameHierarchy = std::map<const std::string, ScopeNameList>;
     struct CmpName {
-        inline bool operator()(const ScopeModPair& lhsp, const ScopeModPair& rhsp) const {
+        bool operator()(const ScopeModPair& lhsp, const ScopeModPair& rhsp) const {
             return lhsp.first->name() < rhsp.first->name();
         }
     };
     struct CmpDpi {
-        inline bool operator()(const AstCFunc* lhsp, const AstCFunc* rhsp) const {
+        bool operator()(const AstCFunc* lhsp, const AstCFunc* rhsp) const {
             if (lhsp->dpiImport() != rhsp->dpiImport()) {
                 // cppcheck-suppress comparisonOfFuncReturningBoolError
                 return lhsp->dpiImport() < rhsp->dpiImport();
@@ -100,8 +98,8 @@ class EmitCSyms : EmitCBaseVisitor {
     std::vector<AstCFunc*> m_dpis;  // DPI functions
     std::vector<ModVarPair> m_modVars;  // Each public {mod,var}
     ScopeNames m_scopeNames;  // Each unique AstScopeName
-    ScopeFuncs m_scopeFuncs;  // Each {scope,dpi-export-func}
-    ScopeVars m_scopeVars;  // Each {scope,public-var}
+    std::map<const std::string, ScopeFuncData> m_scopeFuncs;  // Each {scope,dpi-export-func}
+    std::map<const std::string, ScopeVarData> m_scopeVars;  // Each {scope,public-var}
     ScopeNames m_vpiScopeCandidates;  // All scopes for VPI
     ScopeNameHierarchy m_vpiScopeHierarchy;  // The actual hierarchy of scopes
     int m_coverBins = 0;  // Coverage bin number
@@ -109,13 +107,14 @@ class EmitCSyms : EmitCBaseVisitor {
     int m_numStmts = 0;  // Number of statements output
     int m_funcNum = 0;  // CFunc split function number
     V3OutCFile* m_ofpBase = nullptr;  // Base (not split) C file
-    std::map<int, bool> m_usesVfinal;  // Split method uses __Vfinal
+    std::unordered_map<int, bool> m_usesVfinal;  // Split method uses __Vfinal
 
     // METHODS
     void emitSymHdr();
     void checkSplit(bool usesVfinal);
     void closeSplit();
     void emitSymImpPreamble();
+    void emitScopeHier(bool destroy);
     void emitSymImp();
     void emitDpiHdr();
     void emitDpiImp();
@@ -171,7 +170,7 @@ class EmitCSyms : EmitCBaseVisitor {
             const auto scpit = m_vpiScopeCandidates.find(scp);
             if ((scpit != m_vpiScopeCandidates.end())
                 && (m_scopeNames.find(scp) == m_scopeNames.end())) {
-                m_scopeNames.insert(make_pair(scpit->second.m_symName, scpit->second));
+                m_scopeNames.emplace(scpit->second.m_symName, scpit->second);
             }
             string::size_type pos = scp.rfind("__DOT__");
             if (pos == string::npos) {
@@ -218,12 +217,12 @@ class EmitCSyms : EmitCBaseVisitor {
                     // UINFO(9," scnameins sp "<<scpName<<" sp "<<scpPretty<<" ss "<<scpSym<<endl);
                     if (v3Global.opt.vpi()) varHierarchyScopes(scpName);
                     if (m_scopeNames.find(scpSym) == m_scopeNames.end()) {
-                        m_scopeNames.insert(
-                            make_pair(scpSym, ScopeData(scpSym, scpPretty, 0, "SCOPE_OTHER")));
+                        m_scopeNames.insert(std::make_pair(
+                            scpSym, ScopeData(scpSym, scpPretty, 0, "SCOPE_OTHER")));
                     }
                     m_scopeVars.insert(
-                        make_pair(scpSym + " " + varp->name(),
-                                  ScopeVarData(scpSym, varBasePretty, varp, modp, scopep)));
+                        std::make_pair(scpSym + " " + varp->name(),
+                                       ScopeVarData(scpSym, varBasePretty, varp, modp, scopep)));
                 }
             }
         }
@@ -289,21 +288,22 @@ class EmitCSyms : EmitCBaseVisitor {
             string name_dedot = AstNode::dedotName(name);
             int timeunit = m_modp->timeunit().powerOfTen();
             m_vpiScopeCandidates.insert(
-                make_pair(name, ScopeData(scopeSymString(name), name_dedot, timeunit, type)));
+                std::make_pair(name, ScopeData(scopeSymString(name), name_dedot, timeunit, type)));
         }
     }
     virtual void visit(AstScope* nodep) override {
         if (VN_IS(m_modp, Class)) return;  // The ClassPackage is what is visible
         nameCheck(nodep);
 
-        m_scopes.emplace_back(make_pair(nodep, m_modp));
+        m_scopes.emplace_back(std::make_pair(nodep, m_modp));
 
         if (v3Global.opt.vpi() && !nodep->isTop()) {
+            string type = VN_IS(nodep->modp(), Package) ? "SCOPE_OTHER" : "SCOPE_MODULE";
             string name_dedot = AstNode::dedotName(nodep->shortName());
             int timeunit = m_modp->timeunit().powerOfTen();
             m_vpiScopeCandidates.insert(
-                make_pair(nodep->name(), ScopeData(scopeSymString(nodep->name()), name_dedot,
-                                                   timeunit, "SCOPE_MODULE")));
+                std::make_pair(nodep->name(), ScopeData(scopeSymString(nodep->name()), name_dedot,
+                                                        timeunit, type)));
         }
     }
     virtual void visit(AstScopeName* nodep) override {
@@ -312,26 +312,27 @@ class EmitCSyms : EmitCBaseVisitor {
         // <<" ss"<<name<<endl);
         int timeunit = m_modp ? m_modp->timeunit().powerOfTen() : 0;
         if (m_scopeNames.find(name) == m_scopeNames.end()) {
-            m_scopeNames.insert(make_pair(
-                name, ScopeData(name, nodep->scopePrettySymName(), timeunit, "SCOPE_OTHER")));
+            m_scopeNames.emplace(
+                name, ScopeData(name, nodep->scopePrettySymName(), timeunit, "SCOPE_OTHER"));
         }
         if (nodep->dpiExport()) {
             UASSERT_OBJ(m_cfuncp, nodep, "ScopeName not under DPI function");
-            m_scopeFuncs.insert(
-                make_pair(name + " " + m_cfuncp->name(), ScopeFuncData(nodep, m_cfuncp, m_modp)));
+            m_scopeFuncs.insert(std::make_pair(name + " " + m_cfuncp->name(),
+                                               ScopeFuncData(nodep, m_cfuncp, m_modp)));
         } else {
             if (m_scopeNames.find(nodep->scopeDpiName()) == m_scopeNames.end()) {
                 m_scopeNames.insert(
-                    make_pair(nodep->scopeDpiName(),
-                              ScopeData(nodep->scopeDpiName(), nodep->scopePrettyDpiName(),
-                                        timeunit, "SCOPE_OTHER")));
+                    std::make_pair(nodep->scopeDpiName(),
+                                   ScopeData(nodep->scopeDpiName(), nodep->scopePrettyDpiName(),
+                                             timeunit, "SCOPE_OTHER")));
             }
         }
     }
     virtual void visit(AstVar* nodep) override {
         nameCheck(nodep);
         iterateChildren(nodep);
-        if (nodep->isSigUserRdPublic()) { m_modVars.emplace_back(make_pair(m_modp, nodep)); }
+        if (nodep->isSigUserRdPublic() && !m_cfuncp)
+            m_modVars.emplace_back(std::make_pair(m_modp, nodep));
     }
     virtual void visit(AstCoverDecl* nodep) override {
         // Assign numbers to all bins, so we know how big of an array to use
@@ -402,7 +403,7 @@ void EmitCSyms::emitSymHdr() {
             AstCFunc* funcp = itr.second.m_cfuncp;
             if (funcp->dpiExport()) {
                 string cbtype = protect(v3Global.opt.prefix() + "__Vcb_" + funcp->cname() + "_t");
-                types["typedef void (*" + cbtype + ") (" + cFuncArgs(funcp) + ");\n"] = 1;
+                types["using " + cbtype + " = void (*) (" + cFuncArgs(funcp) + ");\n"] = 1;
             }
         }
         for (const auto& i : types) puts(i.first);
@@ -466,8 +467,9 @@ void EmitCSyms::emitSymHdr() {
     }
 
     puts("\n// CREATORS\n");
-    puts(symClassName() + "(" + topClassName() + "* topp, const char* namep);\n");
-    puts(string("~") + symClassName() + "() {}\n");
+    puts(symClassName() + "(VerilatedContext* contextp, " + topClassName()
+         + "* topp, const char* namep);\n");
+    puts(string("~") + symClassName() + "();\n");
 
     for (const auto& i : m_usesVfinal) {
         puts("void " + symClassName() + "_" + cvtToStr(i.first) + "(");
@@ -555,6 +557,39 @@ void EmitCSyms::emitSymImpPreamble() {
     }
 }
 
+void EmitCSyms::emitScopeHier(bool destroy) {
+    if (v3Global.opt.vpi()) {
+        string verb = destroy ? "Tear down" : "Set up";
+        string method = destroy ? "remove" : "add";
+        puts("\n// " + verb + " scope hierarchy\n");
+        for (ScopeNames::const_iterator it = m_scopeNames.begin(); it != m_scopeNames.end();
+             ++it) {
+            string name = it->second.m_prettyName;
+            if (it->first == "TOP") continue;
+            if ((name.find('.') == string::npos) && (it->second.m_type == "SCOPE_MODULE")) {
+                puts("__Vhier." + method + "(0, &" + protect("__Vscope_" + it->second.m_symName)
+                     + ");\n");
+            }
+        }
+
+        for (auto it = m_vpiScopeHierarchy.cbegin(); it != m_vpiScopeHierarchy.cend(); ++it) {
+            for (ScopeNameList::const_iterator lit = it->second.begin(); lit != it->second.end();
+                 ++lit) {
+                string fromname = scopeSymString(it->first);
+                string toname = scopeSymString(*lit);
+                const auto from = vlstd::as_const(m_scopeNames).find(fromname);
+                const auto to = vlstd::as_const(m_scopeNames).find(toname);
+                UASSERT(from != m_scopeNames.end(), fromname + " not in m_scopeNames");
+                UASSERT(to != m_scopeNames.end(), toname + " not in m_scopeNames");
+                puts("__Vhier." + method + "(");
+                puts("&" + protect("__Vscope_" + from->second.m_symName) + ", ");
+                puts("&" + protect("__Vscope_" + to->second.m_symName) + ");\n");
+            }
+        }
+        puts("\n");
+    }
+}
+
 void EmitCSyms::emitSymImp() {
     UINFO(6, __FUNCTION__ << ": " << endl);
     string filename = v3Global.opt.makeDir() + "/" + symClassName() + ".cpp";
@@ -585,7 +620,7 @@ void EmitCSyms::emitSymImp() {
                  + "& os) {\n");
             puts("// LOCAL STATE\n");
             // __Vm_namep presumably already correct
-            if (v3Global.opt.trace()) { puts("os" + op + "__Vm_activity;\n"); }
+            if (v3Global.opt.trace()) puts("os" + op + "__Vm_activity;\n");
             puts("os" + op + "__Vm_didInit;\n");
             puts("// SUBCELL STATE\n");
             for (std::vector<ScopeModPair>::iterator it = m_scopes.begin(); it != m_scopes.end();
@@ -604,10 +639,15 @@ void EmitCSyms::emitSymImp() {
     puts("\n");
 
     puts("\n// FUNCTIONS\n");
-    puts(symClassName() + "::" + symClassName() + "(" + topClassName()
+    puts(symClassName() + "::~" + symClassName() + "()\n");
+    puts("{\n");
+    emitScopeHier(true);
+    puts("}\n\n");
+    puts(symClassName() + "::" + symClassName() + "(VerilatedContext* contextp, " + topClassName()
          + "* topp, const char* namep)\n");
     puts("    // Setup locals\n");
-    puts("    : __Vm_namep(namep)\n");  // No leak, as gets destroyed when the top is destroyed
+    puts("    : VerilatedSyms{contextp}\n");
+    puts("    , __Vm_namep(namep)\n");  // No leak, as gets destroyed when the top is destroyed
     if (v3Global.needTraceDumper()) {
         puts("    , __Vm_dumping(false)\n");
         puts("    , __Vm_dumperp(nullptr)\n");
@@ -685,35 +725,7 @@ void EmitCSyms::emitSymImp() {
         }
     }
 
-    if (v3Global.opt.vpi()) {
-        puts("\n// Setup scope hierarchy\n");
-        for (ScopeNames::const_iterator it = m_scopeNames.begin(); it != m_scopeNames.end();
-             ++it) {
-            string name = it->second.m_prettyName;
-            if (it->first == "TOP") continue;
-            name = name.replace(0, 4, "");  // Remove the "TOP."
-            if ((name.find('.') == string::npos) && (it->second.m_type == "SCOPE_MODULE")) {
-                puts("__Vhier.add(0, &" + protect("__Vscope_" + it->second.m_symName) + ");\n");
-            }
-        }
-
-        for (ScopeNameHierarchy::const_iterator it = m_vpiScopeHierarchy.begin();
-             it != m_vpiScopeHierarchy.end(); ++it) {
-            for (ScopeNameList::const_iterator lit = it->second.begin(); lit != it->second.end();
-                 ++lit) {
-                string fromname = scopeSymString(it->first);
-                string toname = scopeSymString(*lit);
-                const auto from = vlstd::as_const(m_scopeNames).find(fromname);
-                const auto to = vlstd::as_const(m_scopeNames).find(toname);
-                UASSERT(from != m_scopeNames.end(), fromname + " not in m_scopeNames");
-                UASSERT(to != m_scopeNames.end(), toname + " not in m_scopeNames");
-                puts("__Vhier.add(");
-                puts("&" + protect("__Vscope_" + from->second.m_symName) + ", ");
-                puts("&" + protect("__Vscope_" + to->second.m_symName) + ");\n");
-            }
-        }
-        puts("\n");
-    }
+    emitScopeHier(false);
 
     // Everything past here is in the __Vfinal loop, so start a new split file if needed
     closeSplit();
@@ -721,7 +733,7 @@ void EmitCSyms::emitSymImp() {
     if (v3Global.dpi()) {
         m_ofpBase->puts("// Setup export functions\n");
         m_ofpBase->puts("for (int __Vfinal=0; __Vfinal<2; __Vfinal++) {\n");
-        for (ScopeFuncs::iterator it = m_scopeFuncs.begin(); it != m_scopeFuncs.end(); ++it) {
+        for (auto it = m_scopeFuncs.begin(); it != m_scopeFuncs.end(); ++it) {
             AstScopeName* scopep = it->second.m_scopep;
             AstCFunc* funcp = it->second.m_cfuncp;
             AstNodeModule* modp = it->second.m_modp;
@@ -739,7 +751,7 @@ void EmitCSyms::emitSymImp() {
         }
         // It would be less code if each module inserted its own variables.
         // Someday.  For now public isn't common.
-        for (ScopeVars::iterator it = m_scopeVars.begin(); it != m_scopeVars.end(); ++it) {
+        for (auto it = m_scopeVars.begin(); it != m_scopeVars.end(); ++it) {
             checkSplit(true);
             AstNodeModule* modp = it->second.m_modp;
             AstScope* scopep = it->second.m_scopep;
@@ -752,9 +764,9 @@ void EmitCSyms::emitSymImp() {
                 // Range is always first, it's not in "C" order
                 if (basicp->isRanged()) {
                     bounds += " ,";
-                    bounds += cvtToStr(basicp->msb());
+                    bounds += cvtToStr(basicp->hi());
                     bounds += ",";
-                    bounds += cvtToStr(basicp->lsb());
+                    bounds += cvtToStr(basicp->lo());
                     pdim++;
                 }
                 for (AstNodeDType* dtypep = varp->dtypep(); dtypep;) {
@@ -762,9 +774,9 @@ void EmitCSyms::emitSymImp() {
                         = dtypep->skipRefp();  // Skip AstRefDType/AstTypedef, or return same node
                     if (const AstNodeArrayDType* adtypep = VN_CAST(dtypep, NodeArrayDType)) {
                         bounds += " ,";
-                        bounds += cvtToStr(adtypep->msb());
+                        bounds += cvtToStr(adtypep->left());
                         bounds += ",";
-                        bounds += cvtToStr(adtypep->lsb());
+                        bounds += cvtToStr(adtypep->right());
                         if (VN_IS(dtypep, PackArrayDType)) {
                             pdim++;
                         } else {
@@ -777,7 +789,7 @@ void EmitCSyms::emitSymImp() {
                 }
             }
             //
-            if (pdim > 1 || udim > 1) {
+            if (udim > 1 && (pdim && udim)) {
                 puts("//UNSUP ");  // VerilatedImp can't deal with >2d or packed arrays
             }
             puts(protect("__Vscope_" + it->second.m_scopeName) + ".varInsert(__Vfinal,");
@@ -861,12 +873,14 @@ void EmitCSyms::emitDpiHdr() {
     for (AstCFunc* nodep : m_dpis) {
         if (nodep->dpiExportWrapper()) {
             if (!firstExp++) puts("\n// DPI EXPORTS\n");
-            puts("// DPI export" + ifNoProtect(" at " + nodep->fileline()->ascii()) + "\n");
+            putsDecoration("// DPI export" + ifNoProtect(" at " + nodep->fileline()->ascii())
+                           + "\n");
             puts("extern " + nodep->rtnTypeVoid() + " " + nodep->nameProtect() + "("
                  + cFuncArgs(nodep) + ");\n");
         } else if (nodep->dpiImport()) {
             if (!firstImp++) puts("\n// DPI IMPORTS\n");
-            puts("// DPI import" + ifNoProtect(" at " + nodep->fileline()->ascii()) + "\n");
+            putsDecoration("// DPI import" + ifNoProtect(" at " + nodep->fileline()->ascii())
+                           + "\n");
             puts("extern " + nodep->rtnTypeVoid() + " " + nodep->nameProtect() + "("
                  + cFuncArgs(nodep) + ");\n");
         }
@@ -909,8 +923,9 @@ void EmitCSyms::emitDpiImp() {
 
     for (AstCFunc* nodep : m_dpis) {
         if (nodep->dpiExportWrapper()) {
-            puts("#ifndef _VL_DPIDECL_" + nodep->name() + "\n");
-            puts("#define _VL_DPIDECL_" + nodep->name() + "\n");
+            // Prevent multi-definition if used by multiple models
+            puts("#ifndef VL_DPIDECL_" + nodep->name() + "_\n");
+            puts("#define VL_DPIDECL_" + nodep->name() + "_\n");
             puts(nodep->rtnTypeVoid() + " " + nodep->name() + "(" + cFuncArgs(nodep) + ") {\n");
             puts("// DPI export" + ifNoProtect(" at " + nodep->fileline()->ascii()) + "\n");
             puts("return " + topClassName() + "::" + nodep->name() + "(");
