@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -14,8 +14,8 @@
 //
 //*************************************************************************
 
-#ifndef _V3EMITMK_H_
-#define _V3EMITMK_H_ 1
+#ifndef VERILATOR_V3EMITMK_H_
+#define VERILATOR_V3EMITMK_H_
 
 #include "config_build.h"
 #include "verilatedos.h"
@@ -24,7 +24,7 @@ class V3HierBlockPlan;
 
 //============================================================================
 
-class V3EmitMk {
+class V3EmitMk final {
 public:
     static void emitmk();
     static void emitHierVerilation(const V3HierBlockPlan* planp);
