@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2004-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2004-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -28,7 +28,7 @@
 //######################################################################
 // Emit statements and math operators
 
-class EmitVBaseVisitor : public EmitCBaseVisitor {
+class EmitVBaseVisitor VL_NOT_FINAL : public EmitCBaseVisitor {
     // MEMBERS
     bool m_suppressSemi = false;
     bool m_suppressUnknown = false;
@@ -310,13 +310,15 @@ class EmitVBaseVisitor : public EmitCBaseVisitor {
     }
     virtual void visit(AstNodeFor* nodep) override {
         putfs(nodep, "for (");
-        m_suppressSemi = true;
-        iterateAndNextNull(nodep->initsp());
-        puts(";");
-        iterateAndNextNull(nodep->condp());
-        puts(";");
-        iterateAndNextNull(nodep->incsp());
-        m_suppressSemi = false;
+        {
+            VL_RESTORER(m_suppressSemi);
+            m_suppressSemi = true;
+            iterateAndNextNull(nodep->initsp());
+            puts(";");
+            iterateAndNextNull(nodep->condp());
+            puts(";");
+            iterateAndNextNull(nodep->incsp());
+        }
         puts(") begin\n");
         iterateAndNextNull(nodep->bodysp());
         putqs(nodep, "end\n");
@@ -381,9 +383,13 @@ class EmitVBaseVisitor : public EmitCBaseVisitor {
     }
     virtual void visit(AstTextBlock* nodep) override {
         visit(VN_CAST(nodep, NodeSimpleText));
-        for (AstNode* childp = nodep->nodesp(); childp; childp = childp->nextp()) {
-            iterate(childp);
-            if (nodep->commas() && childp->nextp()) puts(", ");
+        {
+            VL_RESTORER(m_suppressSemi);
+            m_suppressVarSemi = nodep->commas();
+            for (AstNode* childp = nodep->nodesp(); childp; childp = childp->nextp()) {
+                iterate(childp);
+                if (nodep->commas() && childp->nextp()) puts(", ");
+            }
         }
     }
     virtual void visit(AstScopeName* nodep) override {}
@@ -513,11 +519,11 @@ class EmitVBaseVisitor : public EmitCBaseVisitor {
     }
     virtual void visit(AstRange* nodep) override {
         puts("[");
-        if (VN_IS(nodep->msbp(), Const) && VN_IS(nodep->lsbp(), Const)) {
+        if (VN_IS(nodep->leftp(), Const) && VN_IS(nodep->rightp(), Const)) {
             // Looks nicer if we print [1:0] rather than [32'sh1:32sh0]
-            puts(cvtToStr(VN_CAST(nodep->leftp(), Const)->toSInt()));
+            puts(cvtToStr(nodep->leftConst()));
             puts(":");
-            puts(cvtToStr(VN_CAST(nodep->rightp(), Const)->toSInt()));
+            puts(cvtToStr(nodep->rightConst()));
             puts("]");
         } else {
             iterateAndNextNull(nodep->leftp());
@@ -570,7 +576,7 @@ class EmitVBaseVisitor : public EmitCBaseVisitor {
             puts(" ");
         } else if (nodep->isRanged()) {
             puts(" [");
-            puts(cvtToStr(nodep->msb()));
+            puts(cvtToStr(nodep->hi()));
             puts(":0] ");
         }
     }
@@ -608,12 +614,18 @@ class EmitVBaseVisitor : public EmitCBaseVisitor {
         puts(")");
     }
     virtual void visit(AstArg* nodep) override { iterateAndNextNull(nodep->exprp()); }
+    virtual void visit(AstPrintTimeScale* nodep) override {
+        puts(nodep->verilogKwd());
+        puts(";\n");
+    }
+
     // Terminals
     virtual void visit(AstVarRef* nodep) override {
         if (nodep->varScopep()) {
             putfs(nodep, nodep->varScopep()->prettyName());
         } else {
-            putfs(nodep, nodep->hiername());
+            putfs(nodep, nodep->hiernameToUnprot());
+            puts(nodep->hiernameToProt());
             puts(nodep->varp()->prettyName());
         }
     }
@@ -628,16 +640,32 @@ class EmitVBaseVisitor : public EmitCBaseVisitor {
     virtual void visit(AstTopScope* nodep) override { iterateChildren(nodep); }
     virtual void visit(AstScope* nodep) override { iterateChildren(nodep); }
     virtual void visit(AstVar* nodep) override {
-        putfs(nodep, nodep->verilogKwd());
-        puts(" ");
-        iterate(nodep->dtypep());
-        puts(" ");
-        puts(nodep->prettyName());
-        if (!m_suppressVarSemi) {
-            puts(";\n");
-        } else {
-            puts("\n");
+        if (nodep->isIO()) {
+            putfs(nodep, nodep->verilogKwd());
+            puts(" ");
         }
+        std::vector<const AstUnpackArrayDType*> unpackps;
+        for (AstNodeDType* dtypep = nodep->dtypep(); dtypep;) {
+            dtypep = dtypep->skipRefp();
+            if (AstUnpackArrayDType* unpackp = VN_CAST(dtypep, UnpackArrayDType)) {
+                unpackps.push_back(unpackp);
+                dtypep = unpackp->subDTypep();
+            } else {
+                iterate(dtypep);
+                puts(" ");
+                puts(nodep->prettyName());
+                dtypep = nullptr;
+            }
+        }
+        // If nodep is an unpacked array, append unpacked dimensions
+        for (const auto& unpackp : unpackps) {
+            puts("[");
+            puts(cvtToStr(unpackp->rangep()->leftConst()));
+            puts(":");
+            puts(cvtToStr(unpackp->rangep()->rightConst()));
+            puts("]");
+        }
+        puts(m_suppressVarSemi ? "\n" : ";\n");
     }
     virtual void visit(AstActive* nodep) override {
         m_sensesp = nodep->sensesp();
@@ -667,13 +695,13 @@ public:
     explicit EmitVBaseVisitor(bool suppressUnknown, AstSenTree* domainp)
         : m_suppressUnknown{suppressUnknown}
         , m_sensesp{domainp} {}
-    virtual ~EmitVBaseVisitor() override {}
+    virtual ~EmitVBaseVisitor() override = default;
 };
 
 //######################################################################
 // Emit to an output file
 
-class EmitVFileVisitor : public EmitVBaseVisitor {
+class EmitVFileVisitor final : public EmitVBaseVisitor {
     // MEMBERS
     V3OutFile* m_ofp;
     // METHODS
@@ -685,21 +713,19 @@ class EmitVFileVisitor : public EmitVBaseVisitor {
     virtual void putsNoTracking(const string& str) override { ofp()->putsNoTracking(str); }
 
 public:
-    EmitVFileVisitor(AstNode* nodep, V3OutFile* ofp, bool trackText, bool suppressVarSemi,
-                     bool suppressUnknown)
+    EmitVFileVisitor(AstNode* nodep, V3OutFile* ofp, bool trackText, bool suppressUnknown)
         : EmitVBaseVisitor{suppressUnknown, nullptr} {
         m_ofp = ofp;
         m_trackText = trackText;
-        m_suppressVarSemi = suppressVarSemi;
         iterate(nodep);
     }
-    virtual ~EmitVFileVisitor() override {}
+    virtual ~EmitVFileVisitor() override = default;
 };
 
 //######################################################################
 // Emit to a stream (perhaps stringstream)
 
-class EmitVStreamVisitor : public EmitVBaseVisitor {
+class EmitVStreamVisitor final : public EmitVBaseVisitor {
     // MEMBERS
     std::ostream& m_os;
     // METHODS
@@ -715,13 +741,13 @@ public:
         , m_os(os) {  // Need () or GCC 4.8 false warning
         iterate(nodep);
     }
-    virtual ~EmitVStreamVisitor() override {}
+    virtual ~EmitVStreamVisitor() override = default;
 };
 
 //######################################################################
 // Emit to a stream (perhaps stringstream)
 
-class EmitVPrefixedFormatter : public V3OutFormatter {
+class EmitVPrefixedFormatter final : public V3OutFormatter {
     std::ostream& m_os;
     string m_prefix;  // What to print at beginning of each line
     int m_flWidth;  // Padding of fileline
@@ -763,7 +789,7 @@ public:
     }
 };
 
-class EmitVPrefixedVisitor : public EmitVBaseVisitor {
+class EmitVPrefixedVisitor final : public EmitVBaseVisitor {
     // MEMBERS
     EmitVPrefixedFormatter m_formatter;  // Special verilog formatter (Way down the
                                          // inheritance is another unused V3OutFormatter)
@@ -789,10 +815,10 @@ public:
                          AstSenTree* domainp, bool user3mark)
         : EmitVBaseVisitor{false, domainp}
         , m_formatter{os, prefix, flWidth} {
-        if (user3mark) { AstUser3InUse::check(); }
+        if (user3mark) AstUser3InUse::check();
         iterate(nodep);
     }
-    virtual ~EmitVPrefixedVisitor() override {}
+    virtual ~EmitVPrefixedVisitor() override = default;
 };
 
 //######################################################################
@@ -814,7 +840,7 @@ void V3EmitV::emitvFiles() {
             V3OutVFile of(vfilep->name());
             of.puts("// DESCR"
                     "IPTION: Verilator generated Verilog\n");
-            EmitVFileVisitor visitor(vfilep->tblockp(), &of, true, true, false);
+            EmitVFileVisitor visitor(vfilep->tblockp(), &of, true, false);
         }
     }
 }
@@ -823,5 +849,5 @@ void V3EmitV::debugEmitV(const string& stage) {
     UINFO(2, __FUNCTION__ << ": " << endl);
     string filename = v3Global.opt.makeDir() + "/" + v3Global.opt.prefix() + "__" + stage + ".v";
     V3OutVFile of(filename);
-    EmitVFileVisitor visitor(v3Global.rootp(), &of, true, false, true);
+    EmitVFileVisitor visitor(v3Global.rootp(), &of, true, true);
 }
