@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2004-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2004-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -37,7 +37,7 @@
 //######################################################################
 // Expand state, as a visitor of each AstNode
 
-class ExpandVisitor : public AstNVisitor {
+class ExpandVisitor final : public AstNVisitor {
 private:
     // NODE STATE
     //  AstNode::user1()        -> bool.  Processed
@@ -270,15 +270,6 @@ private:
         }
         return true;
     }
-    bool expandWide(AstNodeAssign* nodep, AstXnor* rhsp) {
-        UINFO(8, "    Wordize ASSIGN(XNOR) " << nodep << endl);
-        for (int w = 0; w < nodep->widthWords(); w++) {
-            addWordAssign(nodep, w,
-                          new AstXnor(nodep->fileline(), newAstWordSelClone(rhsp->lhsp(), w),
-                                      newAstWordSelClone(rhsp->rhsp(), w)));
-        }
-        return true;
-    }
     //-------- Triops
     bool expandWide(AstNodeAssign* nodep, AstNodeCond* rhsp) {
         UINFO(8, "    Wordize ASSIGN(COND) " << nodep << endl);
@@ -497,9 +488,21 @@ private:
                                                           maskold.edataWord(w)),
                                              oldvalp);
                         }
-                        addWordAssign(nodep, w, destp,
-                                      new AstOr(lhsp->fileline(), oldvalp,
-                                                newWordGrabShift(lhsp->fileline(), w, rhsp, lsb)));
+
+                        // Appropriate word of new value to insert:
+                        AstNode* newp = newWordGrabShift(lhsp->fileline(), w, rhsp, lsb);
+
+                        // Apply cleaning at the top word of the destination
+                        // (no cleaning to do if dst's width is a whole number
+                        // of words).
+                        if (w == destp->widthWords() - 1 && VL_BITBIT_E(destp->widthMin()) != 0) {
+                            V3Number cleanmask(nodep, VL_EDATASIZE);
+                            cleanmask.setMask(VL_BITBIT_E(destp->widthMin()));
+                            newp = new AstAnd(lhsp->fileline(), newp,
+                                              new AstConst(lhsp->fileline(), cleanmask));
+                        }
+
+                        addWordAssign(nodep, w, destp, new AstOr(lhsp->fileline(), oldvalp, newp));
                     }
                 }
                 VL_DO_DANGLING(rhsp->deleteTree(), rhsp);
@@ -515,15 +518,22 @@ private:
                     oldvalp = new AstAnd(lhsp->fileline(), new AstConst(lhsp->fileline(), maskold),
                                          oldvalp);
                 }
-                AstNode* newp = new AstOr(lhsp->fileline(), oldvalp,
-                                          new AstShiftL(lhsp->fileline(), rhsp,
-                                                        new AstConst(lhsp->fileline(), lsb),
-                                                        destp->width()));
+
+                // The bit-select can refer to bits outside the width of nodep
+                // which we aren't allowed to assign to.  This is a mask of the
+                // valid range of nodep which we apply to the new shifted RHS.
+                V3Number cleanmask(nodep, destp->widthMin());
+                cleanmask.setMask(destp->widthMin());
+                AstNode* shifted = new AstShiftL(
+                    lhsp->fileline(), rhsp, new AstConst(lhsp->fileline(), lsb), destp->width());
+                AstNode* cleaned = new AstAnd(lhsp->fileline(), shifted,
+                                              new AstConst(lhsp->fileline(), cleanmask));
+                AstNode* newp = new AstOr(lhsp->fileline(), oldvalp, cleaned);
                 newp = new AstAssign(nodep->fileline(), destp, newp);
                 insertBefore(nodep, newp);
             }
             return true;
-        } else {  // non-const RHS
+        } else {  // non-const select offset
             if (destwide && lhsp->widthConst() == 1) {
                 UINFO(8, "    ASSIGNSEL(varlsb,wide,1bit) " << nodep << endl);
                 AstNode* rhsp = nodep->rhsp()->unlinkFrBack();
@@ -588,11 +598,21 @@ private:
                                                  lhsp->lsbp()->cloneTree(true), destp->width())),
                         oldvalp);
                 }
-                AstNode* newp
-                    = new AstOr(lhsp->fileline(), oldvalp,
-                                new AstShiftL(lhsp->fileline(), rhsp,
-                                              lhsp->lsbp()->cloneTree(true), destp->width()));
-                newp = new AstAssign(nodep->fileline(), destp, newp);
+                AstNode* newp = new AstShiftL(lhsp->fileline(), rhsp,
+                                              lhsp->lsbp()->cloneTree(true), destp->width());
+                // Apply cleaning to the new value being inserted.  Mask is
+                // slightly wider than necessary to avoid an AND with all ones
+                // being optimized out.  No need to clean if destp is
+                // quad-sized as there are no extra bits to contaminate
+                if (destp->widthMin() != 64) {
+                    V3Number cleanmask(nodep, destp->widthMin() + 1);
+                    cleanmask.setMask(destp->widthMin());
+                    newp = new AstAnd(lhsp->fileline(), newp,
+                                      new AstConst(lhsp->fileline(), cleanmask));
+                }
+
+                newp = new AstAssign(nodep->fileline(), destp,
+                                     new AstOr(lhsp->fileline(), oldvalp, newp));
                 // newp->dumpTree(cout, "-  new: ");
                 insertBefore(nodep, newp);
                 return true;
@@ -859,8 +879,6 @@ private:
                 did = expandWide(nodep, rhsp);
             } else if (AstXor* rhsp = VN_CAST(nodep->rhsp(), Xor)) {
                 did = expandWide(nodep, rhsp);
-            } else if (AstXnor* rhsp = VN_CAST(nodep->rhsp(), Xnor)) {
-                did = expandWide(nodep, rhsp);
             } else if (AstNodeCond* rhsp = VN_CAST(nodep->rhsp(), NodeCond)) {
                 did = expandWide(nodep, rhsp);
             }
@@ -879,7 +897,7 @@ private:
 public:
     // CONSTRUCTORS
     explicit ExpandVisitor(AstNetlist* nodep) { iterate(nodep); }
-    virtual ~ExpandVisitor() override {}
+    virtual ~ExpandVisitor() override = default;
 };
 
 //----------------------------------------------------------------------
