@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -62,23 +62,22 @@ constexpr int INFILTER_CACHE_MAX = (64 * 1024);  // Maximum bytes to cache if sa
 //######################################################################
 // V3File Internal state
 
-class V3FileDependImp {
+class V3FileDependImp final {
     // TYPES
-    class DependFile {
+    class DependFile final {
         // A single file
         bool m_target;  // True if write, else read
-        bool m_exists;
+        bool m_exists = true;
         string m_filename;  // Filename
         struct stat m_stat;  // Stat information
     public:
         DependFile(const string& filename, bool target)
             : m_target{target}
-            , m_exists{true}
             , m_filename{filename} {
             m_stat.st_ctime = 0;
             m_stat.st_mtime = 0;
         }
-        ~DependFile() {}
+        ~DependFile() = default;
         const string& filename() const { return m_filename; }
         bool target() const { return m_target; }
         bool exists() const { return m_exists; }
@@ -160,12 +159,12 @@ inline void V3FileDependImp::writeDepend(const string& filename) {
         if (!i.target()) *ofp << i.filename() << " ";
     }
 
-    *ofp << endl;
+    *ofp << '\n';
 
     if (v3Global.opt.makePhony()) {
-        *ofp << endl;
+        *ofp << '\n';
         for (const DependFile& i : m_filenameList) {
-            if (!i.target()) *ofp << i.filename() << ":" << endl;
+            if (!i.target()) *ofp << i.filename() << ":\n";
         }
     }
 }
@@ -184,9 +183,8 @@ inline void V3FileDependImp::writeTimes(const string& filename, const string& cm
 
     string cmdline = stripQuotes(cmdlineIn);
     *ofp << "# DESCR"
-         << "IPTION: Verilator output: Timestamp data for --skip-identical.  Delete at will."
-         << endl;
-    *ofp << "C \"" << cmdline << "\"" << endl;
+         << "IPTION: Verilator output: Timestamp data for --skip-identical.  Delete at will.\n";
+    *ofp << "C \"" << cmdline << "\"\n";
 
     for (std::set<DependFile>::iterator iter = m_filenameList.begin();
          iter != m_filenameList.end(); ++iter) {
@@ -210,7 +208,7 @@ inline void V3FileDependImp::writeTimes(const string& filename, const string& cm
         *ofp << " " << std::setw(11) << iter->mstime();
         *ofp << " " << std::setw(11) << iter->mnstime();
         *ofp << " \"" << iter->filename() << "\"";
-        *ofp << endl;
+        *ofp << '\n';
     }
 }
 
@@ -322,18 +320,17 @@ void V3File::createMakeDir() {
     if (!created) {
         created = true;
         V3Os::createDir(v3Global.opt.makeDir());
-        if (v3Global.opt.hierTop()) { V3Os::createDir(v3Global.opt.hierTopDataDir()); }
+        if (v3Global.opt.hierTop()) V3Os::createDir(v3Global.opt.hierTopDataDir());
     }
 }
 
 //######################################################################
 // VInFilterImp
 
-class VInFilterImp {
-    typedef std::map<const string, string> FileContentsMap;
-    typedef VInFilter::StrList StrList;
+class VInFilterImp final {
+    using StrList = VInFilter::StrList;
 
-    FileContentsMap m_contentsMap;  // Cache of file contents
+    std::map<const std::string, std::string> m_contentsMap;  // Cache of file contents
     bool m_readEof = false;  // Received EOF on read
 #ifdef INFILTER_PIPE
     pid_t m_pid = 0;  // fork() process id
@@ -582,16 +579,16 @@ protected:
             // Cache small files (only to save space)
             // It's quite common to `include "timescale" thousands of times
             // This isn't so important if it's just an open(), but filtering can be slow
-            m_contentsMap.insert(make_pair(filename, listString(outl)));
+            m_contentsMap.emplace(filename, listString(outl));
         }
         return true;
     }
-    static size_t listSize(StrList& sl) {
+    static size_t listSize(const StrList& sl) {
         size_t out = 0;
         for (const string& i : sl) out += i.length();
         return out;
     }
-    static string listString(StrList& sl) {
+    static string listString(const StrList& sl) {
         string out;
         for (const string& i : sl) out += i;
         return out;
@@ -789,11 +786,11 @@ void V3OutFormatter::puts(const char* strg) {
             wordstart = false;
             break;
         case 'e':
-            if (wordstart && m_lang == LA_VERILOG && tokenEnd(cp)) { indentDec(); }
+            if (wordstart && m_lang == LA_VERILOG && tokenEnd(cp)) indentDec();
             wordstart = false;
             break;
         case 'm':
-            if (wordstart && m_lang == LA_VERILOG && tokenStart(cp, "module")) { indentInc(); }
+            if (wordstart && m_lang == LA_VERILOG && tokenStart(cp, "module")) indentInc();
             wordstart = false;
             break;
         default: wordstart = false; break;
@@ -875,9 +872,7 @@ string V3OutFormatter::quoteNameControls(const string& namein, V3OutFormatter::L
             } else if (isprint(c)) {
                 out += c;
             } else {
-                char decimal[10];
-                sprintf(decimal, "&#%u;", (unsigned char)c);
-                out += decimal;
+                out += string("&#") + cvtToStr((unsigned int)(c & 0xff)) + ";";
             }
         }
     } else {
@@ -895,9 +890,8 @@ string V3OutFormatter::quoteNameControls(const string& namein, V3OutFormatter::L
                 out += c;
             } else {
                 // This will also cover \a etc
-                // Can't use %03o as messes up when signed
-                char octal[10];
-                sprintf(octal, "\\%o%o%o", (c >> 6) & 3, (c >> 3) & 7, c & 7);
+                string octal = string("\\") + cvtToStr((c >> 6) & 3) + cvtToStr((c >> 3) & 7)
+                               + cvtToStr(c & 7);
                 out += octal;
             }
         }
@@ -909,10 +903,11 @@ string V3OutFormatter::quoteNameControls(const string& namein, V3OutFormatter::L
 // Simple wrappers
 
 void V3OutFormatter::printf(const char* fmt...) {
-    char sbuff[5000];
+    constexpr size_t bufsize = 5000;
+    char sbuff[bufsize];
     va_list ap;
     va_start(ap, fmt);
-    vsprintf(sbuff, fmt, ap);
+    VL_VSNPRINTF(sbuff, bufsize, fmt, ap);
     va_end(ap);
     this->puts(sbuff);
 }
@@ -940,7 +935,7 @@ void V3OutFile::putsForceIncs() {
 void V3OutCFile::putsGuard() {
     UASSERT(!m_guard, "Already called putsGuard in emit file");
     m_guard = true;
-    string var = VString::upcase(string("_") + V3Os::filenameNonDir(filename()) + "_");
+    string var = VString::upcase(string("VERILATED_") + V3Os::filenameNonDir(filename()) + "_");
     for (char& c : var) {
         if (!isalnum(c)) c = '_';
     }
@@ -951,12 +946,10 @@ void V3OutCFile::putsGuard() {
 //######################################################################
 // VIdProtect
 
-class VIdProtectImp {
+class VIdProtectImp final {
     // MEMBERS
-    typedef std::map<const string, string> IdMap;
-    IdMap m_nameMap;  // Map of old name into new name
-    typedef std::unordered_set<std::string> IdSet;
-    IdSet m_newIdSet;  // Which new names exist
+    std::map<const std::string, std::string> m_nameMap;  // Map of old name into new name
+    std::unordered_set<std::string> m_newIdSet;  // Which new names exist
 protected:
     // CONSTRUCTORS
     friend class VIdProtect;
@@ -972,7 +965,7 @@ public:
         passthru("vlTOPp");
         passthru("vlSymsp");
     }
-    ~VIdProtectImp() {}
+    ~VIdProtectImp() = default;
     // METHODS
     string passthru(const string& old) {
         if (!v3Global.opt.protectIds()) return old;
@@ -982,7 +975,7 @@ public:
             UASSERT(old == it->second,
                     "Passthru request for '" + old + "' after already --protect-ids of it.");
         } else {
-            m_nameMap.insert(make_pair(old, old));
+            m_nameMap.emplace(old, old);
             m_newIdSet.insert(old);
         }
         return old;
@@ -1013,7 +1006,7 @@ public:
                     }
                 }
             }
-            m_nameMap.insert(make_pair(old, out));
+            m_nameMap.emplace(old, out);
             m_newIdSet.insert(out);
             return out;
         }
