@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -14,8 +14,8 @@
 //
 //*************************************************************************
 
-#ifndef _V3GLOBAL_H_
-#define _V3GLOBAL_H_ 1
+#ifndef VERILATOR_V3GLOBAL_H_
+#define VERILATOR_V3GLOBAL_H_
 
 // clang-format off
 #include "config_build.h"
@@ -52,7 +52,7 @@ template <typename T> class VRestorer {
     const T m_saved;  // Value saved, for later restore
 
 public:
-    VRestorer(T& permr)
+    explicit VRestorer(T& permr)
         : m_ref(permr)
         , m_saved(permr) {}
     ~VRestorer() { m_ref = m_saved; }
@@ -61,16 +61,16 @@ public:
 
 //######################################################################
 
-class VWidthMinUsage {
+class VWidthMinUsage final {
 public:
     enum en : uint8_t { LINT_WIDTH, MATCHES_WIDTH, VERILOG_WIDTH };
     enum en m_e;
-    inline VWidthMinUsage()
+    VWidthMinUsage()
         : m_e{LINT_WIDTH} {}
     // cppcheck-suppress noExplicitConstructor
-    inline VWidthMinUsage(en _e)
+    VWidthMinUsage(en _e)
         : m_e{_e} {}
-    explicit inline VWidthMinUsage(int _e)
+    explicit VWidthMinUsage(int _e)
         : m_e(static_cast<en>(_e)) {}  // Need () or GCC 4.8 false warning
     operator en() const { return m_e; }
 };
@@ -87,7 +87,7 @@ inline bool operator==(VWidthMinUsage::en lhs, const VWidthMinUsage& rhs) {
 //######################################################################
 // V3Global - The top level class for the entire program
 
-class V3Global {
+class V3Global final {
     // Globals
     AstNetlist* m_rootp;  // Root of entire netlist
     V3HierBlockPlan* m_hierPlanp;  // Hierarchical verilation plan, nullptr unless hier_block
@@ -95,15 +95,18 @@ class V3Global {
 
     int m_debugFileNumber = 0;  // Number to append to debug files created
     bool m_assertDTypesResolved = false;  // Tree should have dtypep()'s
+    bool m_assertScoped = false;  // Tree is scoped
     bool m_constRemoveXs = false;  // Const needs to strip any Xs
-    bool m_needHeavy = false;  // Need verilated_heavy.h include
+    // Experimenting with always requiring heavy, see (#2701)
+    bool m_needHeavy = true;  // Need verilated_heavy.h include
     bool m_needTraceDumper = false;  // Need __Vm_dumperp in symbols
     bool m_dpi = false;  // Need __Dpi include files
     bool m_useParallelBuild = false;  // Use parallel build for model
+    bool m_useRandomizeMethods = false;  // Need to define randomize() class methods
 
     // Memory address to short string mapping (for debug)
-    typedef std::unordered_map<const void*, std::string> PtrToIdMap;  // The map type
-    PtrToIdMap m_ptrToId;  // The actual 'address' <=> 'short string' bijection
+    std::unordered_map<const void*, std::string>
+        m_ptrToId;  // The actual 'address' <=> 'short string' bijection
 
 public:
     // Options
@@ -125,6 +128,7 @@ public:
     AstNetlist* rootp() const { return m_rootp; }
     VWidthMinUsage widthMinUsage() const { return m_widthMinUsage; }
     bool assertDTypesResolved() const { return m_assertDTypesResolved; }
+    bool assertScoped() const { return m_assertScoped; }
 
     // METHODS
     void readFiles();
@@ -132,16 +136,12 @@ public:
     static void dumpCheckGlobalTree(const string& stagename, int newNumber = 0,
                                     bool doDump = true);
     void assertDTypesResolved(bool flag) { m_assertDTypesResolved = flag; }
+    void assertScoped(bool flag) { m_assertScoped = flag; }
     void widthMinUsage(const VWidthMinUsage& flag) { m_widthMinUsage = flag; }
     bool constRemoveXs() const { return m_constRemoveXs; }
     void constRemoveXs(bool flag) { m_constRemoveXs = flag; }
-    string debugFilename(const string& nameComment, int newNumber = 0) {
-        ++m_debugFileNumber;
-        if (newNumber) m_debugFileNumber = newNumber;
-        char digits[100];
-        sprintf(digits, "%03d", m_debugFileNumber);
-        return opt.hierTopDataDir() + "/" + opt.prefix() + "_" + digits + "_" + nameComment;
-    }
+    string debugFilename(const string& nameComment, int newNumber = 0);
+    static string digitsFilename(int number);
     bool needHeavy() const { return m_needHeavy; }
     void needHeavy(bool flag) { m_needHeavy = flag; }
     bool needTraceDumper() const { return m_needTraceDumper; }
@@ -155,6 +155,8 @@ public:
     }
     void useParallelBuild(bool flag) { m_useParallelBuild = flag; }
     bool useParallelBuild() const { return m_useParallelBuild; }
+    void useRandomizeMethods(bool flag) { m_useRandomizeMethods = flag; }
+    bool useRandomizeMethods() const { return m_useRandomizeMethods; }
     const std::string& ptrToId(const void* p);
 };
 
