@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -14,8 +14,8 @@
 //
 //*************************************************************************
 
-#ifndef _V3GRAPH_H_
-#define _V3GRAPH_H_ 1
+#ifndef VERILATOR_V3GRAPH_H_
+#define VERILATOR_V3GRAPH_H_
 
 #include "config_build.h"
 #include "verilatedos.h"
@@ -37,7 +37,7 @@ class OrderLogicVertex;
 // Most graph algorithms accept an arbitrary function that returns
 // True for those edges we should honor.
 
-typedef bool (*V3EdgeFuncP)(const V3GraphEdge* edgep);
+using V3EdgeFuncP = bool (*)(const V3GraphEdge* edgep);
 
 //=============================================================================
 // When the Graph represents a directional acyclical graph (DAG), following
@@ -45,7 +45,7 @@ typedef bool (*V3EdgeFuncP)(const V3GraphEdge* edgep);
 // it's useful to have algorithms that can walk in either direction, hence
 // some methods take GraphWay to programmatically select the direction.
 
-class GraphWay {
+class GraphWay final {
 public:
     enum en : uint8_t {
         FORWARD = 0,
@@ -77,7 +77,7 @@ inline bool operator==(GraphWay::en lhs, const GraphWay& rhs) { return lhs == rh
 
 //============================================================================
 
-class V3Graph {
+class V3Graph VL_NOT_FINAL {
 private:
     // MEMBERS
     V3List<V3GraphVertex*> m_vertices;  // All vertices
@@ -145,12 +145,6 @@ public:
     /// Make acyclical (into a tree) by breaking a minimal subset of cutable edges.
     void acyclic(V3EdgeFuncP edgeFuncp);
 
-    /// Delete any nodes with only outputs
-    void deleteCutableOnlyEdges();
-
-    /// Any cutable edges become non-cutable
-    void makeEdgesNonCutable(V3EdgeFuncP edgeFuncp);
-
     /// Remove any redundant edges, weights become MAX of any other weight
     void removeRedundantEdges(V3EdgeFuncP edgeFuncp);
 
@@ -185,7 +179,7 @@ public:
 
 //============================================================================
 
-class V3GraphVertex {
+class V3GraphVertex VL_NOT_FINAL {
     // Vertices may be a 'gate'/wire statement OR a variable
 protected:
     friend class V3Graph;
@@ -218,7 +212,7 @@ public:
     virtual V3GraphVertex* clone(V3Graph* graphp) const {
         return new V3GraphVertex(graphp, *this);
     }
-    virtual ~V3GraphVertex() {}
+    virtual ~V3GraphVertex() = default;
     void unlinkEdges(V3Graph* graphp);
     void unlinkDelete(V3Graph* graphp);
 
@@ -273,7 +267,7 @@ std::ostream& operator<<(std::ostream& os, V3GraphVertex* vertexp);
 
 //============================================================================
 
-class V3GraphEdge {
+class V3GraphEdge VL_NOT_FINAL {
     // Wires/variables aren't edges.  Edges have only a single to/from vertex
 public:
     // ENUMS
@@ -317,7 +311,7 @@ public:
     virtual V3GraphEdge* clone(V3Graph* graphp, V3GraphVertex* fromp, V3GraphVertex* top) const {
         return new V3GraphEdge(graphp, fromp, top, *this);
     }
-    virtual ~V3GraphEdge() {}
+    virtual ~V3GraphEdge() = default;
     // METHODS
     virtual string name() const { return m_fromp->name() + "->" + m_top->name(); }
     virtual string dotLabel() const { return ""; }
