@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -26,50 +26,11 @@
 #include <map>
 #include <list>
 
-//######################################################################
-//######################################################################
-// Algorithms - delete
-
-void V3Graph::deleteCutableOnlyEdges() {
-    // Any vertices with only cutable edges will get deleted
-
-    // Vertex::m_user begin: indicates can be deleted
-    // Pass 1, mark those.  Don't delete now, as we don't want to rip out whole trees
-    for (V3GraphVertex* vertexp = verticesBeginp(); vertexp; vertexp = vertexp->verticesNextp()) {
-        vertexp->user(true);
-        for (V3GraphEdge* edgep = vertexp->inBeginp(); edgep; edgep = edgep->inNextp()) {
-            if (!edgep->cutable()) {
-                vertexp->user(false);  // Can't delete it
-                break;
-            }
-        }
-        for (V3GraphEdge* edgep = vertexp->outBeginp(); edgep; edgep = edgep->outNextp()) {
-            if (!edgep->cutable()) {
-                vertexp->user(false);  // Can't delete it
-                break;
-            }
-        }
-    }
-
-    // Pass 2, delete those marked
-    // Rather than doing a delete() we set the weight to 0 which disconnects the edge.
-    for (V3GraphVertex* vertexp = verticesBeginp(); vertexp; vertexp = vertexp->verticesNextp()) {
-        if (vertexp->user()) {
-            // UINFO(7, "Disconnect " << vertexp->name() << endl);
-            for (V3GraphEdge* edgep = vertexp->outBeginp(); edgep; edgep = edgep->outNextp()) {
-                edgep->cut();
-            }
-        }
-    }
-
-    // Vertex::m_user end, now unused
-}
-
 //######################################################################
 //######################################################################
 // Algorithms - weakly connected components
 
-class GraphRemoveRedundant : GraphAlg<> {
+class GraphRemoveRedundant final : GraphAlg<> {
     bool m_sumWeights;  ///< Sum, rather then maximize weights
 private:
     void main() {
@@ -123,7 +84,7 @@ public:
         , m_sumWeights{sumWeights} {
         main();
     }
-    ~GraphRemoveRedundant() {}
+    ~GraphRemoveRedundant() = default;
 };
 
 void V3Graph::removeRedundantEdges(V3EdgeFuncP edgeFuncp) {
@@ -137,7 +98,7 @@ void V3Graph::removeRedundantEdgesSum(V3EdgeFuncP edgeFuncp) {
 //######################################################################
 // Algorithms - remove transitive
 
-class GraphAlgRemoveTransitiveEdges : GraphAlg<> {
+class GraphAlgRemoveTransitiveEdges final : GraphAlg<> {
 public:
     explicit GraphAlgRemoveTransitiveEdges(V3Graph* graphp)
         : GraphAlg<>(graphp, nullptr) {}
@@ -168,7 +129,7 @@ void V3Graph::removeTransitiveEdges() { GraphAlgRemoveTransitiveEdges(this).go()
 //######################################################################
 // Algorithms - weakly connected components
 
-class GraphAlgWeakly : GraphAlg<> {
+class GraphAlgWeakly final : GraphAlg<> {
 private:
     void main() {
         // Initialize state
@@ -200,7 +161,7 @@ public:
         : GraphAlg<>(graphp, edgeFuncp) {
         main();
     }
-    ~GraphAlgWeakly() {}
+    ~GraphAlgWeakly() = default;
 };
 
 void V3Graph::weaklyConnected(V3EdgeFuncP edgeFuncp) { GraphAlgWeakly(this, edgeFuncp); }
@@ -209,7 +170,7 @@ void V3Graph::weaklyConnected(V3EdgeFuncP edgeFuncp) { GraphAlgWeakly(this, edge
 //######################################################################
 // Algorithms - strongly connected components
 
-class GraphAlgStrongly : GraphAlg<> {
+class GraphAlgStrongly final : GraphAlg<> {
 private:
     uint32_t m_currentDfs;  // DFS count
     std::vector<V3GraphVertex*> m_callTrace;  // List of everything we hit processing so far
@@ -288,7 +249,7 @@ public:
         m_currentDfs = 0;
         main();
     }
-    ~GraphAlgStrongly() {}
+    ~GraphAlgStrongly() = default;
 };
 
 void V3Graph::stronglyConnected(V3EdgeFuncP edgeFuncp) { GraphAlgStrongly(this, edgeFuncp); }
@@ -297,7 +258,7 @@ void V3Graph::stronglyConnected(V3EdgeFuncP edgeFuncp) { GraphAlgStrongly(this,
 //######################################################################
 // Algorithms - ranking
 
-class GraphAlgRank : GraphAlg<> {
+class GraphAlgRank final : GraphAlg<> {
 private:
     void main() {
         // Rank each vertex, ignoring cutable edges
@@ -310,7 +271,9 @@ private:
         }
         for (V3GraphVertex* vertexp = m_graphp->verticesBeginp(); vertexp;
              vertexp = vertexp->verticesNextp()) {
-            if (!vertexp->user()) { vertexIterate(vertexp, 1); }
+            if (!vertexp->user()) {  //
+                vertexIterate(vertexp, 1);
+            }
         }
     }
 
@@ -321,7 +284,7 @@ private:
         if (vertexp->user() == 1) {
             m_graphp->reportLoops(m_edgeFuncp, vertexp);
             m_graphp->loopsMessageCb(vertexp);
-            return;
+            return;  // LCOV_EXCL_LINE  // gcc gprof bug misses this return
         }
         if (vertexp->rank() >= currentRank) return;  // Already processed it
         vertexp->user(1);
@@ -339,7 +302,7 @@ public:
         : GraphAlg<>{graphp, edgeFuncp} {
         main();
     }
-    ~GraphAlgRank() {}
+    ~GraphAlgRank() = default;
 };
 
 void V3Graph::rank() { GraphAlgRank(this, &V3GraphEdge::followAlwaysTrue); }
@@ -350,7 +313,7 @@ void V3Graph::rank(V3EdgeFuncP edgeFuncp) { GraphAlgRank(this, edgeFuncp); }
 //######################################################################
 // Algorithms - ranking
 
-class GraphAlgRLoops : GraphAlg<> {
+class GraphAlgRLoops final : GraphAlg<> {
 private:
     std::vector<V3GraphVertex*> m_callTrace;  // List of everything we hit processing so far
     bool m_done;  // Exit algorithm
@@ -393,7 +356,7 @@ public:
         m_done = false;
         main(vertexp);
     }
-    ~GraphAlgRLoops() {}
+    ~GraphAlgRLoops() = default;
 };
 
 void V3Graph::reportLoops(V3EdgeFuncP edgeFuncp, V3GraphVertex* vertexp) {
@@ -404,7 +367,7 @@ void V3Graph::reportLoops(V3EdgeFuncP edgeFuncp, V3GraphVertex* vertexp) {
 //######################################################################
 // Algorithms - subtrees
 
-class GraphAlgSubtrees : GraphAlg<> {
+class GraphAlgSubtrees final : GraphAlg<> {
 private:
     V3Graph* m_loopGraphp;
 
@@ -441,7 +404,7 @@ public:
         m_graphp->userClearEdges();
         (void)vertexIterateAll(vertexp);
     }
-    ~GraphAlgSubtrees() {}
+    ~GraphAlgSubtrees() = default;
 };
 
 //! Report the entire connected graph with a loop or loops
@@ -449,32 +412,17 @@ void V3Graph::subtreeLoops(V3EdgeFuncP edgeFuncp, V3GraphVertex* vertexp, V3Grap
     GraphAlgSubtrees(this, loopGraphp, edgeFuncp, vertexp);
 }
 
-//######################################################################
-//######################################################################
-// Algorithms - make non cutable
-
-void V3Graph::makeEdgesNonCutable(V3EdgeFuncP edgeFuncp) {
-    for (V3GraphVertex* vertexp = verticesBeginp(); vertexp; vertexp = vertexp->verticesNextp()) {
-        // Only need one direction, we'll always see the other at some point...
-        for (V3GraphEdge* edgep = vertexp->inBeginp(); edgep; edgep = edgep->inNextp()) {
-            if (edgep->cutable() && edgep->weight() && (edgeFuncp)(edgep)) {
-                edgep->cutable(false);
-            }
-        }
-    }
-}
-
 //######################################################################
 //######################################################################
 // Algorithms - sorting
 
 struct GraphSortVertexCmp {
-    inline bool operator()(const V3GraphVertex* lhsp, const V3GraphVertex* rhsp) const {
+    bool operator()(const V3GraphVertex* lhsp, const V3GraphVertex* rhsp) const {
         return lhsp->sortCmp(rhsp) < 0;
     }
 };
 struct GraphSortEdgeCmp {
-    inline bool operator()(const V3GraphEdge* lhsp, const V3GraphEdge* rhsp) const {
+    bool operator()(const V3GraphEdge* lhsp, const V3GraphEdge* rhsp) const {
         return lhsp->sortCmp(rhsp) < 0;
     }
 };
