@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2005-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2005-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -51,15 +51,15 @@ DfaVertex* DfaGraph::findStart() {
 // Algorithms - convert NFA to a DFA
 // Uses the Subset Construction Algorithm
 
-class GraphNfaToDfa : GraphAlg<> {
+class GraphNfaToDfa final : GraphAlg<> {
     // We have two types of nodes in one graph, NFA and DFA nodes.
     // Edges from NFA to NFA come from the user, and indicate input or epsilon transitions
     // Edges from DFA to NFA indicate the NFA from which that DFA was formed.
     // Edges from DFA to DFA indicate a completed input transition
 private:
     // TYPES
-    typedef std::deque<DfaVertex*> DfaStates;
-    typedef std::multimap<vluint64_t, DfaVertex*> HashMap;
+    using DfaStates = std::deque<DfaVertex*>;
+    using HashMap = std::multimap<vluint64_t, DfaVertex*>;
 
     // MEMBERS
     uint32_t m_step;  // Processing step, so we can avoid clearUser all the time
@@ -167,7 +167,7 @@ private:
     void insertDfaOrigins(DfaVertex* dfaStatep) {
         // Record the NFA states this dfa came from
         uint32_t hash = hashDfaOrigins(dfaStatep);
-        m_hashMap.insert(make_pair(hash, dfaStatep));
+        m_hashMap.emplace(hash, dfaStatep);
     }
 
     DfaVertex* findDfaOrigins(const DfaStates& nfasWithInput) {
@@ -177,7 +177,7 @@ private:
         uint32_t hash = hashDfaOrigins(nfasWithInput);
 
         const auto eqrange = m_hashMap.equal_range(hash);
-        for (HashMap::iterator it = eqrange.first; it != eqrange.second; ++it) {
+        for (auto it = eqrange.first; it != eqrange.second; ++it) {
             DfaVertex* testp = it->second;
             if (compareDfaOrigins(nfasWithInput, testp)) {
                 UINFO(9, "              DFA match for set: " << testp << endl);
@@ -294,7 +294,7 @@ private:
             UINFO(9, "  On dfaState " << dfaStatep << endl);
 
             // From this dfaState, what corresponding nfaStates have what inputs?
-            std::set<int> inputs;
+            std::unordered_set<int> inputs;
             // Foreach NFA state (this DFA state was formed from)
             for (V3GraphEdge* dfaEdgep = dfaStatep->outBeginp(); dfaEdgep;
                  dfaEdgep = dfaEdgep->outNextp()) {
@@ -368,7 +368,7 @@ public:
         m_step = 0;
         main();
     }
-    ~GraphNfaToDfa() {}
+    ~GraphNfaToDfa() = default;
 };
 
 void DfaGraph::nfaToDfa() { GraphNfaToDfa(this, &V3GraphEdge::followAlwaysTrue); }
@@ -379,7 +379,7 @@ void DfaGraph::nfaToDfa() { GraphNfaToDfa(this, &V3GraphEdge::followAlwaysTrue);
 //
 // Scan the DFA, cleaning up trailing states.
 
-class DfaGraphReduce : GraphAlg<> {
+class DfaGraphReduce final : GraphAlg<> {
 private:
     // METHODS
 #ifdef VL_CPPCHECK
@@ -445,7 +445,7 @@ private:
         for (V3GraphVertex *nextp, *vertexp = m_graphp->verticesBeginp(); vertexp;
              vertexp = nextp) {
             nextp = vertexp->verticesNextp();
-            if (!vertexp->user()) { VL_DO_DANGLING(vertexp->unlinkDelete(m_graphp), vertexp); }
+            if (!vertexp->user()) VL_DO_DANGLING(vertexp->unlinkDelete(m_graphp), vertexp);
         }
     }
 
@@ -502,7 +502,7 @@ public:
         optimize_no_outbound();
         if (debug() >= 6) m_graphp->dumpDotFilePrefixed("opt_noout");
     }
-    ~DfaGraphReduce() {}
+    ~DfaGraphReduce() = default;
 };
 
 void DfaGraph::dfaReduce() { DfaGraphReduce(this, &V3GraphEdge::followAlwaysTrue); }
@@ -527,7 +527,7 @@ void DfaGraph::dfaReduce() { DfaGraphReduce(this, &V3GraphEdge::followAlwaysTrue
 // The user's old accept is now the new accept.  This is important as
 // we want the virtual type of it to be intact.
 
-class DfaGraphComplement : GraphAlg<> {
+class DfaGraphComplement final : GraphAlg<> {
 private:
     // MEMBERS
     DfaVertex* m_tempNewerReject;
@@ -598,7 +598,7 @@ public:
         VL_DO_CLEAR(m_tempNewerReject->unlinkDelete(graphp()), m_tempNewerReject = nullptr);
         if (debug() >= 6) m_graphp->dumpDotFilePrefixed("comp_out");
     }
-    ~DfaGraphComplement() {}
+    ~DfaGraphComplement() = default;
     VL_UNCOPYABLE(DfaGraphComplement);
 };
 
