@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -44,7 +44,7 @@ static const int INLINE_MODS_SMALLER = 100;  // If a mod is < this # nodes, can
 //######################################################################
 // Inline state, as a visitor of each AstNode
 
-class InlineMarkVisitor : public AstNVisitor {
+class InlineMarkVisitor final : public AstNVisitor {
 private:
     // NODE STATE
     // Output
@@ -69,15 +69,15 @@ private:
     AstNodeModule* m_modp = nullptr;  // Current module
     VDouble0 m_statUnsup;  // Statistic tracking
 
-    typedef std::vector<AstNodeModule*> ModVec;
+    using ModVec = std::vector<AstNodeModule*>;
     ModVec m_allMods;  // All modules, in top-down order.
 
     // Within the context of a given module, LocalInstanceMap maps
     // from child modules to the count of each child's local instantiations.
-    typedef std::map<AstNodeModule*, int> LocalInstanceMap;
+    using LocalInstanceMap = std::unordered_map<AstNodeModule*, int>;
 
     // We keep a LocalInstanceMap for each module in the design
-    std::map<AstNodeModule*, LocalInstanceMap> m_instances;
+    std::unordered_map<AstNodeModule*, LocalInstanceMap> m_instances;
 
     // METHODS
     VL_DEBUG_FUNC;  // Declare debug()
@@ -108,7 +108,9 @@ private:
             // If inlining moves post-scope this can perhaps be relaxed.
             cantInline("modIface", true);
         }
-        if (m_modp->modPublic()) cantInline("modPublic", false);
+        if (m_modp->modPublic() && (m_modp->isTop() || !v3Global.opt.flatten())) {
+            cantInline("modPublic", false);
+        }
 
         iterateChildren(nodep);
         m_modp = nullptr;
@@ -137,7 +139,7 @@ private:
         } else if (nodep->pragType() == AstPragmaType::NO_INLINE_MODULE) {
             if (!m_modp) {
                 nodep->v3error("Inline pragma not under a module");  // LCOV_EXCL_LINE
-            } else {
+            } else if (!v3Global.opt.flatten()) {
                 cantInline("Pragma NO_INLINE_MODULE", false);
             }
             // Remove so don't propagate to upper cell...
@@ -153,7 +155,7 @@ private:
     virtual void visit(AstNodeFTaskRef* nodep) override {
         // Cleanup link until V3LinkDot can correct it
         // MethodCalls not currently supported by inliner, so keep linked
-        if (!nodep->packagep() && !VN_IS(nodep, MethodCall)) nodep->taskp(nullptr);
+        if (!nodep->classOrPackagep() && !VN_IS(nodep, MethodCall)) nodep->taskp(nullptr);
         iterateChildren(nodep);
     }
     virtual void visit(AstAlways* nodep) override {
@@ -233,7 +235,7 @@ public:
 // Using clonep(), find cell cross references.
 // clone() must not be called inside this visitor
 
-class InlineCollectVisitor : public AstNVisitor {
+class InlineCollectVisitor final : public AstNVisitor {
 private:
     // NODE STATE
     //  Output:
@@ -254,22 +256,20 @@ public:
     explicit InlineCollectVisitor(AstNodeModule* nodep) {  // passed OLD module, not new one
         iterate(nodep);
     }
-    virtual ~InlineCollectVisitor() override {}
+    virtual ~InlineCollectVisitor() override = default;
 };
 
 //######################################################################
 // After cell is cloned, relink the new module's contents
 
-class InlineRelinkVisitor : public AstNVisitor {
+class InlineRelinkVisitor final : public AstNVisitor {
 private:
-    typedef std::unordered_set<string> StringSet;
-
     // NODE STATE
     //  Input:
     //   See InlineVisitor
 
     // STATE
-    StringSet m_renamedInterfaces;  // Name of renamed interface variables
+    std::unordered_set<std::string> m_renamedInterfaces;  // Name of renamed interface variables
     AstNodeModule* m_modp;  // Current module
     AstCell* m_cellp;  // Cell being cloned
 
@@ -379,8 +379,8 @@ private:
         string name = m_cellp->name() + "__DOT__" + nodep->name();
         if (!nodep->isFuncLocal() && !nodep->isClassMember()) nodep->inlineAttrReset(name);
         if (!m_cellp->isTrace()) nodep->trace(false);
-        if (debug() >= 9) { nodep->dumpTree(cout, "varchanged:"); }
-        if (debug() >= 9 && nodep->valuep()) { nodep->valuep()->dumpTree(cout, "varchangei:"); }
+        if (debug() >= 9) nodep->dumpTree(cout, "varchanged:");
+        if (debug() >= 9 && nodep->valuep()) nodep->valuep()->dumpTree(cout, "varchangei:");
         iterateChildren(nodep);
     }
     virtual void visit(AstNodeFTask* nodep) override {
@@ -474,13 +474,13 @@ public:
         , m_cellp{cellp} {
         iterate(cloneModp);
     }
-    virtual ~InlineRelinkVisitor() override {}
+    virtual ~InlineRelinkVisitor() override = default;
 };
 
 //######################################################################
 // Inline state, as a visitor of each AstNode
 
-class InlineVisitor : public AstNVisitor {
+class InlineVisitor final : public AstNVisitor {
 private:
     // NODE STATE
     // Cleared entire netlist
@@ -593,7 +593,7 @@ private:
             // Cleanup var names, etc, to not conflict
             { InlineRelinkVisitor(newmodp, m_modp, nodep); }
             // Move statements to top module
-            if (debug() >= 9) { newmodp->dumpTree(cout, "fixmod:"); }
+            if (debug() >= 9) newmodp->dumpTree(cout, "fixmod:");
             AstNode* stmtsp = newmodp->stmtsp();
             if (stmtsp) stmtsp->unlinkFrBackWithNext();
             if (stmtsp) m_modp->addStmtp(stmtsp);
@@ -601,7 +601,7 @@ private:
             VL_DO_DANGLING(newmodp->deleteTree(), newmodp);  // Clear any leftover ports, etc
             nodep->unlinkFrBack();
             VL_DO_DANGLING(pushDeletep(nodep), nodep);
-            if (debug() >= 9) { m_modp->dumpTree(cout, "donemod:"); }
+            if (debug() >= 9) m_modp->dumpTree(cout, "donemod:");
         }
     }
 
@@ -614,14 +614,14 @@ public:
     // CONSTRUCTORS
     explicit InlineVisitor(AstNode* nodep) { iterate(nodep); }
     virtual ~InlineVisitor() override {  //
-        V3Stats::addStat("Optimizations, Inlined cells", m_statCells);
+        V3Stats::addStat("Optimizations, Inlined instances", m_statCells);
     }
 };
 
 //######################################################################
 // Track interface references under the Cell they reference
 
-class InlineIntfRefVisitor : public AstNVisitor {
+class InlineIntfRefVisitor final : public AstNVisitor {
 private:
     // NODE STATE
     //   AstVar::user1p()   // AstCell which this Var points to
@@ -636,7 +636,7 @@ private:
     // VISITORS
     virtual void visit(AstNetlist* nodep) override { iterateChildren(nodep); }
     virtual void visit(AstModule* nodep) override {
-        if (nodep->isTop()) { iterateChildren(nodep); }
+        if (nodep->isTop()) iterateChildren(nodep);
     }
     virtual void visit(AstCell* nodep) override {
         VL_RESTORER(m_scope);
@@ -702,7 +702,7 @@ private:
 public:
     // CONSTRUCTORS
     explicit InlineIntfRefVisitor(AstNode* nodep) { iterate(nodep); }
-    virtual ~InlineIntfRefVisitor() override {}
+    virtual ~InlineIntfRefVisitor() override = default;
 };
 
 //######################################################################
