@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -34,7 +34,7 @@
 //######################################################################
 // Inst state, as a visitor of each AstNode
 
-class InstVisitor : public AstNVisitor {
+class InstVisitor final : public AstNVisitor {
 private:
     // NODE STATE
     // Cleared each Cell:
@@ -132,17 +132,16 @@ private:
 public:
     // CONSTRUCTORS
     explicit InstVisitor(AstNetlist* nodep) { iterate(nodep); }
-    virtual ~InstVisitor() override {}
+    virtual ~InstVisitor() override = default;
 };
 
 //######################################################################
 
-class InstDeModVarVisitor : public AstNVisitor {
+class InstDeModVarVisitor final : public AstNVisitor {
     // Expand all module variables, and save names for later reference
 private:
     // STATE
-    typedef std::map<const string, AstVar*> VarNameMap;
-    VarNameMap m_modVarNameMap;  // Per module, name of cloned variables
+    std::map<const std::string, AstVar*> m_modVarNameMap;  // Per module, name of cloned variables
 
     VL_DEBUG_FUNC;  // Declare debug()
 
@@ -161,7 +160,7 @@ public:
     // METHODS
     void insert(AstVar* nodep) {
         UINFO(8, "    dmINSERT    " << nodep << endl);
-        m_modVarNameMap.insert(make_pair(nodep->name(), nodep));
+        m_modVarNameMap.emplace(nodep->name(), nodep);
     }
     AstVar* find(const string& name) {
         const auto it = m_modVarNameMap.find(name);
@@ -178,18 +177,18 @@ public:
     }
 
     // CONSTRUCTORS
-    explicit InstDeModVarVisitor() {}
+    InstDeModVarVisitor() = default;
+    virtual ~InstDeModVarVisitor() override = default;
     void main(AstNodeModule* nodep) {
         UINFO(8, "  dmMODULE    " << nodep << endl);
         m_modVarNameMap.clear();
         iterate(nodep);
     }
-    virtual ~InstDeModVarVisitor() override {}
 };
 
 //######################################################################
 
-class InstDeVisitor : public AstNVisitor {
+class InstDeVisitor final : public AstNVisitor {
     // Find all cells with arrays, and convert to non-arrayed
 private:
     // STATE
@@ -207,7 +206,7 @@ private:
             UINFO(8, "   dv-vec-VAR    " << nodep << endl);
             AstUnpackArrayDType* arrdtype = VN_CAST(nodep->dtypep(), UnpackArrayDType);
             AstNode* prevp = nullptr;
-            for (int i = arrdtype->lsb(); i <= arrdtype->msb(); ++i) {
+            for (int i = arrdtype->lo(); i <= arrdtype->hi(); ++i) {
                 string varNewName = nodep->name() + "__BRA__" + cvtToStr(i) + "__KET__";
                 UINFO(8, "VAR name insert " << varNewName << "  " << nodep << endl);
                 if (!m_deModVars.find(varNewName)) {
@@ -255,7 +254,7 @@ private:
             for (int i = 0; i < m_cellRangep->elementsConst(); i++) {
                 m_instSelNum
                     = m_cellRangep->littleEndian() ? (m_cellRangep->elementsConst() - 1 - i) : i;
-                int instNum = m_cellRangep->lsbConst() + i;
+                int instNum = m_cellRangep->loConst() + i;
 
                 AstCell* newp = nodep->cloneTree(false);
                 nodep->addNextHere(newp);
@@ -341,17 +340,17 @@ private:
                 // Arrayed instants: one bit for each of the instants (each
                 // assign is 1 pinwidth wide)
                 if (m_cellRangep->littleEndian()) {
-                    nodep->exprp()->v3warn(
-                        LITENDIAN,
-                        "Little endian cell range connecting to vector: MSB < LSB of cell range: "
-                            << m_cellRangep->lsbConst() << ":" << m_cellRangep->msbConst());
+                    nodep->exprp()->v3warn(LITENDIAN, "Little endian instance range connecting to "
+                                                      "vector: left < right of instance range: ["
+                                                          << m_cellRangep->leftConst() << ":"
+                                                          << m_cellRangep->rightConst() << "]");
                 }
                 AstNode* exprp = nodep->exprp()->unlinkFrBack();
                 bool inputPin = nodep->modVarp()->isNonOutput();
                 if (!inputPin
                     && !VN_IS(exprp, VarRef)
                     // V3Const will collapse the SEL with the one we're about to make
-                    && !VN_IS(exprp, Concat) && !VN_IS(exprp, Sel)) {
+                    && !VN_IS(exprp, Concat) && !VN_IS(exprp, Replicate) && !VN_IS(exprp, Sel)) {
                     nodep->v3warn(E_UNSUPPORTED, "Unsupported: Per-bit array instantiations "
                                                  "with output connections to non-wires.");
                     // Note spec allows more complicated matches such as slices and such
@@ -383,7 +382,7 @@ private:
                                                   varrefp->name() + "__BRA__" + index + "__KET__",
                                                   "", VAccess::WRITE);
                 newp->dtypep(nodep->modVarp()->dtypep());
-                newp->packagep(varrefp->packagep());
+                newp->classOrPackagep(varrefp->classOrPackagep());
                 arrselp->addNextHere(newp);
                 VL_DO_DANGLING(arrselp->unlinkFrBack()->deleteTree(), arrselp);
             }
@@ -396,7 +395,8 @@ private:
             AstNode* prevPinp = nullptr;
             // Clone the var referenced by the pin, and clone each var referenced by the varref
             // Clone pin varp:
-            for (int i = pinArrp->lsb(); i <= pinArrp->msb(); ++i) {
+            for (int in = 0; in < pinArrp->elementsConst(); ++in) {  // 0 = leftmost
+                int i = pinArrp->left() + in * pinArrp->declRange().leftToRightInc();
                 string varNewName = pinVarp->name() + "__BRA__" + cvtToStr(i) + "__KET__";
                 AstVar* varNewp = nullptr;
 
@@ -429,16 +429,23 @@ private:
                 newp->modVarp(varNewp);
                 newp->name(newp->name() + "__BRA__" + cvtToStr(i) + "__KET__");
                 // And replace exprp with a new varxref
-                int offset = 0;
-                const AstVarRef* varrefp = VN_CAST(newp->exprp(), VarRef);
-                if (varrefp) {
-                } else if (AstSliceSel* slicep = VN_CAST(newp->exprp(), SliceSel)) {
+                const AstVarRef* varrefp = VN_CAST(newp->exprp(), VarRef);  // Maybe null
+                int expr_i = i;
+                if (AstSliceSel* slicep = VN_CAST(newp->exprp(), SliceSel)) {
                     varrefp = VN_CAST(slicep->fromp(), VarRef);
                     UASSERT(VN_IS(slicep->rhsp(), Const), "Slices should be constant");
-                    offset = VN_CAST(slicep->rhsp(), Const)->toSInt();
+                    int slice_index
+                        = slicep->declRange().left() + in * slicep->declRange().leftToRightInc();
+                    auto* exprArrp = VN_CAST(varrefp->dtypep(), UnpackArrayDType);
+                    UASSERT_OBJ(exprArrp, slicep, "Slice of non-array");
+                    expr_i = slice_index + exprArrp->lo();
+                } else if (!varrefp) {
+                    newp->exprp()->v3error("Unexpected connection to arrayed port");
+                } else if (auto* exprArrp = VN_CAST(varrefp->dtypep(), UnpackArrayDType)) {
+                    expr_i = exprArrp->left() + in * exprArrp->declRange().leftToRightInc();
                 }
-                if (!varrefp) { newp->exprp()->v3error("Unexpected connection to arrayed port"); }
-                string newname = varrefp->name() + "__BRA__" + cvtToStr(i + offset) + "__KET__";
+
+                string newname = varrefp->name() + "__BRA__" + cvtToStr(expr_i) + "__KET__";
                 AstVarXRef* newVarXRefp
                     = new AstVarXRef(nodep->fileline(), newname, "", VAccess::WRITE);
                 newVarXRefp->varp(newp->modVarp());
@@ -466,16 +473,16 @@ private:
 public:
     // CONSTRUCTORS
     explicit InstDeVisitor(AstNetlist* nodep) { iterate(nodep); }
-    virtual ~InstDeVisitor() override {}
+    virtual ~InstDeVisitor() override = default;
 };
 
 //######################################################################
 // Inst static function
 
-class InstStatic {
+class InstStatic final {
 private:
     VL_DEBUG_FUNC;  // Declare debug()
-    InstStatic() {}  // Static class
+    InstStatic() = default;  // Static class
 
     static AstNode* extendOrSel(FileLine* fl, AstNode* rhsp, AstNode* cmpWidthp) {
         if (cmpWidthp->width() > rhsp->width()) {
@@ -529,7 +536,7 @@ public:
                    && connectXRefp->varp()->isIfaceRef()) {
         } else if (!alwaysCvt && connBasicp && pinBasicp
                    && connBasicp->width() == pinBasicp->width()
-                   && connBasicp->lsb() == pinBasicp->lsb()
+                   && connBasicp->lo() == pinBasicp->lo()
                    && !connectRefp->varp()
                            ->isSc()  // Need the signal as a 'shell' to convert types
                    && connBasicp->width() == pinVarp->width()) {
@@ -538,7 +545,7 @@ public:
             // Done. Constant.
         } else {
             // Make a new temp wire
-            // if (1 || debug() >= 9) { pinp->dumpTree(cout, "-in_pin:"); }
+            // if (1 || debug() >= 9) pinp->dumpTree(cout, "-in_pin:");
             V3Inst::checkOutputShort(pinp);
             AstNode* pinexprp = pinp->exprp()->unlinkFrBack();
             string newvarname
@@ -570,8 +577,8 @@ public:
                 pinp->exprp(new AstVarRef(pinexprp->fileline(), newvarp, VAccess::READ));
             }
             if (assignp) cellp->addNextHere(assignp);
-            // if (debug()) { pinp->dumpTree(cout, "-  out:"); }
-            // if (debug()) { assignp->dumpTree(cout, "- aout:"); }
+            // if (debug()) pinp->dumpTree(cout, "-  out:");
+            // if (debug()) assignp->dumpTree(cout, "- aout:");
         }
         return assignp;
     }
