@@ -7,7 +7,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -15,15 +15,15 @@
 //
 //*************************************************************************
 
-#ifndef _V3INSTRCOUNT_H_
-#define _V3INSTRCOUNT_H_ 1
+#ifndef VERILATOR_V3INSTRCOUNT_H_
+#define VERILATOR_V3INSTRCOUNT_H_
 
 #include "config_build.h"
 #include "verilatedos.h"
 
 class AstNode;
 
-class V3InstrCount {
+class V3InstrCount final {
 public:
     // Return the estimate count of instructions we'd incur while running
     // code in and under nodep.
