@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -14,8 +14,8 @@
 //
 //*************************************************************************
 
-#ifndef _V3LANGCODE_H_
-#define _V3LANGCODE_H_ 1
+#ifndef VERILATOR_V3LANGCODE_H_
+#define VERILATOR_V3LANGCODE_H_
 
 #include "config_build.h"
 #include "verilatedos.h"
@@ -28,7 +28,7 @@
 //! Class for the different languages supported.
 //! A separate file, since used both in V3Options (globally) and FileLine 9per
 //! file).
-class V3LangCode {
+class V3LangCode final {
 public:
     enum en : uint8_t {
         L_ERROR,  // Must be first.
