@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -41,21 +41,21 @@
 //######################################################################
 // Graph subclasses
 
-class LinkCellsGraph : public V3Graph {
+class LinkCellsGraph final : public V3Graph {
 public:
-    LinkCellsGraph() {}
-    virtual ~LinkCellsGraph() override {}
+    LinkCellsGraph() = default;
+    virtual ~LinkCellsGraph() override = default;
     virtual void loopsMessageCb(V3GraphVertex* vertexp) override;
 };
 
-class LinkCellsVertex : public V3GraphVertex {
+class LinkCellsVertex final : public V3GraphVertex {
     AstNodeModule* m_modp;
 
 public:
     LinkCellsVertex(V3Graph* graphp, AstNodeModule* modp)
         : V3GraphVertex{graphp}
         , m_modp{modp} {}
-    virtual ~LinkCellsVertex() override {}
+    virtual ~LinkCellsVertex() override = default;
     AstNodeModule* modp() const { return m_modp; }
     virtual string name() const override { return modp()->name(); }
     virtual FileLine* fileline() const override { return modp()->fileline(); }
@@ -65,11 +65,11 @@ public:
     }
 };
 
-class LibraryVertex : public V3GraphVertex {
+class LibraryVertex final : public V3GraphVertex {
 public:
     explicit LibraryVertex(V3Graph* graphp)
         : V3GraphVertex{graphp} {}
-    virtual ~LibraryVertex() override {}
+    virtual ~LibraryVertex() override = default;
     virtual string name() const override { return "*LIBRARY*"; }
 };
 
@@ -78,7 +78,7 @@ void LinkCellsGraph::loopsMessageCb(V3GraphVertex* vertexp) {
         vvertexp->modp()->v3warn(E_UNSUPPORTED,
                                  "Unsupported: Recursive multiple modules (module instantiates "
                                  "something leading back to itself): "
-                                     << vvertexp->modp()->prettyNameQ() << endl
+                                     << vvertexp->modp()->prettyNameQ() << '\n'
                                      << V3Error::warnMore()
                                      << "... note: self-recursion (module instantiating itself "
                                         "directly) is supported.");
@@ -91,7 +91,7 @@ void LinkCellsGraph::loopsMessageCb(V3GraphVertex* vertexp) {
 //######################################################################
 // Link state, as a visitor of each AstNode
 
-class LinkCellsVisitor : public AstNVisitor {
+class LinkCellsVisitor final : public AstNVisitor {
 private:
     // NODE STATE
     //  Entire netlist:
@@ -183,7 +183,7 @@ private:
                     }
                     v3error("Specified --top-module '"
                             << v3Global.opt.topModule()
-                            << "' isn't at the top level, it's under another cell '"
+                            << "' isn't at the top level, it's under another instance '"
                             << (abovep ? abovep->prettyName() : "UNKNOWN") << "'");
                 }
             }
@@ -198,7 +198,7 @@ private:
         VL_RESTORER(m_modp);
         {
             m_modp = nodep;
-            UINFO(2, "Link Module: " << nodep << endl);
+            UINFO(4, "Link Module: " << nodep << endl);
             if (nodep->fileline()->filebasenameNoExt() != nodep->prettyName()
                 && !v3Global.opt.isLibraryFile(nodep->fileline()->filename())
                 && !VN_IS(nodep, NotFoundModule) && !nodep->recursiveClone()
@@ -363,7 +363,7 @@ private:
         for (AstPin *nextp, *pinp = nodep->pinsp(); pinp; pinp = nextp) {
             nextp = VN_CAST(pinp->nextp(), Pin);
             if (pinp->dotStar()) {
-                if (pinStar) pinp->v3error("Duplicate .* in a cell");
+                if (pinStar) pinp->v3error("Duplicate .* in an instance");
                 pinStar = true;
                 // Done with this fake pin
                 VL_DO_DANGLING(pinp->unlinkFrBack()->deleteTree(), pinp);
@@ -383,7 +383,7 @@ private:
             std::unordered_set<string> ports;  // Symbol table of all connected port names
             for (AstPin* pinp = nodep->pinsp(); pinp; pinp = VN_CAST(pinp->nextp(), Pin)) {
                 if (pinp->name() == "")
-                    pinp->v3error("Connect by position is illegal in .* connected cells");
+                    pinp->v3error("Connect by position is illegal in .* connected instances");
                 if (!pinp->exprp()) {
                     if (pinp->name().substr(0, 11) == "__pinNumber") {
                         pinp->v3warn(PINNOCONNECT,
@@ -453,10 +453,19 @@ private:
                 nodep->hasIfaceVar(true);
             }
         }
-        if (nodep->modp()) { iterateChildren(nodep); }
+        if (nodep->modp()) {  //
+            iterateChildren(nodep);
+        }
         UINFO(4, " Link Cell done: " << nodep << endl);
     }
 
+    virtual void visit(AstRefDType* nodep) override {
+        for (AstPin* pinp = nodep->paramsp(); pinp; pinp = VN_CAST(pinp->nextp(), Pin)) {
+            pinp->param(true);
+            if (pinp->name() == "") pinp->name("__paramNumber" + cvtToStr(pinp->pinNum()));
+        }
+    }
+
     // Accelerate the recursion
     // Must do statements to support Generates, math though...
     virtual void visit(AstNodeMath*) override {}
@@ -482,10 +491,10 @@ private:
                       || nodep->fileline()->warnIsOff(V3ErrorCode::MODDUP)
                       || hierBlocks.find(nodep->name()) != hierBlocks.end())) {
                     nodep->v3warn(MODDUP, "Duplicate declaration of module: "
-                                              << nodep->prettyNameQ() << endl
-                                              << nodep->warnContextPrimary() << endl
+                                              << nodep->prettyNameQ() << '\n'
+                                              << nodep->warnContextPrimary() << '\n'
                                               << foundp->warnOther()
-                                              << "... Location of original declaration" << endl
+                                              << "... Location of original declaration\n"
                                               << foundp->warnContextSecondary());
                 }
                 nodep->unlinkFrBack();
@@ -517,7 +526,7 @@ public:
         }
         iterate(nodep);
     }
-    virtual ~LinkCellsVisitor() override {}
+    virtual ~LinkCellsVisitor() override = default;
 };
 
 //######################################################################
