@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -47,7 +47,7 @@
 
 //######################################################################
 
-class LinkIncVisitor : public AstNVisitor {
+class LinkIncVisitor final : public AstNVisitor {
 private:
     // TYPES
     enum InsertMode : uint8_t {
@@ -238,7 +238,7 @@ private:
 public:
     // CONSTRUCTORS
     explicit LinkIncVisitor(AstNetlist* nodep) { iterate(nodep); }
-    virtual ~LinkIncVisitor() override {}
+    virtual ~LinkIncVisitor() override = default;
 };
 
 //######################################################################
