@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -14,8 +14,8 @@
 //
 //*************************************************************************
 
-#ifndef _V3LINKLEVEL_H_
-#define _V3LINKLEVEL_H_ 1
+#ifndef VERILATOR_V3LINKLEVEL_H_
+#define VERILATOR_V3LINKLEVEL_H_
 
 #include "config_build.h"
 #include "verilatedos.h"
@@ -25,8 +25,11 @@
 
 //============================================================================
 
-class V3LinkLevel {
+class V3LinkLevel final {
 private:
+    using ModVec = std::vector<AstNodeModule*>;
+
+    static void timescaling(const ModVec& mods);
     static void wrapTopCell(AstNetlist* rootp);
     static void wrapTopPackages(AstNetlist* rootp);
 
