@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -34,7 +34,7 @@
 //######################################################################
 // Link state, as a visitor of each AstNode
 
-class LinkParseVisitor : public AstNVisitor {
+class LinkParseVisitor final : public AstNVisitor {
 private:
     // NODE STATE
     // Cleared on netlist
@@ -44,19 +44,20 @@ private:
     AstUser2InUse m_inuser2;
 
     // TYPES
-    typedef std::map<const std::pair<void*, string>, AstTypedef*> ImplTypedefMap;
-    typedef std::set<FileLine*> FileLineSet;
+    using ImplTypedefMap = std::map<const std::pair<void*, std::string>, AstTypedef*>;
 
     // STATE
     AstVar* m_varp = nullptr;  // Variable we're under
     ImplTypedefMap m_implTypedef;  // Created typedefs for each <container,name>
-    FileLineSet m_filelines;  // Filelines that have been seen
+    std::unordered_set<FileLine*> m_filelines;  // Filelines that have been seen
     bool m_inAlways = false;  // Inside an always
     AstNodeModule* m_valueModp
         = nullptr;  // If set, move AstVar->valuep() initial values to this module
     AstNodeModule* m_modp = nullptr;  // Current module
     AstNodeFTask* m_ftaskp = nullptr;  // Current task
     AstNodeDType* m_dtypep = nullptr;  // Current data type
+    int m_genblkAbove = 0;  // Begin block number of if/case/for above
+    int m_genblkNum = 0;  // Begin block number, 0=none seen
     VLifetime m_lifetime = VLifetime::STATIC;  // Propagating lifetime
 
     // METHODS
@@ -96,10 +97,11 @@ private:
     void visitIterateNodeDType(AstNodeDType* nodep) {
         if (!nodep->user1SetOnce()) {  // Process only once.
             cleanFileline(nodep);
-            AstNodeDType* upperDtypep = m_dtypep;
-            m_dtypep = nodep;
-            iterateChildren(nodep);
-            m_dtypep = upperDtypep;
+            {
+                VL_RESTORER(m_dtypep);
+                m_dtypep = nodep;
+                iterateChildren(nodep);
+            }
         }
     }
 
@@ -125,10 +127,11 @@ private:
         if (!nodep->user1SetOnce()) {  // Process only once.
             cleanFileline(nodep);
             UINFO(5, "   " << nodep << endl);
-            AstNodeModule* upperValueModp = m_valueModp;
-            m_valueModp = nullptr;
-            iterateChildren(nodep);
-            m_valueModp = upperValueModp;
+            {
+                VL_RESTORER(m_valueModp);
+                m_valueModp = nullptr;
+                iterateChildren(nodep);
+            }
         }
     }
     virtual void visit(AstNodeDType* nodep) override { visitIterateNodeDType(nodep); }
@@ -149,16 +152,17 @@ private:
         cleanFileline(nodep);
         iterateChildren(nodep);
         if (nodep->rangep()) {
-            if (!VN_IS(nodep->rangep()->msbp(), Const)  //
-                || !VN_IS(nodep->rangep()->lsbp(), Const)) {
+            if (VL_UNCOVERABLE(!VN_IS(nodep->rangep()->leftp(), Const)  // LCOV_EXCL_START
+                               || !VN_IS(nodep->rangep()->rightp(), Const))) {
+                // We check this rule in the parser, so shouldn't fire
                 nodep->v3error("Enum ranges must be integral, per spec");
-            }
-            int msb = nodep->rangep()->msbConst();
-            int lsb = nodep->rangep()->lsbConst();
-            int increment = (msb > lsb) ? -1 : 1;
+            }  // LCOV_EXCL_STOP
+            int left = nodep->rangep()->leftConst();
+            int right = nodep->rangep()->rightConst();
+            int increment = (left > right) ? -1 : 1;
             int offset_from_init = 0;
             AstNode* addp = nullptr;
-            for (int i = msb; i != (lsb + increment); i += increment, offset_from_init++) {
+            for (int i = left; i != (right + increment); i += increment, offset_from_init++) {
                 string name = nodep->name() + cvtToStr(i);
                 AstNode* valuep = nullptr;
                 if (nodep->valuep()) {
@@ -214,6 +218,8 @@ private:
         if (v3Global.opt.publicFlatRW()) {
             switch (nodep->varType()) {
             case AstVarType::VAR:  // FALLTHRU
+            case AstVarType::GPARAM:  // FALLTHRU
+            case AstVarType::LPARAM:  // FALLTHRU
             case AstVarType::PORT:  // FALLTHRU
             case AstVarType::WIRE: nodep->sigUserRWPublic(true); break;
             default: break;
@@ -359,7 +365,7 @@ private:
         // a new type won't change every verilated module.
         AstTypedef* defp = nullptr;
         ImplTypedefMap::iterator it
-            = m_implTypedef.find(make_pair(nodep->containerp(), nodep->name()));
+            = m_implTypedef.find(std::make_pair(nodep->containerp(), nodep->name()));
         if (it != m_implTypedef.end()) {
             defp = it->second;
         } else {
@@ -383,7 +389,7 @@ private:
                 defp = new AstTypedef(nodep->fileline(), nodep->name(), nullptr, VFlagChildDType(),
                                       dtypep);
                 m_implTypedef.insert(
-                    make_pair(make_pair(nodep->containerp(), defp->name()), defp));
+                    std::make_pair(std::make_pair(nodep->containerp(), defp->name()), defp));
                 backp->addNextHere(defp);
             }
         }
@@ -492,12 +498,16 @@ private:
         V3Config::applyModule(nodep);
 
         VL_RESTORER(m_modp);
+        VL_RESTORER(m_genblkAbove);
+        VL_RESTORER(m_genblkNum);
         VL_RESTORER(m_lifetime);
         {
             // Module: Create sim table for entire module and iterate
             cleanFileline(nodep);
             //
             m_modp = nodep;
+            m_genblkAbove = 0;
+            m_genblkNum = 0;
             m_valueModp = nodep;
             m_lifetime = nodep->lifetime();
             if (m_lifetime.isNone()) {
@@ -510,17 +520,17 @@ private:
     void visitIterateNoValueMod(AstNode* nodep) {
         // Iterate a node which shouldn't have any local variables moved to an Initial
         cleanFileline(nodep);
-        //
-        AstNodeModule* upperValueModp = m_valueModp;
-        m_valueModp = nullptr;
-        iterateChildren(nodep);
-        m_valueModp = upperValueModp;
+        {
+            VL_RESTORER(m_valueModp);
+            m_valueModp = nullptr;
+            iterateChildren(nodep);
+        }
     }
     virtual void visit(AstNodeProcedure* nodep) override { visitIterateNoValueMod(nodep); }
     virtual void visit(AstAlways* nodep) override {
+        VL_RESTORER(m_inAlways);
         m_inAlways = true;
         visitIterateNoValueMod(nodep);
-        m_inAlways = false;
     }
     virtual void visit(AstCover* nodep) override { visitIterateNoValueMod(nodep); }
     virtual void visit(AstRestrict* nodep) override { visitIterateNoValueMod(nodep); }
@@ -535,13 +545,45 @@ private:
                              || VN_IS(nodep->stmtsp(), GenCase))  // Has an if/case
                          && !nodep->stmtsp()->nextp());  // Has only one item
         // It's not FOR(BEGIN(...)) but we earlier changed it to BEGIN(FOR(...))
-        if (nodep->genforp() && nodep->name() == "") {
-            nodep->name("genblk");
-        } else if (nodep->generate() && nodep->name() == ""
-                   && (VN_IS(backp, CaseItem) || VN_IS(backp, GenIf)) && !nestedIf) {
-            nodep->name("genblk");
+        if (nodep->genforp()) {
+            ++m_genblkNum;
+            if (nodep->name() == "") nodep->name("genblk" + cvtToStr(m_genblkNum));
+        }
+        if (nodep->generate() && nodep->name() == ""
+            && (VN_IS(backp, CaseItem) || VN_IS(backp, GenIf)) && !nestedIf) {
+            nodep->name("genblk" + cvtToStr(m_genblkAbove));
+        }
+        if (nodep->name() != "") {
+            VL_RESTORER(m_genblkAbove);
+            VL_RESTORER(m_genblkNum);
+            m_genblkAbove = 0;
+            m_genblkNum = 0;
+            iterateChildren(nodep);
+        } else {
+            iterateChildren(nodep);
+        }
+    }
+    virtual void visit(AstGenCase* nodep) override {
+        ++m_genblkNum;
+        cleanFileline(nodep);
+        {
+            VL_RESTORER(m_genblkAbove);
+            VL_RESTORER(m_genblkNum);
+            m_genblkAbove = m_genblkNum;
+            m_genblkNum = 0;
+            iterateChildren(nodep);
+        }
+    }
+    virtual void visit(AstGenIf* nodep) override {
+        ++m_genblkNum;
+        cleanFileline(nodep);
+        {
+            VL_RESTORER(m_genblkAbove);
+            VL_RESTORER(m_genblkNum);
+            m_genblkAbove = m_genblkNum;
+            m_genblkNum = 0;
+            iterateChildren(nodep);
         }
-        iterateChildren(nodep);
     }
     virtual void visit(AstCase* nodep) override {
         V3Config::applyCase(nodep);
@@ -606,7 +648,7 @@ private:
 public:
     // CONSTRUCTORS
     explicit LinkParseVisitor(AstNetlist* rootp) { iterate(rootp); }
-    virtual ~LinkParseVisitor() override {}
+    virtual ~LinkParseVisitor() override = default;
 };
 
 //######################################################################
