@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -38,7 +38,7 @@
 //######################################################################
 // Link state, as a visitor of each AstNode
 
-class LinkResolveVisitor : public AstNVisitor {
+class LinkResolveVisitor final : public AstNVisitor {
 private:
     // NODE STATE
     //  Entire netlist:
@@ -96,8 +96,6 @@ private:
     virtual void visit(AstVar* nodep) override {
         iterateChildren(nodep);
         if (m_classp && !nodep->isParam()) nodep->varType(AstVarType::MEMBER);
-        if (m_classp && nodep->isParam())
-            nodep->v3warn(E_UNSUPPORTED, "Unsupported: class parameter");
         if (m_ftaskp) nodep->funcLocal(true);
         if (nodep->isSigModPublic()) {
             nodep->sigModPublic(false);  // We're done with this attribute
@@ -107,14 +105,23 @@ private:
 
     virtual void visit(AstNodeVarRef* nodep) override {
         // VarRef: Resolve its reference
-        if (nodep->varp()) { nodep->varp()->usedParam(true); }
+        if (nodep->varp()) nodep->varp()->usedParam(true);
         iterateChildren(nodep);
     }
 
     virtual void visit(AstNodeFTask* nodep) override {
         // NodeTask: Remember its name for later resolution
         // Remember the existing symbol table scope
-        if (m_classp) nodep->classMethod(true);
+        if (m_classp) {
+            if (nodep->name() == "pre_randomize" || nodep->name() == "post_randomize") {
+                nodep->v3warn(E_UNSUPPORTED, "Unsupported: " << nodep->prettyNameQ());
+            } else if (nodep->name() == "randomize") {
+                nodep->v3error(nodep->prettyNameQ()
+                               << " is a predefined class method; redefinition not allowed (IEEE "
+                                  "1800-2017 18.6.3)");
+            }
+            nodep->classMethod(true);
+        }
         // V3LinkDot moved the isExternDef into the class, the extern proto was
         // checked to exist, and now isn't needed
         nodep->isExternDef(false);
@@ -127,7 +134,7 @@ private:
             iterateChildren(nodep);
         }
         m_ftaskp = nullptr;
-        if (nodep->dpiExport()) { nodep->scopeNamep(new AstScopeName(nodep->fileline())); }
+        if (nodep->dpiExport()) nodep->scopeNamep(new AstScopeName(nodep->fileline()));
     }
     virtual void visit(AstNodeFTaskRef* nodep) override {
         iterateChildren(nodep);
@@ -189,7 +196,7 @@ private:
                     did = true;
                 }
                 if (AstNodePreSel* selp = VN_CAST(nodep->sensp(), NodePreSel)) {
-                    AstNode* fromp = selp->lhsp()->unlinkFrBack();
+                    AstNode* fromp = selp->fromp()->unlinkFrBack();
                     selp->replaceWith(fromp);
                     VL_DO_DANGLING(selp->deleteTree(), selp);
                     did = true;
@@ -211,7 +218,7 @@ private:
             // variable we're extracting from (to determine MSB/LSB/endianness/etc.)
             // So we replicate it in another node
             // Note that V3Param knows not to replace AstVarRef's under AstAttrOf's
-            AstNode* basefromp = AstArraySel::baseFromp(nodep);
+            AstNode* basefromp = AstArraySel::baseFromp(nodep, false);
             if (AstNodeVarRef* varrefp
                 = VN_CAST(basefromp, NodeVarRef)) {  // Maybe varxref - so need to clone
                 nodep->attrp(new AstAttrOf(nodep->fileline(), AstAttrType::VAR_BASE,
@@ -220,6 +227,9 @@ private:
                        = VN_CAST(basefromp, UnlinkedRef)) {  // Maybe unlinked - so need to clone
                 nodep->attrp(new AstAttrOf(nodep->fileline(), AstAttrType::VAR_BASE,
                                            uvxrp->cloneTree(false)));
+            } else if (auto* fromp = VN_CAST(basefromp, LambdaArgRef)) {
+                nodep->attrp(new AstAttrOf(nodep->fileline(), AstAttrType::VAR_BASE,
+                                           fromp->cloneTree(false)));
             } else if (AstMemberSel* fromp = VN_CAST(basefromp, MemberSel)) {
                 nodep->attrp(new AstAttrOf(nodep->fileline(), AstAttrType::MEMBER_BASE,
                                            fromp->cloneTree(false)));
@@ -228,11 +238,9 @@ private:
                                            fromp->cloneTree(false)));
             } else if (VN_IS(basefromp, Replicate)) {
                 // From {...}[...] syntax in IEEE 2017
-                if (basefromp) { UINFO(1, "    Related node: " << basefromp << endl); }
-                nodep->v3warn(E_UNSUPPORTED, "Unsupported: Select of concatenation");
-                nodep = nullptr;
+                if (basefromp) UINFO(1, "    Related node: " << basefromp << endl);
             } else {
-                if (basefromp) { UINFO(1, "    Related node: " << basefromp << endl); }
+                if (basefromp) UINFO(1, "    Related node: " << basefromp << endl);
                 nodep->v3fatalSrc("Illegal bit select; no signal/member being extracted from");
             }
         }
@@ -472,10 +480,10 @@ private:
                         }
                         varoutp = varp;
                         // Tie off
-                        m_modp->addStmtp(new AstAssignW(
-                            varp->fileline(),
-                            new AstVarRef(varp->fileline(), varp, VAccess::WRITE),
-                            new AstConst(varp->fileline(), AstConst::LogicFalse())));
+                        m_modp->addStmtp(
+                            new AstAssignW(varp->fileline(),
+                                           new AstVarRef(varp->fileline(), varp, VAccess::WRITE),
+                                           new AstConst(varp->fileline(), AstConst::BitFalse())));
                     } else {
                         varp->v3error("Only inputs and outputs are allowed in udp modules");
                     }
@@ -517,7 +525,7 @@ private:
 public:
     // CONSTRUCTORS
     explicit LinkResolveVisitor(AstNetlist* rootp) { iterate(rootp); }
-    virtual ~LinkResolveVisitor() override {}
+    virtual ~LinkResolveVisitor() override = default;
 };
 
 //######################################################################
@@ -525,7 +533,7 @@ public:
 //      Recurses cells backwards, so we can pick up those things that propagate
 //      from child cells up to the top module.
 
-class LinkBotupVisitor : public AstNVisitor {
+class LinkBotupVisitor final : public AstNVisitor {
 private:
     // STATE
     AstNodeModule* m_modp = nullptr;  // Current module
@@ -556,7 +564,7 @@ private:
 public:
     // CONSTRUCTORS
     explicit LinkBotupVisitor(AstNetlist* rootp) { iterate(rootp); }
-    virtual ~LinkBotupVisitor() override {}
+    virtual ~LinkBotupVisitor() override = default;
 };
 
 //######################################################################
