@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -14,8 +14,8 @@
 //
 //*************************************************************************
 
-#ifndef _V3LIST_H_
-#define _V3LIST_H_ 1
+#ifndef VERILATOR_V3LIST_H_
+#define VERILATOR_V3LIST_H_
 
 #include "config_build.h"
 #include "verilatedos.h"
@@ -36,8 +36,8 @@ private:
     friend class V3ListEnt<T>;
 
 public:
-    V3List() {}
-    ~V3List() {}
+    V3List() = default;
+    ~V3List() = default;
     // METHODS
     T begin() const { return m_headp; }
     T end() const { return nullptr; }
@@ -65,7 +65,7 @@ private:
     }
 
 public:
-    V3ListEnt() {}
+    V3ListEnt() = default;
     ~V3ListEnt() {
 #ifdef VL_DEBUG
         // Load bogus pointers so we can catch deletion bugs
