@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -30,7 +30,7 @@
 //######################################################################
 // Name state, as a visitor of each AstNode
 
-class NameVisitor : public AstNVisitor {
+class NameVisitor final : public AstNVisitor {
 private:
     // NODE STATE
     // Cleared on Netlist
@@ -136,7 +136,7 @@ private:
 public:
     // CONSTRUCTORS
     explicit NameVisitor(AstNetlist* nodep) { iterate(nodep); }
-    virtual ~NameVisitor() override {}
+    virtual ~NameVisitor() override = default;
 };
 
 //######################################################################
