@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -227,7 +227,7 @@ void V3Number::V3NumberCreate(AstNode* nodep, const char* sourcep, FileLine* fl)
                     if (product.bitsValue(width(), 4)) {  // Overflowed
                         static int warned = 0;
                         v3error("Too many digits for "
-                                << width() << " bit number: " << sourcep << std::endl
+                                << width() << " bit number: " << sourcep << '\n'
                                 << ((!m_sized && !warned++) ? (
                                         V3Error::warnMore() + "... As that number was unsized"
                                         + " ('d...) it is limited to 32 bits (IEEE 1800-2017 "
@@ -401,7 +401,7 @@ V3Number& V3Number::setLongS(vlsint32_t value) {
     return *this;
 }
 V3Number& V3Number::setDouble(double value) {
-    if (VL_UNCOVERABLE(width() != 64)) { v3fatalSrc("Real operation on wrong sized number"); }
+    if (VL_UNCOVERABLE(width() != 64)) v3fatalSrc("Real operation on wrong sized number");
     m_double = true;
     union {
         double d;
@@ -495,6 +495,7 @@ string V3Number::ascii(bool prefixed, bool cleanVerilog) const {
             out << "'";
             if (bitIs0(0)) {
                 out << '0';
+                if (isNull()) out << "[null]";
             } else if (bitIs1(0)) {
                 out << '1';
             } else if (bitIsZ(0)) {
@@ -515,7 +516,8 @@ string V3Number::ascii(bool prefixed, bool cleanVerilog) const {
 
     bool binary = (isFourState()
 #ifdef V3NUMBER_ASCII_BINARY
-                   || 1
+                   // cppcheck-suppress konwnConditionTrueFalse
+                   || true
 #endif
     );
     // out<<"-"<<hex<<m_value[0]<<"-";
@@ -529,6 +531,7 @@ string V3Number::ascii(bool prefixed, bool cleanVerilog) const {
         // Always deal with 4 bits at once.  Note no 4-state, it's above.
         out << displayed("%0h");
     }
+    if (isNull() && VL_UNCOVERABLE(!isEqZero())) out << "-%E-null-not-zero";
     return out.str();
 }
 
@@ -693,7 +696,7 @@ string V3Number::displayed(FileLine* fl, const string& vformat) const {
     case 'g':
     case '^': {  // Realtime
         char tmp[MAX_SPRINTF_DOUBLE_SIZE];
-        sprintf(tmp, vformat.c_str(), toDouble());
+        VL_SNPRINTF(tmp, MAX_SPRINTF_DOUBLE_SIZE, vformat.c_str(), toDouble());
         return tmp;
     }
     // 'l'   // Library - converted to text by V3LinkResolve
@@ -937,20 +940,21 @@ bool V3Number::isAnyX() const {
     }
     return false;
 }
-bool V3Number::isAnyXZ() const {
+bool V3Number::isAnyXZ() const { return isAnyX() || isAnyZ(); }
+bool V3Number::isAnyZ() const {
     if (isDouble() || isString()) return false;
     for (int bit = 0; bit < width(); bit++) {
-        if (bitIsX(bit) || bitIsZ(bit)) return true;
+        if (bitIsZ(bit)) return true;
     }
     return false;
 }
 bool V3Number::isLtXZ(const V3Number& rhs) const {
     // Include X/Z in comparisons for sort ordering
     for (int bit = 0; bit < std::max(this->width(), rhs.width()); bit++) {
-        if (this->bitIs1(bit) && rhs.bitIs0(bit)) { return true; }
-        if (rhs.bitIs1(bit) && this->bitIs0(bit)) { return false; }
-        if (this->bitIsXZ(bit)) { return true; }
-        if (rhs.bitIsXZ(bit)) { return false; }
+        if (this->bitIs1(bit) && rhs.bitIs0(bit)) return true;
+        if (rhs.bitIs1(bit) && this->bitIs0(bit)) return false;
+        if (this->bitIsXZ(bit)) return true;
+        if (rhs.bitIsXZ(bit)) return false;
     }
     return false;
 }
@@ -1115,26 +1119,6 @@ V3Number& V3Number::opRedXor(const V3Number& lhs) {
     return setSingleBits(outc);
 }
 
-V3Number& V3Number::opRedXnor(const V3Number& lhs) {
-    // op i, 1 bit return
-    NUM_ASSERT_OP_ARGS1(lhs);
-    NUM_ASSERT_LOGIC_ARGS1(lhs);
-    char outc = 1;
-    for (int bit = 0; bit < lhs.width(); bit++) {
-        if (lhs.bitIs1(bit)) {
-            if (outc == 1) {
-                outc = 0;
-            } else if (outc == 0) {
-                outc = 1;
-            }
-        } else if (lhs.bitIs0(bit)) {
-        } else {
-            outc = 'x';
-        }
-    }
-    return setSingleBits(outc);
-}
-
 V3Number& V3Number::opCountBits(const V3Number& expr, const V3Number& ctrl1, const V3Number& ctrl2,
                                 const V3Number& ctrl3) {
     NUM_ASSERT_OP_ARGS4(expr, ctrl1, ctrl2, ctrl3);
@@ -1275,24 +1259,6 @@ V3Number& V3Number::opXor(const V3Number& lhs, const V3Number& rhs) {
     return *this;
 }
 
-V3Number& V3Number::opXnor(const V3Number& lhs, const V3Number& rhs) {
-    // i op j, max(L(lhs),L(rhs)) bit return, careful need to X/Z extend.
-    NUM_ASSERT_OP_ARGS2(lhs, rhs);
-    NUM_ASSERT_LOGIC_ARGS2(lhs, rhs);
-    setZero();
-    for (int bit = 0; bit < this->width(); bit++) {
-        if (lhs.bitIs1(bit) && rhs.bitIs1(bit)) {
-            setBit(bit, 1);
-        } else if (lhs.bitIs0(bit) && rhs.bitIs0(bit)) {
-            setBit(bit, 1);
-        } else if (lhs.bitIsXZ(bit) && rhs.bitIsXZ(bit)) {
-            setBit(bit, 'x');
-        }
-        // else zero
-    }
-    return *this;
-}
-
 V3Number& V3Number::opConcat(const V3Number& lhs, const V3Number& rhs) {
     NUM_ASSERT_OP_ARGS2(lhs, rhs);
     NUM_ASSERT_LOGIC_ARGS2(lhs, rhs);
@@ -1381,14 +1347,14 @@ V3Number& V3Number::opLogAnd(const V3Number& lhs, const V3Number& rhs) {
             loutc = 1;
             break;
         }
-        if (lhs.bitIsXZ(bit) && loutc == 0) { loutc = 'x'; }
+        if (lhs.bitIsXZ(bit) && loutc == 0) loutc = 'x';
     }
     for (int bit = 0; bit < rhs.width(); bit++) {
         if (rhs.bitIs1(bit)) {
             routc = 1;
             break;
         }
-        if (rhs.bitIsXZ(bit) && routc == 0) { routc = 'x'; }
+        if (rhs.bitIsXZ(bit) && routc == 0) routc = 'x';
     }
     char outc = 'x';
     if (routc == 1 && loutc == 1) outc = 1;
@@ -1562,7 +1528,7 @@ bool V3Number::isCaseEq(const V3Number& rhs) const {
     if (this->width() != rhs.width()) return false;
 
     for (int bit = 0; bit < std::max(this->width(), rhs.width()); bit++) {
-        if (this->bitIs(bit) != rhs.bitIs(bit)) { return false; }
+        if (this->bitIs(bit) != rhs.bitIs(bit)) return false;
     }
     return true;
 }
@@ -1684,34 +1650,6 @@ V3Number& V3Number::opLte(const V3Number& lhs, const V3Number& rhs) { return opG
 V3Number& V3Number::opLtS(const V3Number& lhs, const V3Number& rhs) { return opGtS(rhs, lhs); }
 V3Number& V3Number::opLteS(const V3Number& lhs, const V3Number& rhs) { return opGteS(rhs, lhs); }
 
-V3Number& V3Number::opRotR(const V3Number& lhs, const V3Number& rhs) {
-    // L(lhs) bit return
-    NUM_ASSERT_OP_ARGS2(lhs, rhs);
-    NUM_ASSERT_LOGIC_ARGS2(lhs, rhs);
-    if (rhs.isFourState()) return setAllBitsX();
-    setZero();
-    uint32_t rhsval = rhs.toUInt();
-    for (int bit = 0; bit < this->width(); bit++) {
-        setBit(bit, lhs.bitIs((bit + rhsval) % this->width()));
-    }
-    return *this;
-}
-
-V3Number& V3Number::opRotL(const V3Number& lhs, const V3Number& rhs) {
-    // L(lhs) bit return
-    NUM_ASSERT_OP_ARGS2(lhs, rhs);
-    NUM_ASSERT_LOGIC_ARGS2(lhs, rhs);
-    if (rhs.isFourState()) return setAllBitsX();
-    setZero();
-    uint32_t rhsval = rhs.toUInt();
-    for (int bit = 0; bit < this->width(); bit++) {
-        if (bit >= static_cast<int>(rhsval)) {
-            setBit(bit, lhs.bitIs((bit - rhsval) % this->width()));
-        }
-    }
-    return *this;
-}
-
 V3Number& V3Number::opShiftR(const V3Number& lhs, const V3Number& rhs) {
     // L(lhs) bit return
     NUM_ASSERT_OP_ARGS2(lhs, rhs);
@@ -1775,16 +1713,6 @@ V3Number& V3Number::opShiftL(const V3Number& lhs, const V3Number& rhs) {
 //======================================================================
 // Ops - Arithmetic
 
-V3Number& V3Number::opAbsS(const V3Number& lhs) {
-    // op i, L(lhs) bit return
-    NUM_ASSERT_OP_ARGS1(lhs);
-    if (lhs.isFourState()) return setAllBitsX();
-    if (lhs.isNegative()) {
-        return opNegate(lhs);
-    } else {
-        return opAssign(lhs);
-    }
-}
 V3Number& V3Number::opNegate(const V3Number& lhs) {
     // op i, L(lhs) bit return
     NUM_ASSERT_OP_ARGS1(lhs);
