@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -113,7 +113,7 @@ static bool domainsExclusive(const AstSenTree* fromp, const AstSenTree* top);
 // Functions for above graph classes
 
 void OrderGraph::loopsVertexCb(V3GraphVertex* vertexp) {
-    if (debug()) cout << "-Info-Loop: " << vertexp << " " << endl;
+    if (debug()) cout << "-Info-Loop: " << vertexp << "\n";
     if (OrderLogicVertex* vvertexp = dynamic_cast<OrderLogicVertex*>(vertexp)) {
         std::cerr << vvertexp->nodep()->fileline()->warnOther()
                   << "     Example path: " << vvertexp->nodep()->typeName() << endl;
@@ -126,7 +126,7 @@ void OrderGraph::loopsVertexCb(V3GraphVertex* vertexp) {
 
 //######################################################################
 
-class OrderMoveDomScope {
+class OrderMoveDomScope final {
     // Information stored for each unique loop, domain & scope trifecta
 public:
     V3ListEnt<OrderMoveDomScope*> m_readyDomScopeE;  // List of next ready dom scope
@@ -136,8 +136,8 @@ private:
     const AstSenTree* m_domainp;  // Domain all vertices belong to
     const AstScope* m_scopep;  // Scope all vertices belong to
 
-    typedef std::pair<const AstSenTree*, const AstScope*> DomScopeKey;
-    typedef std::map<DomScopeKey, OrderMoveDomScope*> DomScopeMap;
+    using DomScopeKey = std::pair<const AstSenTree*, const AstScope*>;
+    using DomScopeMap = std::map<DomScopeKey, OrderMoveDomScope*>;
     static DomScopeMap s_dsMap;  // Structure registered for each dom/scope pairing
 
 public:
@@ -158,13 +158,13 @@ public:
     }
     V3List<OrderMoveVertex*>& readyVertices() { return m_readyVertices; }
     static OrderMoveDomScope* findCreate(const AstSenTree* domainp, const AstScope* scopep) {
-        const DomScopeKey key = make_pair(domainp, scopep);
+        const DomScopeKey key = std::make_pair(domainp, scopep);
         const auto iter = s_dsMap.find(key);
         if (iter != s_dsMap.end()) {
             return iter->second;
         } else {
             OrderMoveDomScope* domScopep = new OrderMoveDomScope(domainp, scopep);
-            s_dsMap.insert(make_pair(key, domScopep));
+            s_dsMap.emplace(key, domScopep);
             return domScopep;
         }
     }
@@ -184,13 +184,13 @@ inline std::ostream& operator<<(std::ostream& lhs, const OrderMoveDomScope& rhs)
 // Order information stored under each AstNode::user1p()...
 
 // Types of vertex we can create
-enum WhichVertex : uint8_t { WV_STD, WV_PRE, WV_PORD, WV_POST, WV_SETL, WV_MAX };
+enum WhichVertex : uint8_t { WV_STD, WV_PRE, WV_PORD, WV_POST, WV_MAX };
 
-class OrderUser {
+class OrderUser final {
     // Stored in AstVarScope::user1p, a list of all the various vertices
     // that can exist for one given variable
 private:
-    OrderVarVertex* m_vertexp[WV_MAX];  // Vertex of each type (if non nullptr)
+    std::array<OrderVarVertex*, WV_MAX> m_vertexp;  // Vertex of each type (if non nullptr)
 public:
     // METHODS
     OrderVarVertex* newVarUserVertex(V3Graph* graphp, AstScope* scopep, AstVarScope* varscp,
@@ -205,7 +205,6 @@ public:
             case WV_PRE: vertexp = new OrderVarPreVertex(graphp, scopep, varscp); break;
             case WV_PORD: vertexp = new OrderVarPordVertex(graphp, scopep, varscp); break;
             case WV_POST: vertexp = new OrderVarPostVertex(graphp, scopep, varscp); break;
-            case WV_SETL: vertexp = new OrderVarSettleVertex(graphp, scopep, varscp); break;
             default: varscp->v3fatalSrc("Bad case");
             }
             m_vertexp[type] = vertexp;
@@ -219,7 +218,7 @@ public:
     OrderUser() {
         for (auto& vertexp : m_vertexp) vertexp = nullptr;
     }
-    ~OrderUser() {}
+    ~OrderUser() = default;
 };
 
 //######################################################################
@@ -252,7 +251,7 @@ struct OrderVarFanoutCmp {
 // In addition it also check whether clock and data signals are mixed, and
 // produce a CLKDATA warning if so.
 //
-class OrderClkMarkVisitor : public AstNVisitor {
+class OrderClkMarkVisitor final : public AstNVisitor {
 private:
     bool m_hasClk = false;  // flag indicating whether there is clock signal on rhs
     bool m_inClocked = false;  // Currently inside a sequential block
@@ -273,7 +272,7 @@ private:
                 if (m_inClocked) {
                     varrefp->v3warn(
                         CLKDATA, "Clock used as data (on rhs of assignment) in sequential block "
-                                     << varrefp->prettyNameQ() << endl);
+                                     << varrefp->prettyNameQ());
                 } else {
                     m_hasClk = true;
                     UINFO(5, "node is already marked as clocker " << varrefp << endl);
@@ -290,8 +289,8 @@ private:
         // do the marking
         if (m_hasClk) {
             if (nodep->lhsp()->width() > m_rightClkWidth) {
-                nodep->v3warn(CLKDATA, "Clock is assigned to part of data signal " << nodep->lhsp()
-                                                                                   << endl);
+                nodep->v3warn(CLKDATA, "Clock is assigned to part of data signal "
+                                           << nodep->lhsp()->prettyNameQ());
                 UINFO(4, "CLKDATA: lhs with width " << nodep->lhsp()->width() << endl);
                 UINFO(4, "     but rhs clock with width " << m_rightClkWidth << endl);
                 return;  // skip the marking
@@ -366,13 +365,13 @@ public:
             iterate(nodep);
         } while (m_newClkMarked);
     }
-    virtual ~OrderClkMarkVisitor() override {}
+    virtual ~OrderClkMarkVisitor() override = default;
 };
 
 //######################################################################
 // The class checks if the assignment generates a clock.
 
-class OrderClkAssVisitor : public AstNVisitor {
+class OrderClkAssVisitor final : public AstNVisitor {
 private:
     bool m_clkAss = false;  // There is signals marked as clocker in the assignment
     // METHODS
@@ -397,7 +396,7 @@ private:
 public:
     // CONSTRUCTORS
     explicit OrderClkAssVisitor(AstNode* nodep) { iterate(nodep); }
-    virtual ~OrderClkAssVisitor() override {}
+    virtual ~OrderClkAssVisitor() override = default;
     // METHODS
     bool isClkAss() const { return m_clkAss; }
 };
@@ -422,14 +421,11 @@ template <class T_MoveVertex> class ProcessMoveBuildGraph {
     //   works the same way for both cases.
 
     // TYPES
-    typedef std::pair<const V3GraphVertex*, const AstSenTree*> VxDomPair;
-    // Maps an (original graph vertex, domain) pair to a T_MoveVertex
-    // Not std::unordered_map, because std::pair doesn't provide std::hash
-    typedef std::map<VxDomPair, T_MoveVertex*> Var2Move;
-    typedef std::unordered_map<const OrderLogicVertex*, T_MoveVertex*> Logic2Move;
+    using VxDomPair = std::pair<const V3GraphVertex*, const AstSenTree*>;
+    using Logic2Move = std::unordered_map<const OrderLogicVertex*, T_MoveVertex*>;
 
 public:
-    class MoveVertexMaker {
+    class MoveVertexMaker VL_NOT_FINAL {
     public:
         // Clients of ProcessMoveBuildGraph must supply MoveVertexMaker
         // which creates new T_MoveVertex's. Each new vertex wraps lvertexp
@@ -447,7 +443,9 @@ private:
     V3Graph* m_outGraphp;  // Output graph of T_MoveVertex's
     MoveVertexMaker* m_vxMakerp;  // Factory class for T_MoveVertex's
     Logic2Move m_logic2move;  // Map Logic to Vertex
-    Var2Move m_var2move;  // Map Vars to Vertex
+    // Maps an (original graph vertex, domain) pair to a T_MoveVertex
+    // Not std::unordered_map, because std::pair doesn't provide std::hash
+    std::map<VxDomPair, T_MoveVertex*> m_var2move;
 
 public:
     // CONSTRUCTORS
@@ -457,7 +455,7 @@ public:
         : m_graphp{logicGraphp}
         , m_outGraphp{outGraphp}
         , m_vxMakerp{vxMakerp} {}
-    virtual ~ProcessMoveBuildGraph() {}
+    virtual ~ProcessMoveBuildGraph() = default;
 
     // METHODS
     void build() {
@@ -488,7 +486,7 @@ public:
         for (V3GraphVertex* itp = m_graphp->verticesBeginp(); itp; itp = itp->verticesNextp()) {
             if (OrderLogicVertex* lvertexp = dynamic_cast<OrderLogicVertex*>(itp)) {
                 T_MoveVertex* moveVxp = m_logic2move[lvertexp];
-                if (moveVxp) { iterate(moveVxp, lvertexp, lvertexp->domainp()); }
+                if (moveVxp) iterate(moveVxp, lvertexp, lvertexp->domainp());
             }
         }
     }
@@ -551,7 +549,7 @@ private:
 //######################################################################
 // OrderMoveVertexMaker and related
 
-class OrderMoveVertexMaker : public ProcessMoveBuildGraph<OrderMoveVertex>::MoveVertexMaker {
+class OrderMoveVertexMaker final : public ProcessMoveBuildGraph<OrderMoveVertex>::MoveVertexMaker {
     // MEMBERS
     V3Graph* m_pomGraphp;
     V3List<OrderMoveVertex*>* m_pomWaitingp;
@@ -579,7 +577,8 @@ private:
     VL_UNCOPYABLE(OrderMoveVertexMaker);
 };
 
-class OrderMTaskMoveVertexMaker : public ProcessMoveBuildGraph<MTaskMoveVertex>::MoveVertexMaker {
+class OrderMTaskMoveVertexMaker final
+    : public ProcessMoveBuildGraph<MTaskMoveVertex>::MoveVertexMaker {
     V3Graph* m_pomGraphp;
 
 public:
@@ -602,7 +601,7 @@ private:
     VL_UNCOPYABLE(OrderMTaskMoveVertexMaker);
 };
 
-class OrderVerticesByDomainThenScope {
+class OrderVerticesByDomainThenScope final {
     PartPtrIdMap m_ids;
 
 public:
@@ -619,10 +618,10 @@ public:
     }
 };
 
-class MTaskVxIdLessThan {
+class MTaskVxIdLessThan final {
 public:
-    MTaskVxIdLessThan() {}
-    virtual ~MTaskVxIdLessThan() {}
+    MTaskVxIdLessThan() = default;
+    virtual ~MTaskVxIdLessThan() = default;
 
     // Sort vertex's, which must be AbstractMTask's, into a deterministic
     // order by comparing their serial IDs.
@@ -636,7 +635,7 @@ public:
 //######################################################################
 // Order class functions
 
-class OrderVisitor : public AstNVisitor {
+class OrderVisitor final : public AstNVisitor {
 private:
     // NODE STATE
     // Forming graph:
@@ -672,6 +671,7 @@ private:
     bool m_inClkAss = false;  // Underneath AstAssign
     bool m_inPre = false;  // Underneath AstAssignPre
     bool m_inPost = false;  // Underneath AstAssignPost
+    bool m_inPostponed = false;  // Underneath AstAssignPostponed
     OrderLogicVertex* m_activeSenVxp = nullptr;  // Sensitivity vertex
     std::deque<OrderUser*> m_orderUserps;  // All created OrderUser's for later deletion.
     // STATE... for inside process
@@ -686,7 +686,7 @@ protected:
 
 private:
     // STATS
-    VDouble0 m_statCut[OrderVEdgeType::_ENUM_END];  // Count of each edge type cut
+    std::array<VDouble0, OrderVEdgeType::_ENUM_END> m_statCut;  // Count of each edge type cut
 
     // TYPES
     enum VarUsage : uint8_t { VU_NONE = 0, VU_CON = 1, VU_GEN = 2 };
@@ -729,7 +729,7 @@ private:
 
     void process();
     void processCircular();
-    typedef std::deque<OrderEitherVertex*> VertexVec;
+    using VertexVec = std::deque<OrderEitherVertex*>;
     void processInputs();
     void processInputsInIterate(OrderEitherVertex* vertexp, VertexVec& todoVec);
     void processInputsOutIterate(OrderEitherVertex* vertexp, VertexVec& todoVec);
@@ -751,14 +751,13 @@ private:
 
     // processMTask* routines schedule threaded execution
     struct MTaskState {
-        typedef std::list<const OrderLogicVertex*> Logics;
         AstMTaskBody* m_mtaskBodyp = nullptr;
-        Logics m_logics;
+        std::list<const OrderLogicVertex*> m_logics;
         ExecMTask* m_execMTaskp = nullptr;
-        MTaskState() {}
+        MTaskState() = default;
     };
     void processMTasks();
-    typedef enum : uint8_t { LOGIC_INITIAL, LOGIC_SETTLE } InitialLogicE;
+    enum InitialLogicE : uint8_t { LOGIC_INITIAL, LOGIC_SETTLE };
     void processMTasksInitial(InitialLogicE logic_type);
 
     string cfuncName(AstNodeModule* modp, AstSenTree* domainp, AstScope* scopep,
@@ -870,7 +869,7 @@ private:
         m_graph.userClearVertices();
         // May be very large vector, so only report the "most important"
         // elements. Up to 10 of the widest
-        std::cerr << V3Error::warnMore() << "... Widest candidate vars to split:" << endl;
+        std::cerr << V3Error::warnMore() << "... Widest candidate vars to split:\n";
         std::stable_sort(m_unoptflatVars.begin(), m_unoptflatVars.end(), OrderVarWidthCmp());
         std::unordered_set<const AstVar*> canSplitList;
         int lim = m_unoptflatVars.size() < 10 ? m_unoptflatVars.size() : 10;
@@ -885,10 +884,10 @@ private:
                 std::cerr << ", can split_var";
                 canSplitList.insert(varp);
             }
-            std::cerr << std::endl;
+            std::cerr << '\n';
         }
         // Up to 10 of the most fanned out
-        std::cerr << V3Error::warnMore() << "... Most fanned out candidate vars to split:" << endl;
+        std::cerr << V3Error::warnMore() << "... Most fanned out candidate vars to split:\n";
         std::stable_sort(m_unoptflatVars.begin(), m_unoptflatVars.end(), OrderVarFanoutCmp());
         lim = m_unoptflatVars.size() < 10 ? m_unoptflatVars.size() : 10;
         for (int i = 0; i < lim; i++) {
@@ -902,7 +901,7 @@ private:
                 std::cerr << ", can split_var";
                 canSplitList.insert(varp);
             }
-            std::cerr << endl;
+            std::cerr << '\n';
         }
         if (!canSplitList.empty()) {
             std::cerr << V3Error::warnMore()
@@ -1043,7 +1042,8 @@ private:
                 // We don't want to add extra edges if the logic block has many usages of same var
                 bool gen = false;
                 bool con = false;
-                if (nodep->access().isWriteOrRW()) gen = !(varscp->user4() & VU_GEN);
+                if (nodep->access().isWriteOrRW() && !m_inPostponed)
+                    gen = !(varscp->user4() & VU_GEN);
                 if (nodep->access().isReadOrRW()) {
                     con = !(varscp->user4() & VU_CON);
                     if ((varscp->user4() & VU_GEN) && !m_inClocked) {
@@ -1173,6 +1173,11 @@ private:
         iterateNewStmt(nodep);
         m_inPost = false;
     }
+    virtual void visit(AstAlwaysPostponed* nodep) override {
+        VL_RESTORER(m_inPostponed);
+        m_inPostponed = true;
+        iterateNewStmt(nodep);
+    }
     virtual void visit(AstAlways* nodep) override { iterateNewStmt(nodep); }
     virtual void visit(AstAlwaysPublic* nodep) override { iterateNewStmt(nodep); }
     virtual void visit(AstAssignAlias* nodep) override { iterateNewStmt(nodep); }
@@ -1561,8 +1566,6 @@ void OrderVisitor::processEdgeReport() {
                 name += " {POST}";
             } else if (dynamic_cast<OrderVarPordVertex*>(itp)) {
                 name += " {PORD}";
-            } else if (dynamic_cast<OrderVarSettleVertex*>(itp)) {
-                name += " {STL}";
             }
             std::ostringstream os;
             os.setf(std::ios::left);
@@ -1573,9 +1576,9 @@ void OrderVisitor::processEdgeReport() {
         }
     }
 
-    *logp << "Signals and their clock domains:" << endl;
+    *logp << "Signals and their clock domains:\n";
     stable_sort(report.begin(), report.end());
-    for (const string& i : report) *logp << i << endl;
+    for (const string& i : report) *logp << i << '\n';
 }
 
 void OrderVisitor::processMoveClear() {
@@ -1656,7 +1659,7 @@ void OrderVisitor::processMovePrepReady() {
     UINFO(5, "  MovePrepReady\n");
     for (OrderMoveVertex* vertexp = m_pomWaiting.begin(); vertexp;) {
         OrderMoveVertex* nextp = vertexp->pomWaitingNextp();
-        if (vertexp->isWait() && vertexp->inEmpty()) { processMoveReadyOne(vertexp); }
+        if (vertexp->isWait() && vertexp->inEmpty()) processMoveReadyOne(vertexp);
         vertexp = nextp;
     }
 }
