@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -53,8 +53,10 @@
 # ifndef WIFEXITED
 #  ifdef __MINGW32__
 #   define WIFEXITED(w)	(((w) & 0xC0000000) == 0)
+#   define WEXITSTATUS(w)	((w) & ~0xC0000000)
 #  else
 #   define WIFEXITED(w)	(((w) & 0377) == 0)
+#   define WEXITSTATUS(w)	(((w) >> 8) & 0377)
 #  endif
 # endif
 #else
@@ -252,12 +254,12 @@ void V3Os::unlinkRegexp(const string& dir, const string& regexp) {
 //######################################################################
 // METHODS (random)
 
-vluint64_t V3Os::rand64(vluint64_t* statep) {
+vluint64_t V3Os::rand64(std::array<vluint64_t, 2>& stater) {
     // Xoroshiro128+ algorithm
-    vluint64_t result = statep[0] + statep[1];
-    statep[1] ^= statep[0];
-    statep[0] = (((statep[0] << 55) | (statep[0] >> 9)) ^ statep[1] ^ (statep[1] << 14));
-    statep[1] = (statep[1] << 36) | (statep[1] >> 28);
+    vluint64_t result = stater[0] + stater[1];
+    stater[1] ^= stater[0];
+    stater[0] = (((stater[0] << 55) | (stater[0] >> 9)) ^ stater[1] ^ (stater[1] << 14));
+    stater[1] = (stater[1] << 36) | (stater[1] >> 28);
     return result;
 }
 
@@ -269,7 +271,7 @@ string V3Os::trueRandom(size_t size) {
 #if defined(_WIN32) || defined(__MINGW32__)
     NTSTATUS hr = BCryptGenRandom(nullptr, reinterpret_cast<BYTE*>(data), size,
                                   BCRYPT_USE_SYSTEM_PREFERRED_RNG);
-    if (!BCRYPT_SUCCESS(hr)) { v3fatal("Could not acquire random data."); }
+    if (!BCRYPT_SUCCESS(hr)) v3fatal("Could not acquire random data.");
 #else
     std::ifstream is("/dev/urandom", std::ios::in | std::ios::binary);
     // This read uses the size of the buffer.
