@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2009-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2009-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -14,8 +14,8 @@
 //
 //*************************************************************************
 
-#ifndef _V3PARSEIMP_H_
-#define _V3PARSEIMP_H_ 1
+#ifndef VERILATOR_V3PARSEIMP_H_
+#define VERILATOR_V3PARSEIMP_H_
 
 #include "config_build.h"
 #include "verilatedos.h"
@@ -37,9 +37,9 @@ class V3Lexer;
 //======================================================================
 // Types (between parser & lexer)
 
-typedef enum : uint8_t { uniq_NONE, uniq_UNIQUE, uniq_UNIQUE0, uniq_PRIORITY } V3UniqState;
+enum V3UniqState : uint8_t { uniq_NONE, uniq_UNIQUE, uniq_UNIQUE0, uniq_PRIORITY };
 
-typedef enum : uint8_t { iprop_NONE, iprop_CONTEXT, iprop_PURE } V3ImportProperty;
+enum V3ImportProperty : uint8_t { iprop_NONE, iprop_CONTEXT, iprop_PURE };
 
 //============================================================================
 // Member qualifiers
@@ -50,7 +50,7 @@ struct VMemberQualifiers {
         struct {
             uint32_t m_local : 1;  // Local class item (ignored until warning implemented)
             uint32_t m_protected : 1;  // Protected class item (ignored until warning implemented)
-            uint32_t m_rand : 1;  // Rand property/member qualifier (ignored until supported)
+            uint32_t m_rand : 1;  // Rand property/member qualifier
             uint32_t m_randc : 1;  // Randc property/member qualifier (ignored until supported)
             uint32_t m_virtual : 1;  // Virtual property/method qualifier
             uint32_t m_automatic : 1;  // Automatic property/method qualifier
@@ -70,8 +70,8 @@ struct VMemberQualifiers {
     }
     void applyToNodes(AstNodeFTask* nodesp) const {
         for (AstNodeFTask* nodep = nodesp; nodep; nodep = VN_CAST(nodep->nextp(), NodeFTask)) {
-            // Ignored for now: m_local
-            // Ignored for now: m_protected
+            if (m_local) nodep->isHideLocal(true);
+            if (m_protected) nodep->isHideProtected(true);
             if (m_virtual) nodep->isVirtual(true);
             if (m_automatic) nodep->lifetime(VLifetime::AUTOMATIC);
             if (m_static) nodep->lifetime(VLifetime::STATIC);
@@ -83,10 +83,13 @@ struct VMemberQualifiers {
     }
     void applyToNodes(AstVar* nodesp) const {
         for (AstVar* nodep = nodesp; nodep; nodep = VN_CAST(nodep->nextp(), Var)) {
-            // Ignored for now: m_local
-            // Ignored for now: m_protected
-            // Ignored for now: m_rand
-            // Ignored for now: m_randc
+            if (m_randc) {
+                nodep->v3warn(RANDC, "Unsupported: Converting 'randc' to 'rand'");
+                nodep->isRand(true);
+            }
+            if (m_rand) nodep->isRand(true);
+            if (m_local) nodep->isHideLocal(true);
+            if (m_protected) nodep->isHideProtected(true);
             if (m_automatic) nodep->lifetime(VLifetime::AUTOMATIC);
             if (m_static) nodep->lifetime(VLifetime::STATIC);
             if (m_const) nodep->isConst(true);
@@ -156,7 +159,7 @@ std::ostream& operator<<(std::ostream& os, const V3ParseBisonYYSType& rhs);
 
 //######################################################################
 
-class V3ParseImp {
+class V3ParseImp final {
     // MEMBERS
     AstNetlist* m_rootp;  // Root of the design
     VInFilter* m_filterp;  // Reading filter
