@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -32,12 +32,12 @@
 // Lex-derived class
 
 /// Override the base lexer class so we can add some access functions
-class V3Lexer : public V3LexerBase {
+class V3Lexer final : public V3LexerBase {
 public:
     // CONSTRUCTORS
     V3Lexer()
         : V3LexerBase{nullptr} {}
-    ~V3Lexer() override {}
+    ~V3Lexer() override = default;
     // METHODS
     void unputString(const char* textp, size_t length) {
         // Add characters to input stream in back-to-front order
@@ -64,7 +64,7 @@ void V3ParseImp::yylexReadTok() {
 void V3ParseImp::lexNew() {
     if (m_lexerp) delete m_lexerp;  // Restart from clean slate.
     m_lexerp = new V3Lexer();
-    if (debugFlex() >= 9) { m_lexerp->set_debug(~0); }
+    if (debugFlex() >= 9) m_lexerp->set_debug(~0);
 }
 
 void V3ParseImp::lexDestroy() {
