@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2000-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2000-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -18,8 +18,8 @@
 // It is not intended for user applications.
 //*************************************************************************
 
-#ifndef _VPRELEX_H_  // Guard
-#define _VPRELEX_H_ 1
+#ifndef VERILATOR_VPRELEX_H_  // Guard
+#define VERILATOR_VPRELEX_H_
 
 #include "V3Error.h"
 #include "V3FileLine.h"
@@ -126,7 +126,7 @@ void yy_delete_buffer(YY_BUFFER_STATE b);
 //======================================================================
 // Entry for each file processed; a stack of entries included
 
-class VPreStream {
+class VPreStream final {
 public:
     FileLine* m_curFilelinep;  // Current processing point (see also m_tokFilelinep)
     V3PreLex* m_lexp;  // Lexer, for resource tracking
@@ -149,7 +149,7 @@ private:
 //======================================================================
 // Class entry for each per-lexer state
 
-class V3PreLex {
+class V3PreLex final {
 public:  // Used only by V3PreLex.cpp and V3PreProc.cpp
     V3PreProcImp* m_preimpp;  // Preprocessor lexor belongs to
     std::stack<VPreStream*> m_streampStack;  // Stack of processing files
