@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2000-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2000-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -43,7 +43,7 @@
 
 //*************************************************************************
 
-class VDefine {
+class VDefine final {
     // Define class.  One for each define.
     // string    m_name;         // Name of the define (list is keyed by this)
     FileLine* m_fileline;  // Where it was declared
@@ -64,7 +64,7 @@ public:
 
 //*************************************************************************
 
-class VDefineRef {
+class VDefineRef final {
     // One for each pending define substitution
     string m_name;  // Define last name being defined
     string m_params;  // Define parameter list for next expansion
@@ -83,13 +83,13 @@ public:
     VDefineRef(const string& name, const string& params)
         : m_name{name}
         , m_params{params} {}
-    ~VDefineRef() {}
+    ~VDefineRef() = default;
 };
 
 //*************************************************************************
 /// Data for parsing on/off
 
-class VPreIfEntry {
+class VPreIfEntry final {
     // One for each pending ifdef/ifndef
     bool m_on;  // Current parse for this ifdef level is "on"
     bool m_everOn;  // Some if term in elsif tree has been on
@@ -99,17 +99,17 @@ public:
     VPreIfEntry(bool on, bool everOn)
         : m_on{on}
         , m_everOn{everOn || on} {}  // Note everOn includes new state
-    ~VPreIfEntry() {}
+    ~VPreIfEntry() = default;
 };
 
 //*************************************************************************
 // Data for a preprocessor instantiation.
 
-class V3PreProcImp : public V3PreProc {
+class V3PreProcImp final : public V3PreProc {
 public:
     // TYPES
-    typedef std::map<const string, VDefine> DefinesMap;
-    typedef VInFilter::StrList StrList;
+    using DefinesMap = std::map<const std::string, VDefine>;
+    using StrList = VInFilter::StrList;
 
     // debug() -> see V3PreShellImp::debug; use --debugi-V3PreShell
 
@@ -272,7 +272,7 @@ public:
         m_lexp->m_keepComments = keepComments();
         m_lexp->m_keepWhitespace = keepWhitespace();
         m_lexp->m_pedantic = pedantic();
-        m_lexp->debug(debug() >= 5 ? debug() : 0);  // See also V3PreProc::debug() method
+        debug(debug());  // Set lexer debug via V3PreProc::debug() method
     }
     ~V3PreProcImp() override {
         if (m_lexp) VL_DO_CLEAR(delete m_lexp, m_lexp = nullptr);
@@ -335,16 +335,19 @@ void V3PreProcImp::define(FileLine* fl, const string& name, const string& value,
             if (!(defValue(name) == value
                   && defParams(name) == params)) {  // Duplicate defs are OK
                 fl->v3warn(REDEFMACRO, "Redefining existing define: '"
-                                           << name << "', with different value: " << value
-                                           << (params == "" ? "" : " ") << params);
-                defFileline(name)->v3warn(REDEFMACRO, "Previous definition is here, with value: "
-                                                          << defValue(name)
-                                                          << (defParams(name).empty() ? "" : " ")
-                                                          << defParams(name));
+                                           << name << "', with different value: '" << value
+                                           << (params == "" ? "" : " ") << params << "'\n"
+                                           << fl->warnContextPrimary() << '\n'
+                                           << defFileline(name)->warnOther()
+                                           << "... Location of previous definition, with value: '"
+                                           << defValue(name)
+                                           << (defParams(name).empty() ? "" : " ")
+                                           << defParams(name) << "'\n"
+                                           << defFileline(name)->warnContextSecondary());
             }
             undef(name);
         }
-        m_defines.insert(make_pair(name, VDefine(fl, value, params, cmdline)));
+        m_defines.emplace(name, VDefine(fl, value, params, cmdline));
     }
 }
 
@@ -486,6 +489,12 @@ void V3PreProcImp::comment(const string& text) {
 //*************************************************************************
 // VPreProc Methods.
 
+void V3PreProc::debug(int level) {
+    m_debug = level;
+    V3PreProcImp* idatap = static_cast<V3PreProcImp*>(this);
+    if (idatap->m_lexp) idatap->m_lexp->debug(debug() >= 5 ? debug() : 0);
+}
+
 FileLine* V3PreProc::fileline() {
     V3PreProcImp* idatap = static_cast<V3PreProcImp*>(this);
     return idatap->m_lexp->m_tokFilelinep;
@@ -804,6 +813,8 @@ void V3PreProcImp::openFile(FileLine*, VInFilter* filterp, const string& filenam
     // Filter all DOS CR's en-mass.  This avoids bugs with lexing CRs in the wrong places.
     // This will also strip them from strings, but strings aren't supposed
     // to be multi-line without a "\"
+    int eof_newline = 0;  // Number of characters following last newline
+    int eof_lineno = 1;
     for (StrList::iterator it = wholefile.begin(); it != wholefile.end(); ++it) {
         // We don't end-loop at \0 as we allow and strip mid-string '\0's (for now).
         bool strip = false;
@@ -815,6 +826,12 @@ void V3PreProcImp::openFile(FileLine*, VInFilter* filterp, const string& filenam
                 strip = true;
                 break;
             }
+            if (VL_UNLIKELY(*cp == '\n')) {
+                eof_newline = 0;
+                ++eof_lineno;
+            } else {
+                ++eof_newline;
+            }
         }
         if (strip) {
             string out;
@@ -830,6 +847,15 @@ void V3PreProcImp::openFile(FileLine*, VInFilter* filterp, const string& filenam
         // Reclaim memory; the push saved the string contents for us
         *it = "";
     }
+
+    // Warning check
+    if (eof_newline) {
+        FileLine* fl = new FileLine{flsp};
+        fl->contentLineno(eof_lineno);
+        fl->column(eof_newline + 1, eof_newline + 1);
+        fl->v3warn(EOFNEWLINE, "Missing newline at end of file (POSIX 3.206)."
+                                   << fl->warnMore() << "... Suggest add newline.");
+    }
 }
 
 void V3PreProcImp::insertUnreadbackAtBol(const string& text) {
@@ -855,7 +881,7 @@ void V3PreProcImp::dumpDefines(std::ostream& os) {
         // No need to print "()" below as already part of params()
         if (!it->second.params().empty()) os << it->second.params();
         if (!it->second.value().empty()) os << " " << it->second.value();
-        os << endl;
+        os << '\n';
     }
 }
 
@@ -953,7 +979,7 @@ int V3PreProcImp::getStateToken() {
 
         // Most states emit white space and comments between tokens. (Unless collecting a string)
         if (tok == VP_WHITE && state() != ps_STRIFY) return tok;
-        if (tok == VP_BACKQUOTE && state() != ps_STRIFY) { tok = VP_TEXT; }
+        if (tok == VP_BACKQUOTE && state() != ps_STRIFY) tok = VP_TEXT;
         if (tok == VP_COMMENT) {
             if (!m_off) {
                 if (m_lexp->m_keepComments == KEEPCMT_SUB) {
@@ -1317,7 +1343,6 @@ int V3PreProcImp::getStateToken() {
                 // Convert any newlines to spaces, so we don't get a
                 // multiline "..." without \ escapes.
                 // The spec is silent about this either way; simulators vary
-                string::size_type pos;
                 std::replace(out.begin(), out.end(), '\n', ' ');
                 unputString(string("\"") + out + "\"");
                 statePop();
