@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2004-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2004-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -29,7 +29,7 @@
 
 //######################################################################
 
-class V3PreShellImp {
+class V3PreShellImp final {
 protected:
     friend class V3PreShell;
 
@@ -151,8 +151,8 @@ private:
 
 public:
     // CONSTRUCTORS
-    V3PreShellImp() {}
-    ~V3PreShellImp() {}
+    V3PreShellImp() = default;
+    ~V3PreShellImp() = default;
 };
 
 V3PreShellImp V3PreShellImp::s_preImp;
