@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -43,11 +43,8 @@ constexpr unsigned RELOOP_MIN_ITERS = 40;  // Need at least this many loops to d
 
 //######################################################################
 
-class ReloopVisitor : public AstNVisitor {
+class ReloopVisitor final : public AstNVisitor {
 private:
-    // TYPES
-    typedef std::vector<AstNodeAssign*> AssVec;
-
     // NODE STATE
     // AstCFunc::user1p      -> Var* for temp var, 0=not set yet
     AstUser1InUse m_inuser1;
@@ -57,7 +54,7 @@ private:
     VDouble0 m_statReItems;  // Statistic tracking
     AstCFunc* m_cfuncp = nullptr;  // Current block
 
-    AssVec m_mgAssignps;  // List of assignments merging
+    std::vector<AstNodeAssign*> m_mgAssignps;  // List of assignments merging
     AstCFunc* m_mgCfuncp = nullptr;  // Parent C function
     AstNode* m_mgNextp = nullptr;  // Next node
     AstNodeSel* m_mgSelLp = nullptr;  // Parent select, nullptr = idle
