@@ -6,20 +6,16 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2019 by Wilson Snyder.  This program is free software; you can
-// redistribute it and/or modify it under the terms of either the GNU
+// Copyright 2003-2021 by Wilson Snyder. This program is free software; you
+// can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
-//
-// Verilator is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
+// SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0
 //
 //*************************************************************************
 
-#ifndef _V3RELOOP_H_
-#define _V3RELOOP_H_ 1
+#ifndef VERILATOR_V3RELOOP_H_
+#define VERILATOR_V3RELOOP_H_
 
 #include "config_build.h"
 #include "verilatedos.h"
@@ -29,7 +25,7 @@
 
 //============================================================================
 
-class V3Reloop {
+class V3Reloop final {
 public:
     static void reloopAll(AstNetlist* nodep);
 };
