@@ -13,7 +13,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -21,8 +21,8 @@
 //
 //*************************************************************************
 
-#ifndef _V3SCOREBOARD_H_
-#define _V3SCOREBOARD_H_
+#ifndef VERILATOR_V3SCOREBOARD_H_
+#define VERILATOR_V3SCOREBOARD_H_
 
 #include "config_build.h"
 #include "verilatedos.h"
@@ -41,29 +41,28 @@
 /// break ties in the sort when values collide.
 
 template <typename T_Key, typename T_Value, class T_KeyCompare = std::less<T_Key>>
-class SortByValueMap {
+class SortByValueMap final {
     // TYPES
 private:
-    typedef std::unordered_map<T_Key, T_Value> Key2Val;
-    typedef std::set<T_Key, T_KeyCompare> KeySet;
-    typedef std::map<T_Value, KeySet> Val2Keys;
+    using KeySet = std::set<T_Key, T_KeyCompare>;
+    using Val2Keys = std::map<T_Value, KeySet>;
 
     // MEMBERS
-    Key2Val m_keys;  // Map each key to its value. Not sorted.
+    std::unordered_map<T_Key, T_Value> m_keys;  // Map each key to its value. Not sorted.
     Val2Keys m_vals;  // Map each value to its keys. Sorted.
 
 public:
     // CONSTRUCTORS
-    SortByValueMap() {}
+    SortByValueMap() = default;
 
-    class const_iterator {
+    class const_iterator VL_NOT_FINAL {
         // TYPES
     public:
-        typedef const_iterator value_type;
-        typedef const_iterator reference;  // See comment on operator*()
-        typedef void pointer;
-        typedef std::ptrdiff_t difference_type;
-        typedef std::bidirectional_iterator_tag iterator_category;
+        using value_type = const_iterator;
+        using reference = const_iterator;  // See comment on operator*()
+        using pointer = void;
+        using difference_type = std::ptrdiff_t;
+        using iterator_category = std::bidirectional_iterator_tag;
 
     protected:
         friend class SortByValueMap;
@@ -193,11 +192,11 @@ public:
         }
     };
 
-    class iterator : public const_iterator {
+    class iterator final : public const_iterator {
     public:
         // TYPES
-        typedef iterator value_type;
-        typedef iterator reference;
+        using value_type = iterator;
+        using reference = iterator;
         // pointer, difference_type, and iterator_category inherit from
         // const_iterator
 
@@ -223,8 +222,8 @@ public:
         }
     };
 
-    typedef std::reverse_iterator<iterator> reverse_iterator;
-    typedef std::reverse_iterator<const_iterator> const_reverse_iterator;
+    using reverse_iterator = std::reverse_iterator<iterator>;
+    using const_reverse_iterator = std::reverse_iterator<const_iterator>;
 
     // METHODS
 private:
@@ -350,22 +349,21 @@ private:
 /// the full set size.
 
 template <typename T_Elem, typename T_Score, class T_ElemCompare = std::less<T_Elem>>
-class V3Scoreboard {
+class V3Scoreboard final {
 private:
     // TYPES
-    typedef std::unordered_set<const T_Elem*> NeedRescoreSet;
-    class CmpElems {
+    class CmpElems final {
     public:
         bool operator()(const T_Elem* const& ap, const T_Elem* const& bp) const {
             T_ElemCompare cmp;
             return cmp.operator()(*ap, *bp);
         }
     };
-    typedef SortByValueMap<const T_Elem*, T_Score, CmpElems> SortedMap;
-    typedef T_Score (*UserScoreFnp)(const T_Elem*);
+    using SortedMap = SortByValueMap<const T_Elem*, T_Score, CmpElems>;
+    using UserScoreFnp = T_Score (*)(const T_Elem*);
 
     // MEMBERS
-    NeedRescoreSet m_unknown;  // Elements with unknown scores
+    std::unordered_set<const T_Elem*> m_unknown;  // Elements with unknown scores
     SortedMap m_sorted;  // Set of elements with known scores
     UserScoreFnp m_scoreFnp;  // Scoring function
     bool m_slowAsserts;  // Do some asserts that require extra lookups
@@ -375,7 +373,7 @@ public:
     explicit V3Scoreboard(UserScoreFnp scoreFnp, bool slowAsserts)
         : m_scoreFnp{scoreFnp}
         , m_slowAsserts{slowAsserts} {}
-    ~V3Scoreboard() {}
+    ~V3Scoreboard() = default;
 
     // METHODS
 
