@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -16,8 +16,8 @@
 // AstSenTree related utilities.
 //*************************************************************************
 
-#ifndef _V3SENTREE_H_
-#define _V3SENTREE_H_
+#ifndef VERILATOR_V3SENTREE_H_
+#define VERILATOR_V3SENTREE_H_
 
 #include "config_build.h"
 #include "verilatedos.h"
@@ -31,7 +31,7 @@
 // Collect SenTrees under the entire scope
 // And provide functions to find/add a new one
 
-class SenTreeSet {
+class SenTreeSet final {
     // Hash table of sensitive blocks.
 private:
     // TYPES
@@ -48,12 +48,12 @@ private:
     };
 
     // MEMBERS
-    typedef std::unordered_set<AstSenTree*, HashSenTree, EqSenTree> Set;
+    using Set = std::unordered_set<AstSenTree*, HashSenTree, EqSenTree>;
     Set m_trees;  // Set of sensitive blocks, for folding.
 
 public:
     // CONSTRUCTORS
-    SenTreeSet() {}
+    SenTreeSet() = default;
 
     // METHODS
     void add(AstSenTree* nodep) { m_trees.insert(nodep); }
@@ -71,7 +71,7 @@ private:
     VL_UNCOPYABLE(SenTreeSet);
 };
 
-class SenTreeFinder {
+class SenTreeFinder final {
 private:
     // STATE
     AstTopScope* m_topScopep = nullptr;  // Top scope to add global SenTrees to
@@ -81,7 +81,7 @@ private:
 
 public:
     // CONSTRUCTORS
-    SenTreeFinder() {}
+    SenTreeFinder() = default;
 
     // METHODS
     AstSenTree* getSenTree(AstSenTree* senTreep) {
