@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -95,14 +95,14 @@
 //######################################################################
 // Support classes
 
-class SplitNodeVertex : public V3GraphVertex {
+class SplitNodeVertex VL_NOT_FINAL : public V3GraphVertex {
     AstNode* m_nodep;
 
 protected:
     SplitNodeVertex(V3Graph* graphp, AstNode* nodep)
         : V3GraphVertex{graphp}
         , m_nodep{nodep} {}
-    virtual ~SplitNodeVertex() override {}
+    virtual ~SplitNodeVertex() override = default;
     // ACCESSORS
     // Do not make accessor for nodep(),  It may change due to
     // reordering a lower block, but we don't repair it
@@ -115,36 +115,36 @@ public:
     virtual AstNode* nodep() const { return m_nodep; }
 };
 
-class SplitPliVertex : public SplitNodeVertex {
+class SplitPliVertex final : public SplitNodeVertex {
 public:
     explicit SplitPliVertex(V3Graph* graphp, AstNode* nodep)
         : SplitNodeVertex{graphp, nodep} {}
-    virtual ~SplitPliVertex() override {}
+    virtual ~SplitPliVertex() override = default;
     virtual string name() const override { return "*PLI*"; }
     virtual string dotColor() const override { return "green"; }
 };
 
-class SplitLogicVertex : public SplitNodeVertex {
+class SplitLogicVertex final : public SplitNodeVertex {
 public:
     SplitLogicVertex(V3Graph* graphp, AstNode* nodep)
         : SplitNodeVertex{graphp, nodep} {}
-    virtual ~SplitLogicVertex() override {}
+    virtual ~SplitLogicVertex() override = default;
     virtual string dotColor() const override { return "yellow"; }
 };
 
-class SplitVarStdVertex : public SplitNodeVertex {
+class SplitVarStdVertex final : public SplitNodeVertex {
 public:
     SplitVarStdVertex(V3Graph* graphp, AstNode* nodep)
         : SplitNodeVertex{graphp, nodep} {}
-    virtual ~SplitVarStdVertex() override {}
+    virtual ~SplitVarStdVertex() override = default;
     virtual string dotColor() const override { return "skyblue"; }
 };
 
-class SplitVarPostVertex : public SplitNodeVertex {
+class SplitVarPostVertex final : public SplitNodeVertex {
 public:
     SplitVarPostVertex(V3Graph* graphp, AstNode* nodep)
         : SplitNodeVertex{graphp, nodep} {}
-    virtual ~SplitVarPostVertex() override {}
+    virtual ~SplitVarPostVertex() override = default;
     virtual string name() const override { return string("POST ") + SplitNodeVertex::name(); }
     virtual string dotColor() const override { return "CadetBlue"; }
 };
@@ -152,7 +152,7 @@ public:
 //######################################################################
 // Edge types
 
-class SplitEdge : public V3GraphEdge {
+class SplitEdge VL_NOT_FINAL : public V3GraphEdge {
     uint32_t m_ignoreInStep = 0;  // Step number that if set to, causes this edge to be ignored
     static uint32_t s_stepNum;  // Global step number
 protected:
@@ -160,7 +160,7 @@ protected:
     SplitEdge(V3Graph* graphp, V3GraphVertex* fromp, V3GraphVertex* top, int weight,
               bool cutable = CUTABLE)
         : V3GraphEdge{graphp, fromp, top, weight, cutable} {}
-    virtual ~SplitEdge() override {}
+    virtual ~SplitEdge() override = default;
 
 public:
     // Iterator for graph functions
@@ -185,29 +185,29 @@ public:
 };
 uint32_t SplitEdge::s_stepNum = 0;
 
-class SplitPostEdge : public SplitEdge {
+class SplitPostEdge final : public SplitEdge {
 public:
     SplitPostEdge(V3Graph* graphp, V3GraphVertex* fromp, V3GraphVertex* top)
         : SplitEdge{graphp, fromp, top, WEIGHT_NORMAL} {}
-    virtual ~SplitPostEdge() override {}
+    virtual ~SplitPostEdge() override = default;
     virtual bool followScoreboard() const override { return false; }
     virtual string dotColor() const override { return "khaki"; }
 };
 
-class SplitLVEdge : public SplitEdge {
+class SplitLVEdge final : public SplitEdge {
 public:
     SplitLVEdge(V3Graph* graphp, V3GraphVertex* fromp, V3GraphVertex* top)
         : SplitEdge{graphp, fromp, top, WEIGHT_NORMAL} {}
-    virtual ~SplitLVEdge() override {}
+    virtual ~SplitLVEdge() override = default;
     virtual bool followScoreboard() const override { return true; }
     virtual string dotColor() const override { return "yellowGreen"; }
 };
 
-class SplitRVEdge : public SplitEdge {
+class SplitRVEdge final : public SplitEdge {
 public:
     SplitRVEdge(V3Graph* graphp, V3GraphVertex* fromp, V3GraphVertex* top)
         : SplitEdge{graphp, fromp, top, WEIGHT_NORMAL} {}
-    virtual ~SplitRVEdge() override {}
+    virtual ~SplitRVEdge() override = default;
     virtual bool followScoreboard() const override { return true; }
     virtual string dotColor() const override { return "green"; }
 };
@@ -216,7 +216,7 @@ struct SplitScorebdEdge : public SplitEdge {
 public:
     SplitScorebdEdge(V3Graph* graphp, V3GraphVertex* fromp, V3GraphVertex* top)
         : SplitEdge{graphp, fromp, top, WEIGHT_NORMAL} {}
-    virtual ~SplitScorebdEdge() override {}
+    virtual ~SplitScorebdEdge() override = default;
     virtual bool followScoreboard() const override { return true; }
     virtual string dotColor() const override { return "blue"; }
 };
@@ -227,7 +227,7 @@ struct SplitStrictEdge : public SplitEdge {
 public:
     SplitStrictEdge(V3Graph* graphp, V3GraphVertex* fromp, V3GraphVertex* top)
         : SplitEdge{graphp, fromp, top, WEIGHT_NORMAL, NOT_CUTABLE} {}
-    virtual ~SplitStrictEdge() override {}
+    virtual ~SplitStrictEdge() override = default;
     virtual bool followScoreboard() const override { return true; }
     virtual string dotColor() const override { return "blue"; }
 };
@@ -235,7 +235,7 @@ public:
 //######################################################################
 // Split class functions
 
-class SplitReorderBaseVisitor : public AstNVisitor {
+class SplitReorderBaseVisitor VL_NOT_FINAL : public AstNVisitor {
 private:
     // NODE STATE
     // AstVarScope::user1p      -> Var SplitNodeVertex* for usage var, 0=not set yet
@@ -248,12 +248,9 @@ private:
     AstUser4InUse m_inuser4;
 
 protected:
-    // TYPES
-    typedef std::vector<SplitLogicVertex*> VStack;
-
     // STATE
     string m_noReorderWhy;  // Reason we can't reorder
-    VStack m_stmtStackps;  // Current statements being tracked
+    std::vector<SplitLogicVertex*> m_stmtStackps;  // Current statements being tracked
     SplitPliVertex* m_pliVertexp;  // Element specifying PLI ordering
     V3Graph m_graph;  // Scoreboard of var usages/dependencies
     bool m_inDly;  // Inside ASSIGNDLY
@@ -439,11 +436,11 @@ private:
     VL_UNCOPYABLE(SplitReorderBaseVisitor);
 };
 
-class ReorderVisitor : public SplitReorderBaseVisitor {
+class ReorderVisitor final : public SplitReorderBaseVisitor {
     // CONSTRUCTORS
 public:
     explicit ReorderVisitor(AstNetlist* nodep) { iterate(nodep); }
-    virtual ~ReorderVisitor() override {}
+    virtual ~ReorderVisitor() override = default;
 
     // METHODS
 protected:
@@ -523,19 +520,18 @@ protected:
         // Reorder statements in the completed graph
 
         // Map the rank numbers into nodes they associate with
-        typedef std::multimap<uint32_t, AstNode*> RankNodeMap;
-        RankNodeMap rankMap;
+        std::multimap<uint32_t, AstNode*> rankMap;
         int currOrder = 0;  // Existing sequence number of assignment
         for (AstNode* nextp = nodep; nextp; nextp = nextp->nextp()) {
             SplitLogicVertex* vvertexp = reinterpret_cast<SplitLogicVertex*>(nextp->user3p());
-            rankMap.insert(make_pair(vvertexp->rank(), nextp));
+            rankMap.emplace(vvertexp->rank(), nextp);
             nextp->user4(++currOrder);  // Record current ordering
         }
 
         // Is the current ordering OK?
         bool leaveAlone = true;
         int newOrder = 0;  // New sequence number of assignment
-        for (RankNodeMap::const_iterator it = rankMap.begin(); it != rankMap.end(); ++it) {
+        for (auto it = rankMap.cbegin(); it != rankMap.cend(); ++it) {
             AstNode* nextp = it->second;
             if (++newOrder != nextp->user4()) leaveAlone = false;
         }
@@ -544,7 +540,7 @@ protected:
         } else {
             AstNRelinker replaceHandle;  // Where to add the list
             AstNode* newListp = nullptr;
-            for (RankNodeMap::const_iterator it = rankMap.begin(); it != rankMap.end(); ++it) {
+            for (auto it = rankMap.cbegin(); it != rankMap.cend(); ++it) {
                 AstNode* nextp = it->second;
                 UINFO(6, "   New order: " << nextp << endl);
                 if (nextp == nodep) {
@@ -619,18 +615,17 @@ private:
     VL_UNCOPYABLE(ReorderVisitor);
 };
 
-typedef std::unordered_set<uint32_t> ColorSet;
-typedef std::vector<AstAlways*> AlwaysVec;
+using ColorSet = std::unordered_set<uint32_t>;
+using AlwaysVec = std::vector<AstAlways*>;
 
-class IfColorVisitor : public AstNVisitor {
+class IfColorVisitor final : public AstNVisitor {
     // MEMBERS
     ColorSet m_colors;  // All colors in the original always block
 
-    typedef std::vector<AstNodeIf*> IfStack;
-    IfStack m_ifStack;  // Stack of nested if-statements we're currently processing
+    std::vector<AstNodeIf*> m_ifStack;  // Stack of nested if-statements we're currently processing
 
-    typedef std::unordered_map<AstNodeIf*, ColorSet> IfColorMap;
-    IfColorMap m_ifColors;  // Map each if-statement to the set of colors (split blocks)
+    std::unordered_map<AstNodeIf*, ColorSet>
+        m_ifColors;  // Map each if-statement to the set of colors (split blocks)
     // that will get a copy of that if-statement
 
     // CONSTRUCTORS
@@ -638,7 +633,7 @@ public:
     // Visit through *nodep and map each AstNodeIf within to the set of
     // colors it will participate in. Also find the whole set of colors.
     explicit IfColorVisitor(AstAlways* nodep) { iterate(nodep); }
-    virtual ~IfColorVisitor() override {}
+    virtual ~IfColorVisitor() override = default;
 
     // METHODS
     const ColorSet& colors() const { return m_colors; }
@@ -657,7 +652,7 @@ private:
             UINFO(8, "  SVL " << vertexp << " has color " << color << "\n");
 
             // Record that all containing ifs have this color.
-            for (IfStack::const_iterator it = m_ifStack.begin(); it != m_ifStack.end(); ++it) {
+            for (auto it = m_ifStack.cbegin(); it != m_ifStack.cend(); ++it) {
                 m_ifColors[*it].insert(color);
             }
         }
@@ -680,14 +675,13 @@ private:
     VL_UNCOPYABLE(IfColorVisitor);
 };
 
-class EmitSplitVisitor : public AstNVisitor {
+class EmitSplitVisitor final : public AstNVisitor {
     // MEMBERS
     AstAlways* m_origAlwaysp;  // Block that *this will split
     const IfColorVisitor* m_ifColorp;  // Digest of results of prior coloring
 
     // Map each color to our current place within the color's new always
-    typedef std::unordered_map<uint32_t, AstNode*> LocMap;
-    LocMap m_addAfter;
+    std::unordered_map<uint32_t, AstNode*> m_addAfter;
 
     AlwaysVec* m_newBlocksp;  // Split always blocks we have generated
 
@@ -703,7 +697,7 @@ public:
         UINFO(6, "  splitting always " << nodep << endl);
     }
 
-    virtual ~EmitSplitVisitor() override {}
+    virtual ~EmitSplitVisitor() override = default;
 
     // METHODS
     void go() {
@@ -758,7 +752,7 @@ protected:
 
     virtual void visit(AstNodeIf* nodep) override {
         const ColorSet& colors = m_ifColorp->colors(nodep);
-        typedef std::unordered_map<uint32_t, AstNodeIf*> CloneMap;
+        using CloneMap = std::unordered_map<uint32_t, AstNodeIf*>;
         CloneMap clones;
 
         for (unsigned int color : colors) {
@@ -793,9 +787,8 @@ private:
     VL_UNCOPYABLE(EmitSplitVisitor);
 };
 
-class RemovePlaceholdersVisitor : public AstNVisitor {
-    typedef std::unordered_set<AstNode*> NodeSet;
-    NodeSet m_removeSet;  // placeholders to be removed
+class RemovePlaceholdersVisitor final : public AstNVisitor {
+    std::unordered_set<AstNode*> m_removeSet;  // placeholders to be removed
 public:
     explicit RemovePlaceholdersVisitor(AstNode* nodep) {
         iterate(nodep);
@@ -804,7 +797,7 @@ public:
             VL_DO_DANGLING(np->deleteTree(), np);
         }
     }
-    virtual ~RemovePlaceholdersVisitor() override {}
+    virtual ~RemovePlaceholdersVisitor() override = default;
     virtual void visit(AstSplitPlaceholder* nodep) override { m_removeSet.insert(nodep); }
     virtual void visit(AstNode* nodep) override { iterateChildren(nodep); }
 
@@ -812,13 +805,12 @@ private:
     VL_UNCOPYABLE(RemovePlaceholdersVisitor);
 };
 
-class SplitVisitor : public SplitReorderBaseVisitor {
+class SplitVisitor final : public SplitReorderBaseVisitor {
 private:
     // Keys are original always blocks pending delete,
     // values are newly split always blocks pending insertion
     // at the same position as the originals:
-    typedef std::unordered_map<AstAlways*, AlwaysVec> ReplaceMap;
-    ReplaceMap m_replaceBlocks;
+    std::unordered_map<AstAlways*, AlwaysVec> m_replaceBlocks;
 
     // AstNodeIf* whose condition we're currently visiting
     AstNode* m_curIfConditional = nullptr;
@@ -831,8 +823,7 @@ public:
         // Splice newly-split blocks into the tree. Remove placeholders
         // from newly-split blocks. Delete the original always blocks
         // that we're replacing.
-        for (ReplaceMap::iterator it = m_replaceBlocks.begin(); it != m_replaceBlocks.end();
-             ++it) {
+        for (auto it = m_replaceBlocks.begin(); it != m_replaceBlocks.end(); ++it) {
             AstAlways* origp = it->first;
             for (AlwaysVec::iterator addme = it->second.begin(); addme != it->second.end();
                  ++addme) {
@@ -844,14 +835,14 @@ public:
         }
     }
 
-    virtual ~SplitVisitor() override {}
+    virtual ~SplitVisitor() override = default;
 
     // METHODS
 protected:
     virtual void makeRvalueEdges(SplitVarStdVertex* vstdp) override {
         // Each 'if' depends on rvalues in its own conditional ONLY,
         // not rvalues in the if/else bodies.
-        for (VStack::const_iterator it = m_stmtStackps.begin(); it != m_stmtStackps.end(); ++it) {
+        for (auto it = m_stmtStackps.cbegin(); it != m_stmtStackps.cend(); ++it) {
             AstNodeIf* ifNodep = VN_CAST((*it)->nodep(), NodeIf);
             if (ifNodep && (m_curIfConditional != ifNodep)) continue;
             new SplitRVEdge(&m_graph, *it, vstdp);
