@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -33,7 +33,7 @@
 
 //######################################################################
 
-class SplitAsBaseVisitor : public AstNVisitor {
+class SplitAsBaseVisitor VL_NOT_FINAL : public AstNVisitor {
 public:
     // METHODS
     VL_DEBUG_FUNC;  // Declare debug()
@@ -42,7 +42,7 @@ public:
 //######################################################################
 // Find all split variables in a block
 
-class SplitAsFindVisitor : public SplitAsBaseVisitor {
+class SplitAsFindVisitor final : public SplitAsBaseVisitor {
 private:
     // STATE
     AstVarScope* m_splitVscp = nullptr;  // Variable we want to split
@@ -58,7 +58,7 @@ private:
 public:
     // CONSTRUCTORS
     explicit SplitAsFindVisitor(AstAlways* nodep) { iterate(nodep); }
-    virtual ~SplitAsFindVisitor() override {}
+    virtual ~SplitAsFindVisitor() override = default;
     // METHODS
     AstVarScope* splitVscp() const { return m_splitVscp; }
 };
@@ -66,7 +66,7 @@ public:
 //######################################################################
 // Remove nodes not containing proper references
 
-class SplitAsCleanVisitor : public SplitAsBaseVisitor {
+class SplitAsCleanVisitor final : public SplitAsBaseVisitor {
 private:
     // STATE
     AstVarScope* m_splitVscp;  // Variable we want to split
@@ -118,13 +118,13 @@ public:
         , m_modeMatch{modeMatch} {
         iterate(nodep);
     }
-    virtual ~SplitAsCleanVisitor() override {}
+    virtual ~SplitAsCleanVisitor() override = default;
 };
 
 //######################################################################
 // SplitAs class functions
 
-class SplitAsVisitor : public SplitAsBaseVisitor {
+class SplitAsVisitor final : public SplitAsBaseVisitor {
 private:
     // NODE STATE
     //  AstAlways::user()       -> bool.  True if already processed
