@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -232,7 +232,7 @@ struct AstNodeComparator {
     }
 };
 
-class UnpackRef {
+class UnpackRef final {
     // m_nodep is called in this context (AstNodeStmt, AstCell, AstNodeFTask, or AstAlways)
     AstNode* m_contextp;
     AstNode* m_nodep;  // ArraySel, SliceSel, ArrayVarRef (entire value)
@@ -283,11 +283,11 @@ public:
     }
 };
 
-class UnpackRefMap {
+class UnpackRefMap final {
 public:
-    typedef std::map<AstVar*, std::set<UnpackRef>, AstNodeComparator> MapType;
-    typedef MapType::iterator MapIt;
-    typedef MapType::value_type::second_type::iterator SetIt;
+    using MapType = std::map<AstVar*, std::set<UnpackRef>, AstNodeComparator>;
+    using MapIt = MapType::iterator;
+    using SetIt = MapType::value_type::second_type::iterator;
 
 private:
     MapType m_map;
@@ -334,12 +334,9 @@ public:
 
 // Found nodes for SplitPackedVarVisitor
 struct RefsInModule {
-    typedef std::set<AstVar*, AstNodeComparator> VarSet;
-    typedef std::set<AstVarRef*, AstNodeComparator> VarRefSet;
-    typedef std::set<AstSel*, AstNodeComparator> SelSet;
-    VarSet m_vars;
-    VarRefSet m_refs;
-    SelSet m_sels;
+    std::set<AstVar*, AstNodeComparator> m_vars;
+    std::set<AstVarRef*, AstNodeComparator> m_refs;
+    std::set<AstSel*, AstNodeComparator> m_sels;
 
 public:
     void add(AstVar* nodep) { m_vars.insert(nodep); }
@@ -362,7 +359,7 @@ public:
     }
     void visit(AstNVisitor* visitor) {
         for (const auto& varp : m_vars) visitor->iterate(varp);
-        for (SelSet::iterator it = m_sels.begin(), it_end = m_sels.end(); it != it_end; ++it) {
+        for (auto it = m_sels.begin(), it_end = m_sels.end(); it != it_end; ++it) {
             // If m_refs includes VarRef from ArraySel, remove it
             // because the VarRef would not be visited in SplitPackedVarVisitor::visit(AstSel*).
             if (AstVarRef* refp = VN_CAST((*it)->fromp(), VarRef)) {
@@ -375,17 +372,17 @@ public:
             UASSERT_OBJ(reinterpret_cast<uintptr_t>((*it)->op1p()) != 1, *it, "stale");
             visitor->iterate(*it);
         }
-        for (VarRefSet::iterator it = m_refs.begin(), it_end = m_refs.end(); it != it_end; ++it) {
+        for (auto it = m_refs.begin(), it_end = m_refs.end(); it != it_end; ++it) {
             UASSERT_OBJ(reinterpret_cast<uintptr_t>((*it)->op1p()) != 1, *it, "stale");
             visitor->iterate(*it);
         }
     }
 };
 
-typedef std::map<AstNodeModule*, RefsInModule, AstNodeComparator> SplitVarRefsMap;
+using SplitVarRefsMap = std::map<AstNodeModule*, RefsInModule, AstNodeComparator>;
 
-class SplitUnpackedVarVisitor : public AstNVisitor, public SplitVarImpl {
-    typedef std::set<AstVar*, AstNodeComparator> VarSet;
+class SplitUnpackedVarVisitor final : public AstNVisitor, public SplitVarImpl {
+    using VarSet = std::set<AstVar*, AstNodeComparator>;
     VarSet m_foundTargetVar;
     UnpackRefMap m_refs;
     AstNodeModule* m_modp = nullptr;
@@ -405,7 +402,7 @@ class SplitUnpackedVarVisitor : public AstNVisitor, public SplitVarImpl {
     static int outerMostSizeOfUnpackedArray(AstVar* nodep) {
         AstUnpackArrayDType* dtypep = VN_CAST(nodep->dtypep()->skipRefp(), UnpackArrayDType);
         UASSERT_OBJ(dtypep, nodep, "Must be unapcked array");
-        return dtypep->msb() - dtypep->lsb() + 1;
+        return dtypep->elementsConst();
     }
 
     void setContextAndIterateChildren(AstNode* nodep) {
@@ -585,7 +582,7 @@ class SplitUnpackedVarVisitor : public AstNVisitor, public SplitVarImpl {
             const VNumRange selRange{nodep->declRange().hi() + dtypep->declRange().lo(),
                                      nodep->declRange().lo() + dtypep->declRange().lo(),
                                      nodep->declRange().littleEndian()};
-            UASSERT_OBJ(dtypep->lsb() <= selRange.lo() && selRange.hi() <= dtypep->msb(), nodep,
+            UASSERT_OBJ(dtypep->lo() <= selRange.lo() && selRange.hi() <= dtypep->hi(), nodep,
                         "Range check for AstSliceSel must have been finished in V3Width.cpp");
             UINFO(4, "add " << nodep << " for " << refp->varp()->prettyName() << "\n");
             m_refs.tryAdd(m_contextp, refp, nodep, nodep->declRange().hi(),
@@ -614,12 +611,12 @@ class SplitUnpackedVarVisitor : public AstNVisitor, public SplitVarImpl {
         AstVar* varp = newVar(nodep->fileline(), AstVarType::VAR, name, dtypep);
         // Variable will be registered in the caller side.
         UINFO(3, varp->prettyNameQ()
-                     << " is created lsb:" << dtypep->lsb() << " msb:" << dtypep->msb() << "\n");
+                     << " is created lsb:" << dtypep->lo() << " msb:" << dtypep->hi() << "\n");
         // Use AstAssign if true, otherwise AstAssignW
         const bool use_simple_assign
             = (context && VN_IS(context, NodeFTaskRef)) || (assignp && VN_IS(assignp, Assign));
 
-        for (int i = 0; i <= dtypep->msb() - dtypep->lsb(); ++i) {
+        for (int i = 0; i < dtypep->elementsConst(); ++i) {
             AstNode* lhsp = newVarRef(nodep->fileline(), vars.at(start_idx + i),
                                       lvalue ? VAccess::WRITE : VAccess::READ);
             AstNode* rhsp = new AstArraySel(
@@ -689,10 +686,10 @@ class SplitUnpackedVarVisitor : public AstNVisitor, public SplitVarImpl {
             const bool needNext = VN_IS(subTypep, UnpackArrayDType);  // Still unpacked array.
             std::vector<AstVar*> vars;
             // Add the split variables
-            for (vlsint32_t i = 0; i <= dtypep->msb() - dtypep->lsb(); ++i) {
+            for (vlsint32_t i = 0; i < dtypep->elementsConst(); ++i) {
                 // Unpacked array is traced as var(idx), not var[idx].
                 const std::string name
-                    = varp->name() + AstNode::encodeName('(' + cvtToStr(i + dtypep->lsb()) + ')');
+                    = varp->name() + AstNode::encodeName('(' + cvtToStr(i + dtypep->lo()) + ')');
                 AstVar* newp = newVar(varp->fileline(), AstVarType::VAR, name, subTypep);
                 newp->propagateAttrFrom(varp);
                 // If varp is an IO, varp will remain and will be traced.
@@ -722,7 +719,7 @@ class SplitUnpackedVarVisitor : public AstNVisitor, public SplitVarImpl {
                         refp = VN_CAST(selp->fromp(), VarRef);
                         UASSERT_OBJ(refp, selp, "Unexpected op is registered");
                         adtypep = VN_CAST(selp->dtypep()->skipRefp(), UnpackArrayDType);
-                        lsb = adtypep->lsb();
+                        lsb = adtypep->lo();
                     }
                     AstVarRef* newrefp = createTempVar(sit->context(), refp, adtypep, varp->name(),
                                                        vars, lsb, refp->access(), sit->ftask());
@@ -802,7 +799,7 @@ public:
 //  Split packed variables
 
 // Split variable
-class SplitNewVar {
+class SplitNewVar final {
     int m_lsb;  // LSB in the original bitvector
     int m_bitwidth;
     AstVar* m_varp;  // The LSB of this variable is always 0, not m_lsb
@@ -828,7 +825,7 @@ public:
 };
 
 // One Entry instance for an AstVarRef instance
-class PackedVarRefEntry {
+class PackedVarRefEntry final {
     AstNode* m_nodep;  // Either AstSel or AstVarRef is expected.
     int m_lsb;
     int m_bitwidth;
@@ -853,13 +850,13 @@ public:
     // If this is AstVarRef and referred in the sensitivity list of always@,
     // return the sensitivity item
     AstSenItem* backSenItemp() const {
-        if (AstVarRef* refp = VN_CAST(m_nodep, VarRef)) { return VN_CAST(refp->backp(), SenItem); }
+        if (AstVarRef* refp = VN_CAST(m_nodep, VarRef)) return VN_CAST(refp->backp(), SenItem);
         return nullptr;
     }
 };
 
 // How a variable is used
-class PackedVarRef {
+class PackedVarRef final {
     struct SortByFirst {
         bool operator()(const std::pair<int, bool>& a, const std::pair<int, bool>& b) const {
             if (a.first == b.first) return a.second < b.second;
@@ -871,23 +868,19 @@ class PackedVarRef {
     bool m_dedupDone = false;
     static void dedupRefs(std::vector<PackedVarRefEntry>& refs) {
         // Use raw pointer to dedup
-        typedef std::map<AstNode*, size_t, AstNodeComparator> NodeIndices;
-        NodeIndices nodes;
-        for (size_t i = 0; i < refs.size(); ++i) {
-            nodes.insert(std::make_pair(refs[i].nodep(), i));
-        }
+        std::map<AstNode*, size_t, AstNodeComparator> nodes;
+        for (size_t i = 0; i < refs.size(); ++i) { nodes.emplace(refs[i].nodep(), i); }
         std::vector<PackedVarRefEntry> vect;
         vect.reserve(nodes.size());
-        for (NodeIndices::const_iterator it = nodes.begin(), it_end = nodes.end(); it != it_end;
-             ++it) {
+        for (auto it = nodes.cbegin(), it_end = nodes.cend(); it != it_end; ++it) {
             vect.push_back(refs[it->second]);
         }
         refs.swap(vect);
     }
 
 public:
-    typedef std::vector<PackedVarRefEntry>::iterator iterator;
-    typedef std::vector<PackedVarRefEntry>::const_iterator const_iterator;
+    using iterator = std::vector<PackedVarRefEntry>::iterator;
+    using const_iterator = std::vector<PackedVarRefEntry>::const_iterator;
     std::vector<PackedVarRefEntry>& lhs() {
         UASSERT(m_dedupDone, "cannot read before dedup()");
         return m_lhs;
@@ -922,8 +915,8 @@ public:
             points.emplace_back(std::make_pair(it->msb() + 1, true));  // End of a region
         }
         if (skipUnused && !m_rhs.empty()) {  // Range to be read must be kept, so add points here
-            int lsb = m_basicp->msb() + 1;
-            int msb = m_basicp->lsb() - 1;
+            int lsb = m_basicp->hi() + 1;
+            int msb = m_basicp->lo() - 1;
             for (size_t i = 0; i < m_rhs.size(); ++i) {
                 lsb = std::min(lsb, m_rhs[i].lsb());
                 msb = std::max(msb, m_rhs[i].msb());
@@ -933,8 +926,8 @@ public:
             points.emplace_back(std::make_pair(msb + 1, true));
         }
         if (!skipUnused) {  // All bits are necessary
-            points.emplace_back(std::make_pair(m_basicp->lsb(), false));
-            points.emplace_back(std::make_pair(m_basicp->msb() + 1, true));
+            points.emplace_back(std::make_pair(m_basicp->lo(), false));
+            points.emplace_back(std::make_pair(m_basicp->hi() + 1, true));
         }
         std::sort(points.begin(), points.end(), SortByFirst());
 
@@ -956,13 +949,12 @@ public:
     }
 };
 
-class SplitPackedVarVisitor : public AstNVisitor, public SplitVarImpl {
-    typedef std::map<AstVar*, PackedVarRef, AstNodeComparator> PackedVarRefMap;
+class SplitPackedVarVisitor final : public AstNVisitor, public SplitVarImpl {
     AstNetlist* m_netp;
-    AstNodeModule* m_modp;  // Current module (just for log)
-    int m_numSplit;  // Total number of split variables
+    AstNodeModule* m_modp = nullptr;  // Current module (just for log)
+    int m_numSplit = 0;  // Total number of split variables
     // key:variable to be split. value:location where the variable is referenced.
-    PackedVarRefMap m_refs;
+    std::map<AstVar*, PackedVarRef, AstNodeComparator> m_refs;
     virtual void visit(AstNodeFTask* nodep) override {
         if (!cannotSplitTaskReason(nodep)) iterateChildren(nodep);
     }
@@ -973,7 +965,7 @@ class SplitPackedVarVisitor : public AstNVisitor, public SplitVarImpl {
             nodep->attrSplitVar(false);
         } else {  // Finally find a good candidate
             const bool inserted = m_refs.insert(std::make_pair(nodep, PackedVarRef(nodep))).second;
-            if (inserted) { UINFO(3, nodep->prettyNameQ() << " is added to candidate list.\n"); }
+            if (inserted) UINFO(3, nodep->prettyNameQ() << " is added to candidate list.\n");
         }
     }
     virtual void visit(AstVarRef* nodep) override {
@@ -982,13 +974,13 @@ class SplitPackedVarVisitor : public AstNVisitor, public SplitVarImpl {
         const auto refit = m_refs.find(varp);
         if (refit == m_refs.end()) return;  // variable without split_var metacomment
         UASSERT_OBJ(varp->attrSplitVar(), varp, "split_var attribute must be attached");
-        UASSERT_OBJ(!nodep->packagep(), nodep,
+        UASSERT_OBJ(!nodep->classOrPackagep(), nodep,
                     "variable in package must have been dropped beforehand.");
         const AstBasicDType* basicp = refit->second.basicp();
-        refit->second.append(PackedVarRefEntry(nodep, basicp->lsb(), varp->width()),
+        refit->second.append(PackedVarRefEntry(nodep, basicp->lo(), varp->width()),
                              nodep->access());
         UINFO(5, varp->prettyName()
-                     << " Entire bit of [" << basicp->lsb() << ":+" << varp->width() << "] \n");
+                     << " Entire bit of [" << basicp->lo() << "+:" << varp->width() << "] \n");
     }
     virtual void visit(AstSel* nodep) override {
         AstVarRef* vrefp = VN_CAST(nodep->fromp(), VarRef);
@@ -1005,15 +997,17 @@ class SplitPackedVarVisitor : public AstNVisitor, public SplitVarImpl {
         }
         UASSERT_OBJ(varp->attrSplitVar(), varp, "split_var attribute must be attached");
 
-        AstConst* consts[2] = {VN_CAST(nodep->lsbp(), Const), VN_CAST(nodep->widthp(), Const)};
+        std::array<AstConst*, 2> consts
+            = {{VN_CAST(nodep->lsbp(), Const),
+                VN_CAST(nodep->widthp(), Const)}};  // GCC 3.8.0 wants {{}}
         if (consts[0] && consts[1]) {  // OK
             refit->second.append(
-                PackedVarRefEntry(nodep, consts[0]->toSInt() + refit->second.basicp()->lsb(),
+                PackedVarRefEntry(nodep, consts[0]->toSInt() + refit->second.basicp()->lo(),
                                   consts[1]->toUInt()),
                 vrefp->access());
             UINFO(5, varp->prettyName()
                          << " [" << consts[0]->toSInt() << ":+" << consts[1]->toSInt()
-                         << "] lsb:" << refit->second.basicp()->lsb() << "\n");
+                         << "] lsb:" << refit->second.basicp()->lo() << "\n");
         } else {
             nodep->v3warn(SPLITVAR, vrefp->prettyNameQ()
                                         << notSplitMsg
@@ -1079,7 +1073,8 @@ class SplitPackedVarVisitor : public AstNVisitor, public SplitVarImpl {
     void createVars(AstVar* varp, const AstBasicDType* basicp, std::vector<SplitNewVar>& vars) {
         for (size_t i = 0; i < vars.size(); ++i) {
             SplitNewVar* newvarp = &vars[i];
-            int left = newvarp->msb(), right = newvarp->lsb();
+            int left = newvarp->msb();
+            int right = newvarp->lsb();
             if (basicp->littleEndian()) std::swap(left, right);
             const std::string name
                 = (left == right)
@@ -1099,8 +1094,8 @@ class SplitPackedVarVisitor : public AstNVisitor, public SplitVarImpl {
                 break;
             default: UASSERT_OBJ(false, basicp, "Only bit and logic are allowed");
             }
-            dtypep->rangep(new AstRange(varp->fileline(), newvarp->msb(), newvarp->lsb()));
-            dtypep->rangep()->littleEndian(basicp->littleEndian());
+            dtypep->rangep(new AstRange{varp->fileline(), VNumRange{newvarp->msb(), newvarp->lsb(),
+                                                                    basicp->littleEndian()}});
             newvarp->varp(new AstVar(varp->fileline(), AstVarType::VAR, name, dtypep));
             newvarp->varp()->propagateAttrFrom(varp);
             newvarp->varp()->funcLocal(varp->isFuncLocal() || varp->isFuncReturn());
@@ -1162,8 +1157,7 @@ class SplitPackedVarVisitor : public AstNVisitor, public SplitVarImpl {
     }
     // Do the actual splitting operation
     void split() {
-        for (PackedVarRefMap::iterator it = m_refs.begin(), it_end = m_refs.end(); it != it_end;
-             ++it) {
+        for (auto it = m_refs.begin(), it_end = m_refs.end(); it != it_end; ++it) {
             it->second.dedup();
             AstVar* varp = it->first;
             UINFO(3, "In module " << m_modp->name() << " var " << varp->prettyNameQ()
@@ -1206,9 +1200,7 @@ class SplitPackedVarVisitor : public AstNVisitor, public SplitVarImpl {
 public:
     // When reusing the information from SplitUnpackedVarVisitor
     SplitPackedVarVisitor(AstNetlist* nodep, SplitVarRefsMap& refs)
-        : m_netp{nodep}
-        , m_modp{nullptr}
-        , m_numSplit{0} {
+        : m_netp{nodep} {
         // If you want ignore refs and walk the tne entire AST,
         // just call iterateChildren(m_modp) and split() for each module
         for (auto& i : refs) {
