@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2005-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2005-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -30,11 +30,11 @@
 //######################################################################
 // Stats class functions
 
-class StatsVisitor : public AstNVisitor {
+class StatsVisitor final : public AstNVisitor {
 private:
     // NODE STATE/TYPES
 
-    typedef std::map<const string, int> NameMap;  // Number of times a name appears
+    using NameMap = std::map<const std::string, int>;  // Number of times a name appears
 
     // STATE
     string m_stage;  // Name of the stage we are scanning
@@ -49,7 +49,7 @@ private:
 
     std::vector<VDouble0> m_statTypeCount;  // Nodes of given type
     VDouble0 m_statAbove[AstType::_ENUM_END][AstType::_ENUM_END];  // Nodes of given type
-    VDouble0 m_statPred[VBranchPred::_ENUM_END];  // Nodes of given type
+    std::array<VDouble0, VBranchPred::_ENUM_END> m_statPred;  // Nodes of given type
     VDouble0 m_statInstr;  // Instruction count
     VDouble0 m_statInstrFast;  // Instruction count, non-slow() eval functions only
     std::vector<VDouble0> m_statVarWidths;  // Variables of given width
@@ -134,7 +134,7 @@ private:
             // Need to do even if !m_counting because maybe determining upstream if/else
             double ifInstrs = 0.0;
             double elseInstrs = 0.0;
-            if (nodep->branchPred() != VBranchPred::BP_UNLIKELY) {  // Check if
+            if (!nodep->branchPred().unlikely()) {  // Check if
                 VL_RESTORER(m_instrs);
                 VL_RESTORER(m_counting);
                 {
@@ -144,7 +144,7 @@ private:
                     ifInstrs = m_instrs;
                 }
             }
-            if (nodep->branchPred() != VBranchPred::BP_LIKELY) {  // Check else
+            if (!nodep->branchPred().likely()) {  // Check else
                 VL_RESTORER(m_instrs);
                 VL_RESTORER(m_counting);
                 {
