@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2005-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2005-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -30,32 +30,30 @@
 //######################################################################
 // Stats dumping
 
-class StatsReport {
+class StatsReport final {
     // TYPES
-    typedef std::vector<V3Statistic> StatColl;
+    using StatColl = std::vector<V3Statistic>;
 
     // STATE
     std::ofstream& os;  ///< Output stream
     static StatColl s_allStats;  ///< All statistics
 
     void header() {
-        os << "Verilator Statistics Report\n";
-        os << endl;
+        os << "Verilator Statistics Report\n\n";
 
-        os << "Information:" << endl;
-        os << "  " << V3Options::version() << endl;
-        os << "  Arguments: " << v3Global.opt.allArgsString() << endl;
-        os << endl;
+        os << "Information:\n";
+        os << "  " << V3Options::version() << '\n';
+        os << "  Arguments: " << v3Global.opt.allArgsString() << '\n';
+        os << '\n';
     }
 
     void sumit() {
         // If sumit is set on a statistic, combine with others of same name
-        typedef std::multimap<string, V3Statistic*> ByName;
-        ByName byName;
+        std::multimap<std::string, V3Statistic*> byName;
         // * is always first
         for (auto& itr : s_allStats) {
             V3Statistic* repp = &itr;
-            byName.insert(make_pair(repp->name(), repp));
+            byName.emplace(repp->name(), repp);
         }
 
         // Process duplicates
@@ -73,40 +71,37 @@ class StatsReport {
     void stars() {
         // Find all stages
         size_t maxWidth = 0;
-        typedef std::multimap<string, const V3Statistic*> ByName;
-        ByName byName;
+        std::multimap<std::string, const V3Statistic*> byName;
         // * is always first
         for (const auto& itr : s_allStats) {
             const V3Statistic* repp = &itr;
             if (repp->stage() == "*" && repp->printit()) {
                 if (maxWidth < repp->name().length()) maxWidth = repp->name().length();
-                byName.insert(make_pair(repp->name(), repp));
+                byName.emplace(repp->name(), repp);
             }
         }
 
         // Print organized by stage
-        os << "Global Statistics:\n";
-        os << endl;
+        os << "Global Statistics:\n\n";
         for (const auto& itr : byName) {
             const V3Statistic* repp = itr.second;
             if (repp->perf()) continue;
             os << "  " << std::left << std::setw(maxWidth) << repp->name();
             repp->dump(os);
-            os << endl;
+            os << '\n';
         }
-        os << endl;
+        os << '\n';
 
         // Print organized by stage
-        os << "Performance Statistics:\n";
-        os << endl;
+        os << "Performance Statistics:\n\n";
         for (const auto& itr : byName) {
             const V3Statistic* repp = itr.second;
             if (!repp->perf()) continue;
             os << "  " << std::left << std::setw(maxWidth) << repp->name();
             repp->dump(os);
-            os << endl;
+            os << '\n';
         }
-        os << endl;
+        os << '\n';
     }
 
     void stages() {
@@ -115,28 +110,26 @@ class StatsReport {
         // Find all stages
         int stage = 0;
         size_t maxWidth = 0;
-        typedef std::vector<string> Stages;
-        Stages stages;
+        std::vector<std::string> stages;
         std::unordered_map<string, int> stageInt;
-        typedef std::multimap<string, const V3Statistic*> ByName;
-        ByName byName;
+        std::multimap<std::string, const V3Statistic*> byName;
         // * is always first
-        for (StatColl::iterator it = s_allStats.begin(); it != s_allStats.end(); ++it) {
+        for (auto it = s_allStats.begin(); it != s_allStats.end(); ++it) {
             const V3Statistic* repp = &(*it);
             if (repp->stage() != "*" && repp->printit()) {
                 if (maxWidth < repp->name().length()) maxWidth = repp->name().length();
                 if (stageInt.find(repp->stage()) == stageInt.end()) {
-                    stageInt.insert(make_pair(repp->stage(), stage++));
+                    stageInt.emplace(repp->stage(), stage++);
                     stages.push_back(repp->stage());
                 }
-                byName.insert(make_pair(repp->name(), repp));
+                byName.emplace(repp->name(), repp);
             }
         }
 
         // Header
         os << "  Stat     " << std::left << std::setw(maxWidth - 5 - 2) << "";
         for (const string& i : stages) os << "  " << std::left << std::setw(9) << i;
-        os << endl;
+        os << '\n';
         os << "  -------- " << std::left << std::setw(maxWidth - 5 - 2) << "";
         for (auto it = stages.begin(); it != stages.end(); ++it) {
             os << "  " << std::left << std::setw(9) << "-------";
@@ -147,7 +140,7 @@ class StatsReport {
         string lastName = "__NONE__";
         string lastCommaName = "__NONE__";
         unsigned col = 0;
-        for (ByName::const_iterator it = byName.begin(); it != byName.end(); ++it) {
+        for (auto it = byName.cbegin(); it != byName.cend(); ++it) {
             const V3Statistic* repp = it->second;
             if (lastName != repp->name()) {
                 lastName = repp->name();
@@ -157,10 +150,10 @@ class StatsReport {
                     if ((pos = commaName.find(',')) != string::npos) commaName.erase(pos);
                     if (lastCommaName != commaName) {
                         lastCommaName = commaName;
-                        os << endl;
+                        os << '\n';
                     }
                 }
-                os << endl;
+                os << '\n';
                 col = 0;
                 os << "  " << std::left << std::setw(maxWidth) << repp->name();
             }
@@ -171,7 +164,7 @@ class StatsReport {
             repp->dump(os);
             col++;
         }
-        os << endl;
+        os << '\n';
     }
 
 public:
@@ -186,7 +179,7 @@ public:
         stars();
         stages();
     }
-    ~StatsReport() {}
+    ~StatsReport() = default;
 };
 
 StatsReport::StatColl StatsReport::s_allStats;
@@ -211,9 +204,7 @@ void V3Stats::statsStage(const string& name) {
     static double lastWallTime = -1;
     static int fileNumber = 0;
 
-    char digits[100];
-    sprintf(digits, "%03d", ++fileNumber);
-    const string digitName = string(digits) + "_" + name;
+    const string digitName = V3Global::digitsFilename(++fileNumber) + "_" + name;
 
     double wallTime = V3Os::timeUsecs() / 1.0e6;
     if (lastWallTime < 0) lastWallTime = wallTime;
