@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -25,6 +25,7 @@
 
 size_t VName::s_minLength = 32;
 size_t VName::s_maxLength = 0;  // Disabled
+std::map<string, string> VName::s_dehashMap;
 
 //######################################################################
 // Wildcard
@@ -306,7 +307,7 @@ uint64_t VHashSha256::digestUInt64() {
 }
 
 string VHashSha256::digestHex() {
-    static const char digits[16 + 1] = "0123456789abcdef";
+    static const char* const digits = "0123456789abcdef";
     const string& binhash = digestBinary();
     string out;
     out.reserve(70);
@@ -322,7 +323,7 @@ string VHashSha256::digestSymbol() {
     // has + and / for last two digits, but need C symbol, and we also
     // avoid conflicts with use of _, so use "AB" at the end.
     // Thus this function is non-reversible.
-    static const char digits[64 + 1]
+    static const char* const digits
         = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789AB";
     const string& binhash = digestBinary();
     string out;
@@ -345,13 +346,13 @@ void VHashSha256::selfTestOne(const string& data, const string& data2, const str
     VHashSha256 digest(data);
     if (data2 != "") digest.insert(data2);
     if (VL_UNCOVERABLE(digest.digestHex() != exp)) {
-        std::cerr << "%Error: When hashing '" << data + data2 << "'" << endl  // LCOV_EXCL_LINE
-                  << "        ... got=" << digest.digestHex() << endl  // LCOV_EXCL_LINE
+        std::cerr << "%Error: When hashing '" << data + data2 << "'\n"  // LCOV_EXCL_LINE
+                  << "        ... got=" << digest.digestHex() << '\n'  // LCOV_EXCL_LINE
                   << "        ... exp=" << exp << endl;  // LCOV_EXCL_LINE
     }
     if (VL_UNCOVERABLE(digest.digestSymbol() != exp64)) {
-        std::cerr << "%Error: When hashing '" << data + data2 << "'" << endl  // LCOV_EXCL_LINE
-                  << "        ... got=" << digest.digestSymbol() << endl  // LCOV_EXCL_LINE
+        std::cerr << "%Error: When hashing '" << data + data2 << "'\n"  // LCOV_EXCL_LINE
+                  << "        ... got=" << digest.digestSymbol() << '\n'  // LCOV_EXCL_LINE
                   << "        ... exp=" << exp64 << endl;  // LCOV_EXCL_LINE
     }
 }
@@ -378,6 +379,53 @@ void VHashSha256::selfTest() {
 //######################################################################
 // VName
 
+string VName::dehash(const string& in) {
+    static const char VHSH[] = "__Vhsh";
+    static const size_t DOT_LEN = strlen("__DOT__");
+    std::string dehashed;
+
+    // Need to split 'in' into components separated by __DOT__, 'last_dot_pos'
+    // keeps track of the position after the most recently found instance of __DOT__
+    for (string::size_type last_dot_pos = 0; last_dot_pos < in.size();) {
+        const string::size_type next_dot_pos = in.find("__DOT__", last_dot_pos);
+        // Two iterators defining the range between the last and next dots.
+        auto search_begin = std::begin(in) + last_dot_pos;
+        auto search_end
+            = next_dot_pos == string::npos ? std::end(in) : std::begin(in) + next_dot_pos;
+
+        // Search for __Vhsh between the two dots.
+        auto begin_vhsh
+            = std::search(search_begin, search_end, std::begin(VHSH), std::end(VHSH) - 1);
+        if (begin_vhsh != search_end) {
+            std::string vhsh(begin_vhsh, search_end);
+            const auto& it = s_dehashMap.find(vhsh);
+            UASSERT(it != s_dehashMap.end(), "String not in reverse hash map '" << vhsh << "'");
+            // Is this not the first component, but the first to require dehashing?
+            if (last_dot_pos > 0 && dehashed.empty()) {
+                // Seed 'dehashed' with the previously processed components.
+                dehashed = in.substr(0, last_dot_pos);
+            }
+            // Append the unhashed part of the component.
+            dehashed += std::string(search_begin, begin_vhsh);
+            // Append the bit that was lost to truncation but retrieved from the dehash map.
+            dehashed += it->second;
+        }
+        // This component doesn't need dehashing but a previous one might have.
+        else if (!dehashed.empty()) {
+            dehashed += std::string(search_begin, search_end);
+        }
+
+        if (next_dot_pos != string::npos) {
+            // Is there a __DOT__ to add to the dehashed version of 'in'?
+            if (!dehashed.empty()) { dehashed += "__DOT__"; }
+            last_dot_pos = next_dot_pos + DOT_LEN;
+        } else {
+            last_dot_pos = string::npos;
+        }
+    }
+    return dehashed.empty() ? in : dehashed;
+}
+
 string VName::hashedName() {
     if (m_name == "") return "";
     if (m_hashed != "") return m_hashed;  // Memoized
@@ -388,8 +436,10 @@ string VName::hashedName() {
         VHashSha256 hash(m_name);
         string suffix = "__Vhsh" + hash.digestSymbol();
         if (s_minLength < s_maxLength) {
+            s_dehashMap[suffix] = m_name.substr(s_minLength);
             m_hashed = m_name.substr(0, s_minLength) + suffix;
         } else {
+            s_dehashMap[suffix] = m_name;
             m_hashed = suffix;
         }
         return m_hashed;
@@ -408,9 +458,9 @@ VSpellCheck::EditDistance VSpellCheck::editDistance(const string& s, const strin
     if (sLen >= LENGTH_LIMIT) return sLen;
     if (tLen >= LENGTH_LIMIT) return tLen;
 
-    static EditDistance s_v_two_ago[LENGTH_LIMIT + 1];
-    static EditDistance s_v_one_ago[LENGTH_LIMIT + 1];
-    static EditDistance s_v_next[LENGTH_LIMIT + 1];
+    static std::array<EditDistance, LENGTH_LIMIT + 1> s_v_two_ago;
+    static std::array<EditDistance, LENGTH_LIMIT + 1> s_v_one_ago;
+    static std::array<EditDistance, LENGTH_LIMIT + 1> s_v_next;
 
     for (size_t i = 0; i < sLen + 1; i++) s_v_one_ago[i] = i;
 
@@ -448,7 +498,7 @@ VSpellCheck::EditDistance VSpellCheck::cutoffDistance(size_t goal_len, size_t ca
     return (max_length + 2) / 3;
 }
 
-string VSpellCheck::bestCandidateInfo(const string& goal, EditDistance& distancer) {
+string VSpellCheck::bestCandidateInfo(const string& goal, EditDistance& distancer) const {
     string bestCandidate;
     size_t gLen = goal.length();
     distancer = LENGTH_LIMIT * 10;
@@ -488,10 +538,10 @@ void VSpellCheck::selfTestSuggestOne(bool matches, const string& c, const string
     speller.pushCandidate(c);
     string got = speller.bestCandidateInfo(goal, gdist /*ref*/);
     if (matches) {
-        UASSERT_SELFTEST(string, got, c);
+        UASSERT_SELFTEST(const string&, got, c);
         UASSERT_SELFTEST(EditDistance, gdist, dist);
     } else {
-        UASSERT_SELFTEST(string, got, "");
+        UASSERT_SELFTEST(const string&, got, "");
     }
 }
 
