@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -39,13 +39,13 @@
 //######################################################################
 // Graph subclasses
 
-class TaskBaseVertex : public V3GraphVertex {
+class TaskBaseVertex VL_NOT_FINAL : public V3GraphVertex {
     AstNode* m_impurep = nullptr;  // Node causing impure function w/ outside references
     bool m_noInline = false;  // Marked with pragma
 public:
     explicit TaskBaseVertex(V3Graph* graphp)
         : V3GraphVertex{graphp} {}
-    virtual ~TaskBaseVertex() override {}
+    virtual ~TaskBaseVertex() override = default;
     bool pure() const { return m_impurep == nullptr; }
     AstNode* impureNode() const { return m_impurep; }
     void impure(AstNode* nodep) { m_impurep = nodep; }
@@ -53,7 +53,7 @@ public:
     void noInline(bool flag) { m_noInline = flag; }
 };
 
-class TaskFTaskVertex : public TaskBaseVertex {
+class TaskFTaskVertex final : public TaskBaseVertex {
     // Every task gets a vertex, and we link tasks together based on funcrefs.
     AstNodeFTask* m_nodep;
     AstCFunc* m_cFuncp = nullptr;
@@ -62,36 +62,35 @@ public:
     TaskFTaskVertex(V3Graph* graphp, AstNodeFTask* nodep)
         : TaskBaseVertex{graphp}
         , m_nodep{nodep} {}
-    virtual ~TaskFTaskVertex() override {}
+    virtual ~TaskFTaskVertex() override = default;
     AstNodeFTask* nodep() const { return m_nodep; }
     virtual string name() const override { return nodep()->name(); }
     virtual string dotColor() const override { return pure() ? "black" : "red"; }
-    virtual FileLine* fileline() const override { return nodep()->fileline(); }
     AstCFunc* cFuncp() const { return m_cFuncp; }
     void cFuncp(AstCFunc* nodep) { m_cFuncp = nodep; }
 };
 
-class TaskCodeVertex : public TaskBaseVertex {
+class TaskCodeVertex final : public TaskBaseVertex {
     // Top vertex for all calls not under another task
 public:
     explicit TaskCodeVertex(V3Graph* graphp)
         : TaskBaseVertex{graphp} {}
-    virtual ~TaskCodeVertex() override {}
+    virtual ~TaskCodeVertex() override = default;
     virtual string name() const override { return "*CODE*"; }
     virtual string dotColor() const override { return "green"; }
 };
 
-class TaskEdge : public V3GraphEdge {
+class TaskEdge final : public V3GraphEdge {
 public:
     TaskEdge(V3Graph* graphp, TaskBaseVertex* fromp, TaskBaseVertex* top)
         : V3GraphEdge{graphp, fromp, top, 1, false} {}
-    virtual ~TaskEdge() override {}
+    virtual ~TaskEdge() override = default;
     virtual string dotLabel() const override { return "w" + cvtToStr(weight()); }
 };
 
 //######################################################################
 
-class TaskStateVisitor : public AstNVisitor {
+class TaskStateVisitor final : public AstNVisitor {
 private:
     // NODE STATE
     //  Output:
@@ -103,9 +102,8 @@ private:
     AstUser4InUse m_inuser4;
 
     // TYPES
-    typedef std::map<std::pair<AstScope*, AstVar*>, AstVarScope*> VarToScopeMap;
-    typedef std::map<const AstNodeFTask*, AstClass*> FuncToClassMap;
-    typedef std::vector<AstInitial*> Initials;
+    using VarToScopeMap = std::map<std::pair<AstScope*, AstVar*>, AstVarScope*>;
+    using FuncToClassMap = std::unordered_map<const AstNodeFTask*, AstClass*>;
     // MEMBERS
     VarToScopeMap m_varToScopeMap;  // Map for Var -> VarScope mappings
     FuncToClassMap m_funcToClassMap;  // Map for ctor func -> class
@@ -114,7 +112,7 @@ private:
     AstClass* m_classp = nullptr;  // Current class
     V3Graph m_callGraph;  // Task call graph
     TaskBaseVertex* m_curVxp;  // Current vertex we're adding to
-    Initials m_initialps;  // Initial blocks to move
+    std::vector<AstInitial*> m_initialps;  // Initial blocks to move
 
 public:
     // METHODS
@@ -124,7 +122,7 @@ public:
         return scopep;
     }
     AstVarScope* findVarScope(AstScope* scopep, AstVar* nodep) {
-        const auto iter = m_varToScopeMap.find(make_pair(scopep, nodep));
+        const auto iter = m_varToScopeMap.find(std::make_pair(scopep, nodep));
         UASSERT_OBJ(iter != m_varToScopeMap.end(), nodep, "No scope for var");
         return iter->second;
     }
@@ -146,11 +144,11 @@ public:
         if (!vxp->pure()) {
             nodep->v3warn(
                 IMPURE, "Unsupported: External variable referenced by non-inlined function/task: "
-                            << nodep->prettyNameQ() << endl
-                            << nodep->warnContextPrimary() << endl
+                            << nodep->prettyNameQ() << '\n'
+                            << nodep->warnContextPrimary() << '\n'
                             << vxp->impureNode()->warnOther()
                             << "... Location of the external reference: "
-                            << vxp->impureNode()->prettyNameQ() << endl
+                            << vxp->impureNode()->prettyNameQ() << '\n'
                             << vxp->impureNode()->warnContextSecondary());
         }
         // And, we need to check all tasks this task calls
@@ -182,7 +180,7 @@ private:
         }
         // Likewise, all FTask->scope mappings
         for (AstNode* stmtp = nodep->blocksp(); stmtp; stmtp = stmtp->nextp()) {
-            if (AstNodeFTask* taskp = VN_CAST(stmtp, NodeFTask)) { taskp->user3p(nodep); }
+            if (AstNodeFTask* taskp = VN_CAST(stmtp, NodeFTask)) taskp->user3p(nodep);
         }
         iterateChildren(nodep);
     }
@@ -206,18 +204,19 @@ private:
     }
     virtual void visit(AstNodeFTask* nodep) override {
         UINFO(9, "  TASK " << nodep << endl);
-        TaskBaseVertex* lastVxp = m_curVxp;
-        m_curVxp = getFTaskVertex(nodep);
-        if (nodep->dpiImport()) m_curVxp->noInline(true);
-        if (nodep->classMethod()) m_curVxp->noInline(true);  // Until V3Task supports it
-        if (nodep->isConstructor()) {
-            m_curVxp->noInline(true);
-            m_ctorp = nodep;
-            UASSERT_OBJ(m_classp, nodep, "Ctor not under class");
-            m_funcToClassMap[nodep] = m_classp;
+        {
+            VL_RESTORER(m_curVxp);
+            m_curVxp = getFTaskVertex(nodep);
+            if (nodep->dpiImport()) m_curVxp->noInline(true);
+            if (nodep->classMethod()) m_curVxp->noInline(true);  // Until V3Task supports it
+            if (nodep->isConstructor()) {
+                m_curVxp->noInline(true);
+                m_ctorp = nodep;
+                UASSERT_OBJ(m_classp, nodep, "Ctor not under class");
+                m_funcToClassMap[nodep] = m_classp;
+            }
+            iterateChildren(nodep);
         }
-        iterateChildren(nodep);
-        m_curVxp = lastVxp;
     }
     virtual void visit(AstPragma* nodep) override {
         if (nodep->pragType() == AstPragmaType::NO_INLINE_TASK) {
@@ -281,13 +280,13 @@ public:
         m_callGraph.removeRedundantEdgesSum(&TaskEdge::followAlwaysTrue);
         m_callGraph.dumpDotFilePrefixed("task_call");
     }
-    virtual ~TaskStateVisitor() override {}
+    virtual ~TaskStateVisitor() override = default;
     VL_UNCOPYABLE(TaskStateVisitor);
 };
 
 //######################################################################
 
-class TaskRelinkVisitor : public AstNVisitor {
+class TaskRelinkVisitor final : public AstNVisitor {
     // Replace varrefs with new var pointer
 private:
     // NODE STATE
@@ -317,13 +316,57 @@ public:
     explicit TaskRelinkVisitor(AstBegin* nodep) {  // Passed temporary tree
         iterate(nodep);
     }
-    virtual ~TaskRelinkVisitor() override {}
+    virtual ~TaskRelinkVisitor() override = default;
+};
+
+//######################################################################
+// DPI related utility functions
+
+struct TaskDpiUtils {
+    static std::vector<std::pair<AstUnpackArrayDType*, int>>
+    unpackDimsAndStrides(AstNodeDType* dtypep) {
+        std::vector<std::pair<AstUnpackArrayDType*, int>> dimStrides;
+        if (AstUnpackArrayDType* unpackp = VN_CAST(dtypep->skipRefp(), UnpackArrayDType)) {
+            const std::vector<AstUnpackArrayDType*> dims = unpackp->unpackDimensions();
+            dimStrides.resize(dims.size(), {nullptr, 0});
+            dimStrides.back() = {dims.back(), 1};
+            for (ssize_t i = dims.size() - 2; i >= 0; --i) {
+                dimStrides[i].first = dims[i];
+                dimStrides[i].second = dimStrides[i + 1].second * dims[i + 1]->elementsConst();
+            }
+        }
+        return dimStrides;
+    }
+    static bool dpiToInternalFrStmt(AstVar* portp, const string& frName, string& frstmt,
+                                    string& ket) {
+        ket.clear();
+        if (portp->basicp() && portp->basicp()->keyword() == AstBasicDTypeKwd::CHANDLE) {
+            frstmt = "VL_CVT_VP_Q(" + frName;
+            ket = ")";
+        } else if ((portp->basicp() && portp->basicp()->isDpiPrimitive())) {
+            frstmt = frName;
+        } else {
+            const string frSvType = portp->basicp()->isDpiBitVec() ? "SVBV" : "SVLV";
+            if (portp->isWide()) {
+                // Need to convert to wide, using special function
+                frstmt = "VL_SET_W_" + frSvType + "(" + cvtToStr(portp->width()) + ",";
+                return true;
+            } else {
+                const AstNodeDType* dtypep = portp->dtypep()->skipRefp();
+                frstmt = "VL_SET_" + string(dtypep->charIQWN()) + "_" + frSvType + "(";
+                if (VN_IS(dtypep, UnpackArrayDType)) frstmt += "&";
+                frstmt += frName;
+                ket = ")";
+            }
+        }
+        return false;
+    }
 };
 
 //######################################################################
 // Task state, as a visitor of each AstNode
 
-class TaskVisitor : public AstNVisitor {
+class TaskVisitor final : public AstNVisitor {
 private:
     // NODE STATE
     // Each module:
@@ -341,7 +384,7 @@ private:
         IM_AFTER,  // Pointing at last inserted stmt, insert after
         IM_WHILE_PRECOND  // Pointing to for loop, add to body end
     };
-    typedef std::map<const string, std::pair<AstNodeFTask*, string>> DpiNames;
+    using DpiNames = std::map<const string, std::pair<AstNodeFTask*, std::string>>;
 
     // STATE
     TaskStateVisitor* m_statep;  // Common state between visitors
@@ -641,11 +684,7 @@ private:
     }
 
     AstNode* createDpiTemp(AstVar* portp, const string& suffix) {
-        string stmt = portp->dpiArgType(false, true) + " " + portp->name() + suffix;
-        if (!portp->basicp()->isDpiPrimitive()) {
-            stmt += "[" + cvtToStr(portp->widthWords()) + "]";
-        }
-        stmt += ";\n";
+        const string stmt = portp->dpiTmpVarType(portp->name() + suffix) + ";\n";
         return new AstCStmt(portp->fileline(), stmt);
     }
 
@@ -657,28 +696,64 @@ private:
 
     AstNode* createAssignDpiToInternal(AstVarScope* portvscp, const string& frName) {
         // Create assignment from DPI temporary into internal format
+        // DPI temporary is scalar or 1D array (if unpacked array)
+        // Internal representation is scalar, 1D, or multi-dimensional array (similar to SV)
         AstVar* portp = portvscp->varp();
         string frstmt;
-        bool useSetWSvlv = V3Task::dpiToInternalFrStmt(portp, frName, frstmt);
-        if (useSetWSvlv) {
-            AstNode* linesp = new AstText(portp->fileline(), frstmt);
-            linesp->addNext(new AstVarRef(portp->fileline(), portvscp, VAccess::WRITE));
-            linesp->addNext(new AstText(portp->fileline(), "," + frName + ");"));
-            return new AstCStmt(portp->fileline(), linesp);
-        }
+        string ket;
+        const bool useSetWSvlv = TaskDpiUtils::dpiToInternalFrStmt(portp, frName, frstmt, ket);
         // Use a AstCMath, as we want V3Clean to mask off bits that don't make sense.
         int cwidth = VL_IDATASIZE;
-        if (portp->basicp()) {
+        if (!useSetWSvlv && portp->basicp()) {
             if (portp->basicp()->keyword().isBitLogic()) {
                 cwidth = VL_EDATASIZE * portp->widthWords();
             } else {
                 cwidth = portp->basicp()->keyword().width();
             }
         }
-        AstNode* newp = new AstAssign(
-            portp->fileline(), new AstVarRef(portp->fileline(), portvscp, VAccess::WRITE),
-            new AstSel(portp->fileline(), new AstCMath(portp->fileline(), frstmt, cwidth, false),
-                       0, portp->width()));
+
+        const std::vector<std::pair<AstUnpackArrayDType*, int>> dimStrides
+            = TaskDpiUtils::unpackDimsAndStrides(portp->dtypep());
+        const int total = dimStrides.empty() ? 1
+                                             : dimStrides.front().first->elementsConst()
+                                                   * dimStrides.front().second;
+        AstNode* newp = nullptr;
+        const int widthWords = portp->basicp()->widthWords();
+        for (int i = 0; i < total; ++i) {
+            AstNode* srcp = new AstVarRef(portvscp->fileline(), portvscp, VAccess::WRITE);
+            // extract a scalar from multi-dimensional array (internal format)
+            for (auto&& dimStride : dimStrides) {
+                const size_t dimIdx = (i / dimStride.second) % dimStride.first->elementsConst();
+                srcp = new AstArraySel(portvscp->fileline(), srcp, dimIdx);
+            }
+            AstNode* stmtp = nullptr;
+            // extract a scalar from DPI temporary var that is scalar or 1D array
+            if (useSetWSvlv) {
+                AstNode* linesp = new AstText(portvscp->fileline(), frstmt + ket);
+                linesp->addNext(srcp);
+                linesp->addNext(
+                    new AstText(portvscp->fileline(),
+                                "," + frName + " + " + cvtToStr(i * widthWords) + ");\n"));
+                stmtp = new AstCStmt(portvscp->fileline(), linesp);
+            } else {
+                string from = frstmt;
+                if (!dimStrides.empty()) {
+                    // e.g. time is 64bit svLogicVector
+                    const int coef = portp->basicp()->isDpiLogicVec() ? widthWords : 1;
+                    from += "[" + cvtToStr(i * coef) + "]";
+                }
+                from += ket;
+                AstNode* rhsp = new AstSel(portp->fileline(),
+                                           new AstCMath(portp->fileline(), from, cwidth, false), 0,
+                                           portp->width());
+                stmtp = new AstAssign(portp->fileline(), srcp, rhsp);
+            }
+            if (i > 0) {
+                newp->addNext(stmtp);
+            } else {
+                newp = stmtp;
+            }
+        }
         return newp;
     }
 
@@ -709,8 +784,8 @@ private:
             // Static doesn't need save-restore as if below will re-fill proper value
             stmt += "static int __Vfuncnum = -1;\n";
             // First time init (faster than what the compiler does if we did a singleton
-            stmt += "if (VL_UNLIKELY(__Vfuncnum==-1)) { __Vfuncnum = Verilated::exportFuncNum(\""
-                    + nodep->cname() + "\"); }\n";
+            stmt += "if (VL_UNLIKELY(__Vfuncnum == -1)) __Vfuncnum = Verilated::exportFuncNum(\""
+                    + nodep->cname() + "\");\n";
             // If the find fails, it will throw an error
             stmt += "const VerilatedScope* __Vscopep = Verilated::dpiScope();\n";
             // If dpiScope is fails and is null; the exportFind function throws and error
@@ -749,7 +824,10 @@ private:
 
                     if (portp->isNonOutput()) {
                         std::string frName
-                            = portp->isInoutish() && portp->basicp()->isDpiPrimitive() ? "*" : "";
+                            = portp->isInoutish() && portp->basicp()->isDpiPrimitive()
+                                      && portp->dtypep()->skipRefp()->arrayUnpackedElements() == 1
+                                  ? "*"
+                                  : "";
                         frName += portp->name();
                         dpip->addStmtsp(createAssignDpiToInternal(outvscp, frName));
                     }
@@ -832,16 +910,16 @@ private:
         // as it's legal for the user to attach multiple tasks to one dpi cname
         const auto iter = m_dpiNames.find(nodep->cname());
         if (iter == m_dpiNames.end()) {
-            m_dpiNames.insert(make_pair(nodep->cname(), make_pair(nodep, dpiproto)));
+            m_dpiNames.emplace(nodep->cname(), std::make_pair(nodep, dpiproto));
             return false;
         } else if (iter->second.second != dpiproto) {
             nodep->v3error(
                 "Duplicate declaration of DPI function with different formal arguments: "
-                << nodep->prettyNameQ() << endl
-                << nodep->warnContextPrimary() << endl
-                << nodep->warnMore() << "... New prototype:      " << dpiproto << endl
+                << nodep->prettyNameQ() << '\n'
+                << nodep->warnContextPrimary() << '\n'
+                << nodep->warnMore() << "... New prototype:      " << dpiproto << '\n'
                 << iter->second.first->warnOther()
-                << "... Original prototype: " << iter->second.second << endl
+                << "... Original prototype: " << iter->second.second << '\n'
                 << iter->second.first->warnContextSecondary());
             return true;
         } else {
@@ -862,7 +940,7 @@ private:
                         portp->v3warn(
                             E_UNSUPPORTED,
                             "Unsupported: DPI argument of type "
-                                << portp->basicp()->prettyTypeName() << endl
+                                << portp->basicp()->prettyTypeName() << '\n'
                                 << portp->warnMore()
                                 << "... For best portability, use bit, byte, int, or longint");
                         // We don't warn on logic either, although the 4-stateness is lost.
@@ -913,7 +991,7 @@ private:
                         args += "&" + name;
                     } else {
                         if (portp->isWritable() && portp->basicp()->isDpiPrimitive()) {
-                            args += "&";
+                            if (!VN_IS(portp->dtypep()->skipRefp(), UnpackArrayDType)) args += "&";
                         }
 
                         args += portp->name() + tmpSuffixp;
@@ -1378,7 +1456,7 @@ public:
         : m_statep{statep} {
         iterate(nodep);
     }
-    virtual ~TaskVisitor() override {}
+    virtual ~TaskVisitor() override = default;
 };
 
 //######################################################################
@@ -1390,8 +1468,7 @@ V3TaskConnects V3Task::taskConnects(AstNodeFTaskRef* nodep, AstNode* taskStmtsp)
     // Missing pin/expr?  We return (pinvar, nullptr)
     // Extra   pin/expr?  We clean it up
 
-    typedef std::map<const string, int> NameToIndex;
-    NameToIndex nameToIndex;
+    std::map<const std::string, int> nameToIndex;
     V3TaskConnects tconnects;
     UASSERT_OBJ(nodep->taskp(), nodep, "unlinked");
 
@@ -1401,9 +1478,9 @@ V3TaskConnects V3Task::taskConnects(AstNodeFTaskRef* nodep, AstNode* taskStmtsp)
     for (AstNode* stmtp = taskStmtsp; stmtp; stmtp = stmtp->nextp()) {
         if (AstVar* portp = VN_CAST(stmtp, Var)) {
             if (portp->isIO()) {
-                tconnects.push_back(make_pair(portp, static_cast<AstArg*>(nullptr)));
+                tconnects.push_back(std::make_pair(portp, static_cast<AstArg*>(nullptr)));
                 nameToIndex.insert(
-                    make_pair(portp->name(), tpinnum));  // For name based connections
+                    std::make_pair(portp->name(), tpinnum));  // For name based connections
                 tpinnum++;
                 if (portp->attrSFormat()) {
                     sformatp = portp;
@@ -1443,7 +1520,7 @@ V3TaskConnects V3Task::taskConnects(AstNodeFTaskRef* nodep, AstNode* taskStmtsp)
         } else {  // By pin number
             if (ppinnum >= tpinnum) {
                 if (sformatp) {
-                    tconnects.push_back(make_pair(sformatp, static_cast<AstArg*>(nullptr)));
+                    tconnects.push_back(std::make_pair(sformatp, static_cast<AstArg*>(nullptr)));
                     tconnects[ppinnum].second = argp;
                     tpinnum++;
                 } else {
@@ -1532,55 +1609,113 @@ V3TaskConnects V3Task::taskConnects(AstNodeFTaskRef* nodep, AstNode* taskStmtsp)
 string V3Task::assignInternalToDpi(AstVar* portp, bool isPtr, const string& frSuffix,
                                    const string& toSuffix, const string& frPrefix) {
     // Create assignment from internal format into DPI temporary
+    // Internal representation is scalar, 1D, or multi-dimensional array (similar to SV)
+    // DPI temporary is scalar or 1D array (if unpacked array)
     string stmt;
     string ket;
     // Someday we'll have better type support, and this can make variables and casts.
     // But for now, we'll just text-bash it.
     string frName = frPrefix + portp->name() + frSuffix;
     string toName = portp->name() + toSuffix;
-    if (portp->basicp()->isDpiBitVec()) {
-        stmt += ("VL_SET_SVBV_" + string(portp->dtypep()->charIQWN()) + "("
-                 + cvtToStr(portp->width()) + ", " + toName + ", " + frName + ")");
-    } else if (portp->basicp()->isDpiLogicVec()) {
-        stmt += ("VL_SET_SVLV_" + string(portp->dtypep()->charIQWN()) + "("
-                 + cvtToStr(portp->width()) + ", " + toName + ", " + frName + ")");
+    size_t unpackSize = 1;  // non-unpacked array is treated as size 1
+    int unpackDim = 0;
+    if (AstUnpackArrayDType* unpackp = VN_CAST(portp->dtypep()->skipRefp(), UnpackArrayDType)) {
+        unpackSize = unpackp->arrayUnpackedElements();
+        unpackDim = unpackp->dimensions(false).second;
+        if (unpackDim > 0) UASSERT_OBJ(unpackSize > 0, portp, "size must be greater than 0");
+    }
+    if (portp->basicp()->isDpiBitVec() || portp->basicp()->isDpiLogicVec()) {
+        const bool isBit = portp->basicp()->isDpiBitVec();
+        const string idx = portp->name() + "__Vidx";
+        stmt = "for (size_t " + idx + " = 0; " + idx + " < " + cvtToStr(unpackSize) + "; ++" + idx
+               + ") ";
+        stmt += (isBit ? "VL_SET_SVBV_" : "VL_SET_SVLV_")
+                + string(portp->dtypep()->skipRefp()->charIQWN()) + "(" + cvtToStr(portp->width())
+                + ", ";
+        stmt += toName + " + " + cvtToStr(portp->dtypep()->skipRefp()->widthWords()) + " * " + idx
+                + ", ";
+        if (unpackDim > 0) {  // Access multi-dimensional array as a 1D array
+            stmt += "(&" + frName;
+            for (int i = 0; i < unpackDim; ++i) stmt += "[0]";
+            stmt += ")[" + idx + "])";
+        } else {
+            stmt += frName + ")";
+        }
     } else {
-        if (isPtr) stmt += "*";  // DPI outputs are pointers
-        stmt += toName + " = ";
-        if (portp->basicp() && portp->basicp()->keyword() == AstBasicDTypeKwd::CHANDLE) {
+        const bool isChandle
+            = portp->basicp() && portp->basicp()->keyword() == AstBasicDTypeKwd::CHANDLE;
+        const bool isString
+            = portp->basicp() && portp->basicp()->keyword() == AstBasicDTypeKwd::STRING;
+        const string idx = portp->name() + "__Vidx";
+        stmt = "for (size_t " + idx + " = 0; " + idx + " < " + cvtToStr(unpackSize) + "; ++" + idx
+               + ") ";
+        if (unpackDim > 0) {
+            stmt += toName + "[" + idx + "]";
+        } else {
+            if (isPtr) stmt += "*";  // DPI outputs are pointers
+            stmt += toName;
+        }
+        stmt += " = ";
+        if (isChandle) {
             stmt += "VL_CVT_Q_VP(";
             ket += ")";
         }
-        stmt += frName;
-        if (portp->basicp() && portp->basicp()->keyword() == AstBasicDTypeKwd::STRING) {
-            stmt += ".c_str()";
+        if (unpackDim > 0) {
+            stmt += "(&" + frName;
+            for (int i = 0; i < unpackDim; ++i) stmt += "[0]";
+            stmt += ")[" + idx + "]";
+        } else {
+            stmt += frName;
         }
+        if (isString) stmt += ".c_str()";
     }
     stmt += ket + ";\n";
     return stmt;
 }
 
-bool V3Task::dpiToInternalFrStmt(AstVar* portp, const string& frName, string& frstmt) {
-    if (portp->basicp() && portp->basicp()->keyword() == AstBasicDTypeKwd::CHANDLE) {
-        frstmt = "VL_CVT_VP_Q(" + frName + ")";
-    } else if ((portp->basicp() && portp->basicp()->isDpiPrimitive())) {
-        frstmt = frName;
-    } else {
-        const string frSvType = portp->basicp()->isDpiBitVec() ? "SVBV" : "SVLV";
-        if (portp->isWide()) {
-            // Need to convert to wide, using special function
-            frstmt = "VL_SET_W_" + frSvType + "(" + cvtToStr(portp->width()) + ",";
-            return true;
+string V3Task::assignDpiToInternal(const string& lhsName, AstVar* varp) {
+    // Create assignment from DPI temporary into internal format
+    // DPI temporary is scalar or 1D array (if unpacked array)
+    // Internal representation is scalar, 1D, or multi-dimensional array (similar to SV)
+    const string frName = varp->name();
+    string frstmt;
+    string ket;
+    const bool useSetWSvlv = TaskDpiUtils::dpiToInternalFrStmt(varp, frName, frstmt, ket);
+
+    const std::vector<std::pair<AstUnpackArrayDType*, int>> dimStrides
+        = TaskDpiUtils::unpackDimsAndStrides(varp->dtypep());
+    const int total = dimStrides.empty()
+                          ? 1
+                          : dimStrides.front().first->elementsConst() * dimStrides.front().second;
+    const int widthWords = varp->basicp()->widthWords();
+    string statements;
+    for (int i = 0; i < total; ++i) {
+        string lhs = lhsName;
+        // extract a scalar from multi-dimensional array (internal format)
+        for (auto&& dimStride : dimStrides) {
+            const size_t dimIdx = (i / dimStride.second) % dimStride.first->elementsConst();
+            lhs += "[" + cvtToStr(dimIdx) + "]";
+        }
+        // extract a scalar from DPI temporary var that is scalar or 1D array
+        if (useSetWSvlv) {
+            statements += frstmt + ket + " " + lhs + ", " + frName + " + "
+                          + cvtToStr(i * widthWords) + ");\n";
         } else {
-            frstmt = "VL_SET_" + string(portp->dtypep()->charIQWN()) + "_" + frSvType + "("
-                     + frName + ")";
+            string rhs = frstmt;
+            if (!dimStrides.empty()) {
+                // e.g. time is 64bit svLogicVector
+                const int coef = varp->basicp()->isDpiLogicVec() ? widthWords : 1;
+                rhs += "[" + cvtToStr(i * coef) + "]";
+            }
+            rhs += ket;
+            statements += lhs + " = " + rhs + ";\n";
         }
     }
-    return false;
+    return statements;
 }
 
 const char* V3Task::dpiTemporaryVarSuffix() {
-    static const char suffix[] = "__Vcvt";
+    static const char* const suffix = "__Vcvt";
     return suffix;
 }
 
