@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -46,12 +46,13 @@
 #include "V3Stats.h"
 
 #include <map>
+#include <limits>
 #include <set>
 
 //######################################################################
 // Graph vertexes
 
-class TraceActivityVertex : public V3GraphVertex {
+class TraceActivityVertex final : public V3GraphVertex {
     AstNode* const m_insertp;
     vlsint32_t m_activityCode;
     bool m_slow;  // If always slow, we can use the same code
@@ -71,7 +72,7 @@ public:
         m_activityCode = code;
         m_slow = false;
     }
-    virtual ~TraceActivityVertex() override {}
+    virtual ~TraceActivityVertex() override = default;
     // ACCESSORS
     AstNode* insertp() const {
         if (!m_insertp) v3fatalSrc("Null insertp; probably called on a special always/slow.");
@@ -95,14 +96,14 @@ public:
     }
 };
 
-class TraceCFuncVertex : public V3GraphVertex {
+class TraceCFuncVertex final : public V3GraphVertex {
     AstCFunc* m_nodep;
 
 public:
     TraceCFuncVertex(V3Graph* graphp, AstCFunc* nodep)
         : V3GraphVertex{graphp}
         , m_nodep{nodep} {}
-    virtual ~TraceCFuncVertex() override {}
+    virtual ~TraceCFuncVertex() override = default;
     // ACCESSORS
     AstCFunc* nodep() const { return m_nodep; }
     virtual string name() const override { return nodep()->name(); }
@@ -110,7 +111,7 @@ public:
     virtual FileLine* fileline() const override { return nodep()->fileline(); }
 };
 
-class TraceTraceVertex : public V3GraphVertex {
+class TraceTraceVertex final : public V3GraphVertex {
     AstTraceDecl* const m_nodep;  // TRACEINC this represents
     // nullptr, or other vertex with the real code() that duplicates this one
     TraceTraceVertex* m_duplicatep = nullptr;
@@ -119,7 +120,7 @@ public:
     TraceTraceVertex(V3Graph* graphp, AstTraceDecl* nodep)
         : V3GraphVertex{graphp}
         , m_nodep{nodep} {}
-    virtual ~TraceTraceVertex() override {}
+    virtual ~TraceTraceVertex() override = default;
     // ACCESSORS
     AstTraceDecl* nodep() const { return m_nodep; }
     virtual string name() const override { return nodep()->name(); }
@@ -132,14 +133,14 @@ public:
     }
 };
 
-class TraceVarVertex : public V3GraphVertex {
+class TraceVarVertex final : public V3GraphVertex {
     AstVarScope* m_nodep;
 
 public:
     TraceVarVertex(V3Graph* graphp, AstVarScope* nodep)
         : V3GraphVertex{graphp}
         , m_nodep{nodep} {}
-    virtual ~TraceVarVertex() override {}
+    virtual ~TraceVarVertex() override = default;
     // ACCESSORS
     AstVarScope* nodep() const { return m_nodep; }
     virtual string name() const override { return nodep()->name(); }
@@ -150,7 +151,7 @@ public:
 //######################################################################
 // Trace state, as a visitor of each AstNode
 
-class TraceVisitor : public EmitCBaseVisitor {
+class TraceVisitor final : public EmitCBaseVisitor {
 private:
     // NODE STATE
     // V3Hashed
@@ -183,9 +184,9 @@ private:
     VDouble0 m_statUniqCodes;  // Statistic tracking
 
     // All activity numbers applying to a given trace
-    typedef std::set<uint32_t> ActCodeSet;
+    using ActCodeSet = std::set<uint32_t>;
     // For activity set, what traces apply
-    typedef std::multimap<ActCodeSet, TraceTraceVertex*> TraceVec;
+    using TraceVec = std::multimap<ActCodeSet, TraceTraceVertex*>;
 
     // METHODS
     VL_DEBUG_FUNC;  // Declare debug()
@@ -299,7 +300,7 @@ private:
             nextp = itp->verticesNextp();
             if (TraceActivityVertex* const vtxp = dynamic_cast<TraceActivityVertex*>(itp)) {
                 // Leave in the always vertex for later use.
-                if (vtxp != m_alwaysVtxp && !vtxp->outBeginp()) { vtxp->unlinkDelete(&m_graph); }
+                if (vtxp != m_alwaysVtxp && !vtxp->outBeginp()) vtxp->unlinkDelete(&m_graph);
             }
         }
     }
@@ -363,7 +364,7 @@ private:
                     // make slow routines set all activity flags.
                     actSet.erase(TraceActivityVertex::ACTIVITY_SLOW);
                 }
-                traces.insert(make_pair(actSet, vtxp));
+                traces.emplace(actSet, vtxp);
             }
         }
     }
@@ -426,7 +427,7 @@ private:
     void addActivitySetter(AstNode* insertp, uint32_t code) {
         FileLine* const fl = insertp->fileline();
         AstAssign* const setterp = new AstAssign(fl, selectActivity(fl, code, VAccess::WRITE),
-                                                 new AstConst(fl, AstConst::LogicTrue()));
+                                                 new AstConst(fl, AstConst::BitTrue()));
         if (AstCCall* const callp = VN_CAST(insertp, CCall)) {
             callp->addNextHere(setterp);
         } else if (AstCFunc* const funcp = VN_CAST(insertp, CFunc)) {
@@ -447,7 +448,8 @@ private:
         FileLine* const flp = m_topScopep->fileline();
         AstNodeDType* const newScalarDtp = new AstBasicDType(flp, VFlagLogicPacked(), 1);
         v3Global.rootp()->typeTablep()->addTypesp(newScalarDtp);
-        AstRange* const newArange = new AstRange(flp, VNumRange(m_activityNumber - 1, 0, false));
+        AstRange* const newArange
+            = new AstRange{flp, VNumRange{static_cast<int>(m_activityNumber) - 1, 0}};
         AstNodeDType* const newArrDtp = new AstUnpackArrayDType(flp, newScalarDtp, newArange);
         v3Global.rootp()->typeTablep()->addTypesp(newArrDtp);
         AstVar* const newvarp
@@ -603,7 +605,8 @@ private:
             AstCFunc* topFuncp = nullptr;
             AstCFunc* subFuncp = nullptr;
             int subStmts = 0;
-            const uint32_t maxCodes = (nAllCodes + parallelism - 1) / parallelism;
+            uint32_t maxCodes = (nAllCodes + parallelism - 1) / parallelism;
+            if (maxCodes < 1) maxCodes = 1;
             uint32_t nCodes = 0;
             const ActCodeSet* prevActSet = nullptr;
             AstIf* ifp = nullptr;
@@ -643,7 +646,7 @@ private:
                         }
                     }
                     ifp = new AstIf(flp, condp, nullptr, nullptr);
-                    if (!always) { ifp->branchPred(VBranchPred::BP_UNLIKELY); }
+                    if (!always) ifp->branchPred(VBranchPred::BP_UNLIKELY);
                     subFuncp->addStmtsp(ifp);
                     subStmts += EmitCBaseCounterVisitor(ifp).count();
                     prevActSet = &actSet;
@@ -687,7 +690,7 @@ private:
         // Clear fine grained activity flags
         for (uint32_t i = 0; i < m_activityNumber; ++i) {
             AstNode* const clrp = new AstAssign(fl, selectActivity(fl, i, VAccess::WRITE),
-                                                new AstConst(fl, AstConst::LogicFalse()));
+                                                new AstConst(fl, AstConst::BitFalse()));
             cleanupFuncp->addStmtsp(clrp);
         }
     }
