@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -22,6 +22,7 @@
 
 #include "config_build.h"
 #include "verilatedos.h"
+#include "verilated_trace_defs.h"  // For VLT_TRACE_SCOPE_*
 
 #include "V3Global.h"
 #include "V3TraceDecl.h"
@@ -31,7 +32,7 @@
 //######################################################################
 // TraceDecl state, as a visitor of each AstNode
 
-class TraceDeclVisitor : public EmitCBaseVisitor {
+class TraceDeclVisitor final : public EmitCBaseVisitor {
 private:
     // NODE STATE
 
@@ -59,7 +60,7 @@ private:
         if (!varp->isTrace()) {
             return "Verilator trace_off";
         } else if (!nodep->isTrace()) {
-            return "Verilator cell trace_off";
+            return "Verilator instance trace_off";
         } else if (!v3Global.opt.traceUnderscore()) {
             const string prettyName = varp->prettyName();
             if (!prettyName.empty() && prettyName[0] == '_') return "Leading underscore";
@@ -72,7 +73,7 @@ private:
         FileLine* const flp = m_topScopep->fileline();
         AstCFunc* const funcp = new AstCFunc(flp, name, m_topScopep);
         string argTypes("void* userp, " + v3Global.opt.traceClassBase() + "* tracep");
-        if (m_interface) argTypes += ", const char* scopep";
+        if (m_interface) argTypes += ", int scopet, const char* scopep";
         funcp->argTypes(argTypes);
         funcp->funcType(type);
         funcp->symProlog(true);
@@ -84,8 +85,12 @@ private:
     }
     void callCFuncSub(AstCFunc* basep, AstCFunc* funcp, AstIntfRef* irp) {
         AstCCall* callp = new AstCCall(funcp->fileline(), funcp);
-        callp->argTypes("userp, tracep");
-        if (irp) callp->addArgsp(irp->unlinkFrBack());
+        if (irp) {
+            callp->argTypes("userp, tracep, VLT_TRACE_SCOPE_INTERFACE");
+            callp->addArgsp(irp->unlinkFrBack());
+        } else {
+            callp->argTypes("userp, tracep");
+        }
         basep->addStmtsp(callp);
     }
     AstCFunc* newCFuncSub(AstCFunc* basep) {
@@ -94,12 +99,15 @@ private:
         if (!m_interface) callCFuncSub(basep, funcp, nullptr);
         return funcp;
     }
+
+    std::string getScopeChar(VltTraceScope sct) { return std::string(1, (char)(0x80 + sct)); }
+
     void addTraceDecl(const VNumRange& arrayRange,
                       int widthOverride) {  // If !=0, is packed struct/array where basicp size
                                             // misreflects one element
         VNumRange bitRange;
         if (widthOverride) {
-            bitRange = VNumRange(widthOverride - 1, 0, false);
+            bitRange = VNumRange{widthOverride - 1, 0};
         } else if (const AstBasicDType* const bdtypep = m_traValuep->dtypep()->basicp()) {
             bitRange = bdtypep->nrange();
         }
@@ -238,13 +246,13 @@ private:
             } else {
                 // Unroll now, as have no other method to get right signal names
                 AstNodeDType* const subtypep = nodep->subDTypep()->skipRefToEnump();
-                for (int i = nodep->lsb(); i <= nodep->msb(); ++i) {
+                for (int i = nodep->lo(); i <= nodep->hi(); ++i) {
                     VL_RESTORER(m_traShowname);
                     VL_RESTORER(m_traValuep);
                     {
                         m_traShowname += string("(") + cvtToStr(i) + string(")");
                         m_traValuep = new AstArraySel(
-                            nodep->fileline(), m_traValuep->cloneTree(true), i - nodep->lsb());
+                            nodep->fileline(), m_traValuep->cloneTree(true), i - nodep->lo());
 
                         m_traValuep->dtypep(subtypep);
                         iterate(subtypep);
@@ -263,14 +271,14 @@ private:
                 addTraceDecl(VNumRange(), nodep->width());
             } else {
                 AstNodeDType* const subtypep = nodep->subDTypep()->skipRefToEnump();
-                for (int i = nodep->lsb(); i <= nodep->msb(); ++i) {
+                for (int i = nodep->lo(); i <= nodep->hi(); ++i) {
                     VL_RESTORER(m_traShowname);
                     VL_RESTORER(m_traValuep);
                     {
                         m_traShowname += string("(") + cvtToStr(i) + string(")");
-                        m_traValuep = new AstSel(nodep->fileline(), m_traValuep->cloneTree(true),
-                                                 (i - nodep->lsb()) * subtypep->width(),
-                                                 subtypep->width());
+                        m_traValuep
+                            = new AstSel(nodep->fileline(), m_traValuep->cloneTree(true),
+                                         (i - nodep->lo()) * subtypep->width(), subtypep->width());
                         m_traValuep->dtypep(subtypep);
                         iterate(subtypep);
                         VL_DO_CLEAR(m_traValuep->deleteTree(), m_traValuep = nullptr);
@@ -295,8 +303,12 @@ private:
                     VL_RESTORER(m_traShowname);
                     VL_RESTORER(m_traValuep);
                     {
-                        m_traShowname += string(" ") + itemp->prettyName();
                         if (VN_IS(nodep, StructDType)) {
+                            // Mark scope as a struct by setting the last char to 0x80 + the
+                            // fstScopeType
+                            m_traShowname += getScopeChar(VLT_TRACE_SCOPE_STRUCT) + " "
+                                             + itemp->prettyName();
+
                             m_traValuep
                                 = new AstSel(nodep->fileline(), m_traValuep->cloneTree(true),
                                              itemp->lsb(), subtypep->width());
@@ -304,6 +316,8 @@ private:
                             iterate(subtypep);
                             VL_DO_CLEAR(m_traValuep->deleteTree(), m_traValuep = nullptr);
                         } else {  // Else union, replicate fields
+                            m_traShowname
+                                += getScopeChar(VLT_TRACE_SCOPE_UNION) + " " + itemp->prettyName();
                             iterate(subtypep);
                         }
                     }
