@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -39,7 +39,7 @@
 //######################################################################
 // Unroll state, as a visitor of each AstNode
 
-class UnrollVisitor : public AstNVisitor {
+class UnrollVisitor final : public AstNVisitor {
 private:
     // STATE
     AstVar* m_forVarp;  // Iterator variable
@@ -118,11 +118,10 @@ private:
         m_forVarp = VN_CAST(initAssp->lhsp(), VarRef)->varp();
         m_forVscp = VN_CAST(initAssp->lhsp(), VarRef)->varScopep();
         if (VN_IS(nodep, GenFor) && !m_forVarp->isGenVar()) {
-            nodep->v3error("Non-genvar used in generate for: "  //
-                           << m_forVarp->prettyNameQ() << endl);
+            nodep->v3error("Non-genvar used in generate for: " << m_forVarp->prettyNameQ());
         } else if (!VN_IS(nodep, GenFor) && m_forVarp->isGenVar()) {
             nodep->v3error("Genvar not legal in non-generate for (IEEE 1800-2017 27.4): "
-                           << m_forVarp->prettyNameQ() << endl
+                           << m_forVarp->prettyNameQ() << '\n'
                            << nodep->warnMore()
                            << "... Suggest move for loop upwards to generate-level scope.");
         }
@@ -231,6 +230,7 @@ private:
             AstConst new_con(clonep->fileline(), *res);
             new_con.dtypeFrom(dtypep);
             outNum = new_con.num();
+            outNum.isSigned(dtypep->isSigned());
             VL_DO_DANGLING(clonep->deleteTree(), clonep);
             return true;
         }
@@ -365,10 +365,10 @@ private:
         } else {
             nodep->unlinkFrBack();
         }
-        if (bodysp) { VL_DO_DANGLING(pushDeletep(bodysp), bodysp); }
-        if (precondsp) { VL_DO_DANGLING(pushDeletep(precondsp), precondsp); }
-        if (initp) { VL_DO_DANGLING(pushDeletep(initp), initp); }
-        if (incp && !incp->backp()) { VL_DO_DANGLING(pushDeletep(incp), incp); }
+        if (bodysp) VL_DO_DANGLING(pushDeletep(bodysp), bodysp);
+        if (precondsp) VL_DO_DANGLING(pushDeletep(precondsp), precondsp);
+        if (initp) VL_DO_DANGLING(pushDeletep(initp), initp);
+        if (incp && !incp->backp()) VL_DO_DANGLING(pushDeletep(incp), incp);
         if (debug() >= 9 && newbodysp) newbodysp->dumpTree(cout, "-  _new: ");
         return true;
     }
@@ -385,7 +385,7 @@ private:
             // Grab initial value
             AstNode* initp = nullptr;  // Should be statement before the while.
             if (nodep->backp()->nextp() == nodep) initp = nodep->backp();
-            if (initp) { VL_DO_DANGLING(V3Const::constifyEdit(initp), initp); }
+            if (initp) VL_DO_DANGLING(V3Const::constifyEdit(initp), initp);
             if (nodep->backp()->nextp() == nodep) initp = nodep->backp();
             // Grab assignment
             AstNode* incp = nullptr;  // Should be last statement
