@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -36,7 +36,7 @@
 //######################################################################
 // Width state, as a visitor of each AstNode
 
-class WidthSelVisitor : public AstNVisitor {
+class WidthSelVisitor final : public AstNVisitor {
 private:
     // IMPORTANT
     //**** This is not a normal visitor, in that all iteration is instead
@@ -67,7 +67,7 @@ private:
             : m_errp{errp}
             , m_dtypep{dtypep}
             , m_fromRange{fromRange} {}
-        ~FromData() {}
+        ~FromData() = default;
     };
     FromData fromDataForArray(AstNode* nodep, AstNode* basefromp) {
         // What is the data type and information for this SEL-ish's from()?
@@ -96,8 +96,8 @@ private:
             } else if (adtypep->isRanged()) {
                 UASSERT_OBJ(
                     !(adtypep->rangep()
-                      && (!VN_IS(adtypep->rangep()->msbp(), Const)
-                          || !VN_IS(adtypep->rangep()->lsbp(), Const))),
+                      && (!VN_IS(adtypep->rangep()->leftp(), Const)
+                          || !VN_IS(adtypep->rangep()->rightp(), Const))),
                     nodep,
                     "Non-constant variable range; errored earlier");  // in constifyParam(bfdtypep)
                 fromRange = adtypep->declRange();
@@ -204,7 +204,7 @@ private:
         UINFO(6, "SELBIT " << nodep << endl);
         if (debug() >= 9) nodep->backp()->dumpTree(cout, "--SELBT0: ");
         // lhsp/rhsp do not need to be constant
-        AstNode* fromp = nodep->lhsp()->unlinkFrBack();
+        AstNode* fromp = nodep->fromp()->unlinkFrBack();
         AstNode* rhsp = nodep->rhsp()->unlinkFrBack();  // bit we're extracting
         if (debug() >= 9) nodep->dumpTree(cout, "--SELBT2: ");
         FromData fromdata = fromDataForArray(nodep, fromp);
@@ -225,12 +225,10 @@ private:
         } else if (AstPackArrayDType* adtypep = VN_CAST(ddtypep, PackArrayDType)) {
             // SELBIT(array, index) -> SEL(array, index*width-of-subindex, width-of-subindex)
             AstNode* subp = rhsp;
-            if (fromRange.lo() != 0 || fromRange.hi() < 0) {
-                if (fromRange.littleEndian()) {
-                    subp = newSubNeg(fromRange.hi(), subp);
-                } else {
-                    subp = newSubNeg(subp, fromRange.lo());
-                }
+            if (fromRange.littleEndian()) {
+                subp = newSubNeg(fromRange.hi(), subp);
+            } else {
+                subp = newSubNeg(subp, fromRange.lo());
             }
             UASSERT_OBJ(!(!fromRange.elements() || (adtypep->width() % fromRange.elements()) != 0),
                         adtypep,
@@ -315,7 +313,7 @@ private:
             nodep->replaceWith(fromp);
             VL_DO_DANGLING(pushDeletep(nodep), nodep);
         }
-        if (!rhsp->backp()) { VL_DO_DANGLING(pushDeletep(rhsp), rhsp); }
+        if (!rhsp->backp()) VL_DO_DANGLING(pushDeletep(rhsp), rhsp);
     }
     virtual void visit(AstSelExtract* nodep) override {
         // Select of a range specified part of an array, i.e. "array[2:3]"
@@ -324,14 +322,14 @@ private:
         UINFO(6, "SELEXTRACT " << nodep << endl);
         // if (debug() >= 9) nodep->dumpTree(cout, "--SELEX0: ");
         // Below 2 lines may change nodep->widthp()
-        V3Const::constifyParamsEdit(nodep->lsbp());  // May relink pointed to node
-        V3Const::constifyParamsEdit(nodep->msbp());  // May relink pointed to node
+        V3Const::constifyParamsEdit(nodep->leftp());  // May relink pointed to node
+        V3Const::constifyParamsEdit(nodep->rightp());  // May relink pointed to node
         // if (debug() >= 9) nodep->dumpTree(cout, "--SELEX3: ");
-        checkConstantOrReplace(nodep->lsbp(),
+        checkConstantOrReplace(nodep->leftp(),
                                "First value of [a:b] isn't a constant, maybe you want +: or -:");
-        checkConstantOrReplace(nodep->msbp(),
+        checkConstantOrReplace(nodep->rightp(),
                                "Second value of [a:b] isn't a constant, maybe you want +: or -:");
-        AstNode* fromp = nodep->lhsp()->unlinkFrBack();
+        AstNode* fromp = nodep->fromp()->unlinkFrBack();
         AstNode* msbp = nodep->rhsp()->unlinkFrBack();
         AstNode* lsbp = nodep->thsp()->unlinkFrBack();
         vlsint32_t msb = VN_CAST(msbp, Const)->toSInt();
@@ -351,9 +349,9 @@ private:
                 nodep->replaceWith(newp);
                 VL_DO_DANGLING(pushDeletep(nodep), nodep);
             } else {  // Slice
-                AstSliceSel* newp = new AstSliceSel(
-                    nodep->fileline(), fromp,
-                    VNumRange(VNumRange::LeftRight(), msb - fromRange.lo(), lsb - fromRange.lo()));
+                AstSliceSel* newp
+                    = new AstSliceSel{nodep->fileline(), fromp,
+                                      VNumRange{msb - fromRange.lo(), lsb - fromRange.lo()}};
                 nodep->replaceWith(newp);
                 VL_DO_DANGLING(pushDeletep(nodep), nodep);
             }
@@ -371,10 +369,11 @@ private:
                 lsb = x;
             }
             if (lsb > msb) {
-                nodep->v3error("["
-                               << msb << ":" << lsb
-                               << "] Range extract has backward bit ordering, perhaps you wanted ["
-                               << lsb << ":" << msb << "]");
+                nodep->v3warn(
+                    SELRANGE,
+                    "[" << msb << ":" << lsb
+                        << "] Range extract has backward bit ordering, perhaps you wanted [" << lsb
+                        << ":" << msb << "]");
                 int x = msb;
                 msb = lsb;
                 lsb = x;
@@ -400,10 +399,11 @@ private:
                 lsb = x;
             }
             if (lsb > msb) {
-                nodep->v3error("["
-                               << msb << ":" << lsb
-                               << "] Range extract has backward bit ordering, perhaps you wanted ["
-                               << lsb << ":" << msb << "]");
+                nodep->v3warn(
+                    SELRANGE,
+                    "[" << msb << ":" << lsb
+                        << "] Range extract has backward bit ordering, perhaps you wanted [" << lsb
+                        << ":" << msb << "]");
                 int x = msb;
                 msb = lsb;
                 lsb = x;
@@ -421,10 +421,11 @@ private:
         } else if (VN_IS(ddtypep, NodeUOrStructDType)) {
             // Classes aren't little endian
             if (lsb > msb) {
-                nodep->v3error("["
-                               << msb << ":" << lsb
-                               << "] Range extract has backward bit ordering, perhaps you wanted ["
-                               << lsb << ":" << msb << "]");
+                nodep->v3warn(
+                    SELRANGE,
+                    "[" << msb << ":" << lsb
+                        << "] Range extract has backward bit ordering, perhaps you wanted [" << lsb
+                        << ":" << msb << "]");
                 int x = msb;
                 msb = lsb;
                 lsb = x;
@@ -457,9 +458,9 @@ private:
             VL_DO_DANGLING(pushDeletep(nodep), nodep);
         }
         // delete whatever we didn't use in reconstruction
-        if (!fromp->backp()) { VL_DO_DANGLING(pushDeletep(fromp), fromp); }
-        if (!msbp->backp()) { VL_DO_DANGLING(pushDeletep(msbp), msbp); }
-        if (!lsbp->backp()) { VL_DO_DANGLING(pushDeletep(lsbp), lsbp); }
+        if (!fromp->backp()) VL_DO_DANGLING(pushDeletep(fromp), fromp);
+        if (!msbp->backp()) VL_DO_DANGLING(pushDeletep(msbp), msbp);
+        if (!lsbp->backp()) VL_DO_DANGLING(pushDeletep(lsbp), lsbp);
     }
 
     void replaceSelPlusMinus(AstNodePreSel* nodep) {
@@ -474,7 +475,7 @@ private:
         checkConstantOrReplace(nodep->thsp(), "Width of :+ or :- bit extract isn't a constant");
         if (debug() >= 9) nodep->dumpTree(cout, "--SELPM3: ");
         // Now replace it with an AstSel
-        AstNode* fromp = nodep->lhsp()->unlinkFrBack();
+        AstNode* fromp = nodep->fromp()->unlinkFrBack();
         AstNode* rhsp = nodep->rhsp()->unlinkFrBack();
         AstNode* widthp = nodep->thsp()->unlinkFrBack();
         warnTri(rhsp);
@@ -562,9 +563,9 @@ private:
             VL_DO_DANGLING(pushDeletep(nodep), nodep);
         }
         // delete whatever we didn't use in reconstruction
-        if (!fromp->backp()) { VL_DO_DANGLING(pushDeletep(fromp), fromp); }
-        if (!rhsp->backp()) { VL_DO_DANGLING(pushDeletep(rhsp), rhsp); }
-        if (!widthp->backp()) { VL_DO_DANGLING(pushDeletep(widthp), widthp); }
+        if (!fromp->backp()) VL_DO_DANGLING(pushDeletep(fromp), fromp);
+        if (!rhsp->backp()) VL_DO_DANGLING(pushDeletep(rhsp), rhsp);
+        if (!widthp->backp()) VL_DO_DANGLING(pushDeletep(widthp), widthp);
     }
     virtual void visit(AstSelPlus* nodep) override { replaceSelPlusMinus(nodep); }
     virtual void visit(AstSelMinus* nodep) override { replaceSelPlusMinus(nodep); }
@@ -579,9 +580,9 @@ private:
 
 public:
     // CONSTRUCTORS
-    WidthSelVisitor() {}
+    WidthSelVisitor() = default;
+    virtual ~WidthSelVisitor() override = default;
     AstNode* mainAcceptEdit(AstNode* nodep) { return iterateSubtreeReturnEdits(nodep); }
-    virtual ~WidthSelVisitor() override {}
 };
 
 //######################################################################
