@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -14,8 +14,8 @@
 //
 //*************************************************************************
 
-#ifndef _VLCBUCKET_H_
-#define _VLCBUCKET_H_ 1
+#ifndef VERILATOR_VLCBUCKET_H_
+#define VERILATOR_VLCBUCKET_H_
 
 #include "config_build.h"
 #include "verilatedos.h"
@@ -25,21 +25,21 @@
 // This is a bitmap array - we store a single bit to indicate a test
 // has hit that point with sufficient coverage.
 
-class VlcBuckets {
+class VlcBuckets final {
 private:
     // MEMBERS
     vluint64_t* m_datap = nullptr;  ///< Pointer to first bucket (dynamically allocated)
     vluint64_t m_dataSize = 0;  ///< Current entries in m_datap
     vluint64_t m_bucketsCovered = 0;  ///< Num buckets with sufficient coverage
 
-    static inline vluint64_t covBit(vluint64_t point) { return 1ULL << (point & 63); }
-    inline vluint64_t allocSize() const { return sizeof(vluint64_t) * m_dataSize / 64; }
+    static vluint64_t covBit(vluint64_t point) { return 1ULL << (point & 63); }
+    vluint64_t allocSize() const { return sizeof(vluint64_t) * m_dataSize / 64; }
     void allocate(vluint64_t point) {
         vluint64_t oldsize = m_dataSize;
         if (m_dataSize < point) m_dataSize = (point + 64) & ~63ULL;  // Keep power of two
         m_dataSize *= 2;
         // UINFO(9, "Realloc "<<allocSize()<<" for "<<point<<"  "<<cvtToHex(m_datap)<<endl);
-        vluint64_t* newp = static_cast<vluint64_t*>(realloc(m_datap, allocSize()));
+        vluint64_t* newp = static_cast<vluint64_t*>(std::realloc(m_datap, allocSize()));
         if (VL_UNCOVERABLE(!newp)) {
             // cppcheck-suppress doubleFree  // cppcheck 1.90 bug - realloc doesn't free on fail
             free(m_datap);  // LCOV_EXCL_LINE
