@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -24,7 +24,7 @@
 #include "verilatedos.h"
 
 // Cheat for speed and compile .cpp files into one object
-#define _V3ERROR_NO_GLOBAL_ 1
+#define V3ERROR_NO_GLOBAL_
 #include "V3Error.cpp"
 #include "V3String.cpp"
 #include "V3Os.cpp"
@@ -70,8 +70,6 @@ void VlcOptions::parseOptsList(int argc, char** argv) {
     // Parse parameters
     // Note argc and argv DO NOT INCLUDE the filename in [0]!!!
     // May be called recursively when there are -f files.
-#define shift \
-    do { ++i; } while (false)
     for (int i = 0; i < argc;) {
         UINFO(9, " Option: " << argv[i] << endl);
         if (argv[i][0] == '-') {
@@ -89,39 +87,38 @@ void VlcOptions::parseOptsList(int argc, char** argv) {
             }
             // Parameterized switches
             else if (!strcmp(sw, "-annotate-min") && (i + 1) < argc) {
-                shift;
+                ++i;
                 m_annotateMin = atoi(argv[i]);
             } else if (!strcmp(sw, "-annotate") && (i + 1) < argc) {
-                shift;
+                ++i;
                 m_annotateOut = argv[i];
             } else if (!strcmp(sw, "-debug")) {
                 V3Error::debugDefault(3);
             } else if (!strcmp(sw, "-debugi") && (i + 1) < argc) {
-                shift;
+                ++i;
                 V3Error::debugDefault(atoi(argv[i]));
             } else if (!strcmp(sw, "-V")) {
                 showVersion(true);
-                exit(0);
+                std::exit(0);
             } else if (!strcmp(sw, "-version")) {
                 showVersion(false);
-                exit(0);
+                std::exit(0);
             } else if (!strcmp(sw, "-write") && (i + 1) < argc) {
-                shift;
+                ++i;
                 m_writeFile = argv[i];
             } else if (!strcmp(sw, "-write-info") && (i + 1) < argc) {
-                shift;
+                ++i;
                 m_writeInfoFile = argv[i];
             } else {
                 v3fatal("Invalid option: " << argv[i]);
             }
-            shift;
+            ++i;
         }  // - options
         else {
             addReadFile(argv[i]);
-            shift;
+            ++i;
         }
     }
-#undef shift
 }
 
 void VlcOptions::showVersion(bool verbose) {
@@ -130,7 +127,7 @@ void VlcOptions::showVersion(bool verbose) {
     if (!verbose) return;
 
     cout << endl;
-    cout << "Copyright 2003-2020 by Wilson Snyder.  Verilator is free software; you can\n";
+    cout << "Copyright 2003-2021 by Wilson Snyder.  Verilator is free software; you can\n";
     cout << "redistribute it and/or modify the Verilator internals under the terms of\n";
     cout << "either the GNU Lesser General Public License Version 3 or the Perl Artistic\n";
     cout << "License Version 2.0.\n";
