@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -14,8 +14,8 @@
 //
 //*************************************************************************
 
-#ifndef _VLCOPTIONS_H_
-#define _VLCOPTIONS_H_ 1
+#ifndef VERILATOR_VLCOPTIONS_H_
+#define VERILATOR_VLCOPTIONS_H_
 
 #include "config_build.h"
 #include "verilatedos.h"
@@ -29,9 +29,9 @@
 //######################################################################
 // V3Options - Command line options
 
-typedef std::set<string> VlStringSet;
+using VlStringSet = std::set<std::string>;
 
-class VlcOptions {
+class VlcOptions final {
     // MEMBERS (general options)
     // clang-format off
     string m_annotateOut;       // main switch: --annotate I<output_directory>
@@ -51,8 +51,8 @@ private:
 
 public:
     // CONSTRUCTORS
-    VlcOptions() {}
-    ~VlcOptions() {}
+    VlcOptions() = default;
+    ~VlcOptions() = default;
 
     // METHODS
     void parseOptsList(int argc, char** argv);
