@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -14,8 +14,8 @@
 //
 //*************************************************************************
 
-#ifndef _VLCPOINT_H_
-#define _VLCPOINT_H_ 1
+#ifndef VERILATOR_VLCPOINT_H_
+#define VERILATOR_VLCPOINT_H_
 
 #include "config_build.h"
 #include "verilatedos.h"
@@ -29,7 +29,7 @@
 //********************************************************************
 // VlcPoint - A coverage point (across all tests)
 
-class VlcPoint {
+class VlcPoint final {
 private:
     // MEMBERS
     string m_name;  //< Name of the point
@@ -42,7 +42,7 @@ public:
     VlcPoint(const string& name, vluint64_t pointNum)
         : m_name{name}
         , m_pointNum{pointNum} {}
-    ~VlcPoint() {}
+    ~VlcPoint() = default;
     // ACCESSORS
     const string& name() const { return m_name; }
     vluint64_t pointNum() const { return m_pointNum; }
@@ -56,16 +56,16 @@ public:
     string type() const { return keyExtract(VL_CIK_TYPE); }
     string thresh() const { return keyExtract(VL_CIK_THRESH); }  // string as maybe ""
     string linescov() const { return keyExtract(VL_CIK_LINESCOV); }
-    int lineno() const { return atoi(keyExtract(VL_CIK_LINENO).c_str()); }
-    int column() const { return atoi(keyExtract(VL_CIK_COLUMN).c_str()); }
+    int lineno() const { return std::atoi(keyExtract(VL_CIK_LINENO).c_str()); }
+    int column() const { return std::atoi(keyExtract(VL_CIK_COLUMN).c_str()); }
     // METHODS
     string keyExtract(const char* shortKey) const {
         // Hot function
-        size_t shortLen = strlen(shortKey);
+        size_t shortLen = std::strlen(shortKey);
         const string namestr = name();
         for (const char* cp = namestr.c_str(); *cp; ++cp) {
             if (*cp == '\001') {
-                if (0 == strncmp(cp + 1, shortKey, shortLen) && cp[shortLen + 1] == '\002') {
+                if (0 == std::strncmp(cp + 1, shortKey, shortLen) && cp[shortLen + 1] == '\002') {
                     cp += shortLen + 2;  // Skip \001+short+\002
                     const char* ep = cp;
                     while (*ep && *ep != '\001') ++ep;
@@ -77,37 +77,37 @@ public:
     }
     static void dumpHeader() {
         cout << "Points:\n";
-        cout << "  Num,    TestsCover,    Count,  Name" << endl;
+        cout << "  Num,    TestsCover,    Count,  Name\n";
     }
     void dump() const {
         cout << "  " << std::setw(8) << std::setfill('0') << pointNum();
         cout << ",  " << std::setw(7) << std::setfill(' ') << testsCovering();
         cout << ",  " << std::setw(7) << std::setfill(' ') << count();
-        cout << ",  \"" << name() << "\"" << endl;
+        cout << ",  \"" << name() << "\"\n";
     }
 };
 
 //********************************************************************
 // VlcPoints - Container of all points
 
-class VlcPoints {
+class VlcPoints final {
 private:
     // MEMBERS
-    typedef std::map<const string, vluint64_t> NameMap;  // Sorted by name (ordered)
+    using NameMap = std::map<const std::string, vluint64_t>;  // Sorted by name (ordered)
     NameMap m_nameMap;  //< Name to point-number
     std::vector<VlcPoint> m_points;  //< List of all points
     vluint64_t m_numPoints = 0;  //< Total unique points
 
 public:
     // ITERATORS
-    typedef NameMap ByName;
-    typedef ByName::iterator iterator;
+    using ByName = NameMap;
+    using iterator = ByName::iterator;
     ByName::iterator begin() { return m_nameMap.begin(); }
     ByName::iterator end() { return m_nameMap.end(); }
 
     // CONSTRUCTORS
-    VlcPoints() {}
-    ~VlcPoints() {}
+    VlcPoints() = default;
+    ~VlcPoints() = default;
 
     // METHODS
     void dump() {
@@ -130,7 +130,7 @@ public:
             VlcPoint point(name, pointnum);
             point.countInc(count);
             m_points.push_back(point);
-            m_nameMap.insert(make_pair(point.name(), point.pointNum()));
+            m_nameMap.emplace(point.name(), point.pointNum());
         }
         return pointnum;
     }
