@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -14,8 +14,8 @@
 //
 //*************************************************************************
 
-#ifndef _VLCTEST_H_
-#define _VLCTEST_H_ 1
+#ifndef VERILATOR_VLCTEST_H_
+#define VERILATOR_VLCTEST_H_
 
 #include "config_build.h"
 #include "verilatedos.h"
@@ -29,7 +29,7 @@
 //********************************************************************
 // VlcTest - a single testrun i.e. a file containing coverage data
 
-class VlcTest {
+class VlcTest final {
 private:
     // MEMBERS
     string m_name;  //< Name of the test
@@ -46,7 +46,7 @@ public:
         : m_name{name}
         , m_computrons{comp}
         , m_testrun{testrun} {}
-    ~VlcTest() {}
+    ~VlcTest() = default;
 
     // ACCESSORS
     const string& name() const { return m_name; }
@@ -65,7 +65,7 @@ public:
     static void dumpHeader() {
         cout << "Tests:\n";
         // cout<<"  Testrun, Computrons,";  // Currently not loaded
-        cout << "  Covered,     Rank,  RankPts,  Filename" << endl;
+        cout << "  Covered,     Rank,  RankPts,  Filename\n";
     }
     void dump(bool bucketsToo) {
         if (testrun() || computrons() != 0.0) {  // currently unused // LCOV_EXCL_LINE
@@ -76,7 +76,7 @@ public:
         cout << "  " << std::setw(7) << std::setfill(' ') << bucketsCovered();
         cout << ",  " << std::setw(7) << std::setfill(' ') << rank();
         cout << ",  " << std::setw(7) << std::setfill(' ') << rankPoints();
-        cout << ",  \"" << name() << "\"" << endl;
+        cout << ",  \"" << name() << "\"\n";
         if (bucketsToo) m_buckets.dump();
     }
 };
@@ -84,10 +84,10 @@ public:
 //********************************************************************
 // VlcTests - Container of all tests
 
-class VlcTests {
+class VlcTests final {
 public:
     // TYPES
-    typedef std::vector<VlcTest*> ByName;
+    using ByName = std::vector<VlcTest*>;
 
 private:
     // MEMBERS
@@ -95,12 +95,12 @@ private:
 
 public:
     // ITERATORS
-    typedef ByName::iterator iterator;
+    using iterator = ByName::iterator;
     ByName::iterator begin() { return m_tests.begin(); }
     ByName::iterator end() { return m_tests.end(); }
 
     // CONSTRUCTORS
-    VlcTests() {}
+    VlcTests() = default;
     ~VlcTests() {
         for (auto it = begin(); it != end(); ++it) { VL_DO_CLEAR(delete *it, *it = nullptr); }
     }
