@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2020 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2021 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -45,7 +45,7 @@ void VlcTop::readCoverage(const string& filename, bool nonfatal) {
                 if (line[secspace] == '\'' && line[secspace + 1] == ' ') break;
             }
             string point = line.substr(3, secspace - 3);
-            vluint64_t hits = atoll(line.c_str() + secspace + 1);
+            vluint64_t hits = std::atoll(line.c_str() + secspace + 1);
             // UINFO(9,"   point '"<<point<<"'"<<" "<<hits<<endl);
 
             vluint64_t pointnum = points().findAddPoint(point, hits);
@@ -69,10 +69,10 @@ void VlcTop::writeCoverage(const string& filename) {
         return;
     }
 
-    os << "# SystemC::Coverage-3" << endl;
+    os << "# SystemC::Coverage-3\n";
     for (const auto& i : m_points) {
         const VlcPoint& point = m_points.pointNumber(i.second);
-        os << "C '" << point.name() << "' " << point.count() << endl;
+        os << "C '" << point.name() << "' " << point.count() << '\n';
     }
 }
 
@@ -109,7 +109,7 @@ void VlcTop::writeInfo(const string& filename) {
     os << "TN:verilator_coverage\n";
     for (auto& si : m_sources) {
         VlcSource& source = si.second;
-        os << "SF:" << source.name() << endl;
+        os << "SF:" << source.name() << '\n';
         VlcSource::LinenoMap& lines = source.lines();
         for (auto& li : lines) {
             int lineno = li.first;
@@ -134,7 +134,7 @@ void VlcTop::writeInfo(const string& filename) {
 //********************************************************************
 
 struct CmpComputrons {
-    inline bool operator()(const VlcTest* lhsp, const VlcTest* rhsp) const {
+    bool operator()(const VlcTest* lhsp, const VlcTest* rhsp) const {
         if (lhsp->computrons() != rhsp->computrons()) {
             return lhsp->computrons() < rhsp->computrons();
         }
@@ -203,10 +203,11 @@ void VlcTop::annotateCalc() {
         if (!filename.empty() && lineno != 0) {
             VlcSource& source = sources().findNewSource(filename);
             string threshStr = point.thresh();
-            unsigned thresh = (!threshStr.empty()) ? atoi(threshStr.c_str()) : opt.annotateMin();
+            unsigned thresh
+                = (!threshStr.empty()) ? std::atoi(threshStr.c_str()) : opt.annotateMin();
             bool ok = (point.count() >= thresh);
             UINFO(9, "AnnoCalc count " << filename << ":" << lineno << ":" << point.column() << " "
-                                       << point.count() << " " << point.linescov() << endl);
+                                       << point.count() << " " << point.linescov() << '\n');
             // Base coverage
             source.incCount(lineno, point.column(), point.count(), ok);
             // Additional lines covered by this statement
@@ -219,18 +220,18 @@ void VlcTop::annotateCalc() {
                     for (int lni = start; start && lni <= end; ++lni) {
                         source.incCount(lni, point.column(), point.count(), ok);
                     }
-                    if (!*covp) { break; }
+                    if (!*covp) break;
                     start = 0;  // Prep for next
                     end = 0;
                     range = false;
                 } else if (*covp == '-') {
                     range = true;
-                } else if (isdigit(*covp)) {
+                } else if (std::isdigit(*covp)) {
                     const char* digitsp = covp;
-                    while (isdigit(*covp)) ++covp;
+                    while (std::isdigit(*covp)) ++covp;
                     --covp;  // Will inc in for loop
-                    if (!range) start = atoi(digitsp);
-                    end = atoi(digitsp);
+                    if (!range) start = std::atoi(digitsp);
+                    end = std::atoi(digitsp);
                 }
             }
         }
@@ -263,8 +264,8 @@ void VlcTop::annotateCalcNeeded() {
     }
     float pct = totCases ? (100 * totOk / totCases) : 0;
     cout << "Total coverage (" << totOk << "/" << totCases << ") ";
-    cout << std::fixed << std::setw(3) << std::setprecision(2) << pct << "%" << endl;
-    if (totOk != totCases) cout << "See lines with '%00' in " << opt.annotateOut() << endl;
+    cout << std::fixed << std::setw(3) << std::setprecision(2) << pct << "%\n";
+    if (totOk != totCases) cout << "See lines with '%00' in " << opt.annotateOut() << '\n';
 }
 
 void VlcTop::annotateOutputFiles(const string& dirname) {
@@ -290,7 +291,7 @@ void VlcTop::annotateOutputFiles(const string& dirname) {
             return;
         }
 
-        os << "\t// verilator_coverage annotation" << endl;
+        os << "\t// verilator_coverage annotation\n";
 
         int lineno = 0;
         while (!is.eof()) {
@@ -308,13 +309,13 @@ void VlcTop::annotateOutputFiles(const string& dirname) {
                     // UINFO(0,"Source
                     // "<<source.name()<<":"<<col.lineno()<<":"<<col.column()<<endl);
                     os << (col.ok() ? " " : "%") << std::setfill('0') << std::setw(6)
-                       << col.count() << "\t" << line << endl;
+                       << col.count() << "\t" << line << '\n';
                     if (first) {
                         first = false;
                         // Multiple columns on same line; print line just once
                         string indent;
                         for (string::const_iterator pos = line.begin();
-                             pos != line.end() && isspace(*pos); ++pos) {
+                             pos != line.end() && std::isspace(*pos); ++pos) {
                             indent += *pos;
                         }
                         line = indent + "verilator_coverage: (next point on previous line)\n";
@@ -322,7 +323,7 @@ void VlcTop::annotateOutputFiles(const string& dirname) {
                 }
             }
 
-            if (first) os << "\t" << line << endl;
+            if (first) os << "\t" << line << '\n';
         }
     }
 }
