@@ -12,6 +12,8 @@
 #include "Vt_enum_public_p3.h"
 #include "Vt_enum_public_p62.h"
 
+double sc_time_stamp() { return 0; }
+
 int main(int argc, char* argv[]) {
     Vt_enum_public* topp = new Vt_enum_public;
 
@@ -24,4 +26,7 @@ int main(int argc, char* argv[]) {
     for (int i = 0; i < 10; i++) {  //
         topp->eval();
     }
+
+    topp->final();
+    VL_DO_DANGLING(delete topp, topp);
 }
