@@ -1,4 +1,4 @@
-#!/usr/bin/perl
+#!/usr/bin/env perl
 if (!$::Driver) { use FindBin; exec("$FindBin::Bin/bootstrap.pl", @ARGV, $0); die; }
 # DESCRIPTION: Verilator: Verilog Test driver/expect definition
 #
@@ -13,7 +13,7 @@ scenarios(simulator => 1);
 top_filename("t/t_inst_tree.v");
 
 compile(
-    verilator_flags2 => ['--stats', '--norelative-cfuncs',
+    verilator_flags2 => ['--stats', '-Wno-DEPRECATED', '--norelative-cfuncs',
                          "$Self->{t_dir}/t_inst_tree_inl0_pub1.vlt",
                          $Self->wno_unopthreads_for_few_cores()]
     );
