@@ -12,6 +12,8 @@
 #include "Vt_param_public_p.h"
 #include "Vt_param_public_t.h"
 
+double sc_time_stamp() { return 0; }
+
 int main(int argc, char* argv[]) {
     Vt_param_public* topp = new Vt_param_public;
 
@@ -26,4 +28,7 @@ int main(int argc, char* argv[]) {
     for (int i = 0; i < 10; i++) {  //
         topp->eval();
     }
+
+    topp->final();
+    VL_DO_DANGLING(delete topp, topp);
 }
