@@ -46,6 +46,8 @@ int main(int argc, char** argv, char** env) {
     }
     tfp->close();
     top->final();
+    VL_DO_DANGLING(delete top, top);
+    VL_DO_DANGLING(delete tfp, tfp);
     printf("*-* All Finished *-*\n");
     return 0;
 }
