@@ -1,4 +1,4 @@
-#!/usr/bin/perl
+#!/usr/bin/env perl
 if (!$::Driver) { use FindBin; exec("$FindBin::Bin/bootstrap.pl", @ARGV, $0); die; }
 # DESCRIPTION: Verilator: Verilog Test driver/expect definition
 #
@@ -23,7 +23,7 @@ compile(
     );
 
 execute(
-    iv_pli => 1,
+    use_libvpi => 1,
     check_finished => 1
     );
 
