@@ -1,4 +1,4 @@
-#!/usr/bin/perl
+#!/usr/bin/env perl
 if (!$::Driver) { use FindBin; exec("$FindBin::Bin/bootstrap.pl", @ARGV, $0); die; }
 # DESCRIPTION: Verilator: Verilog Test driver/expect definition
 #
@@ -8,23 +8,19 @@ if (!$::Driver) { use FindBin; exec("$FindBin::Bin/bootstrap.pl", @ARGV, $0); di
 # Version 2.0.
 # SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0
 
-scenarios(simulator => 1, iv => 1);
+scenarios(simulator => 1);
 
 compile(
     make_top_shell => 0,
     make_main => 0,
-    make_pli => 1,
     sim_time => 2100,
-    iv_flags2 => ["-g2005-sv -D USE_VPI_NOT_DPI -DWAVES -DIVERILOG"],
-    v_flags2 => ["+define+USE_VPI_NOT_DPI"],
-    verilator_flags2 => ["-CFLAGS '-DVL_DEBUG -ggdb' --exe --vpi --no-l2name $Self->{t_dir}/t_vpi_time_cb.cpp -LDFLAGS '-ldl -rdynamic'"],
+    v_flags2 => ["t/t_vpi_time_cb_c.cpp"],
+    iv_flags2 => ["-g2005-sv -DWAVES -DIVERILOG"],
+    verilator_flags2 => ["-CFLAGS '-DVL_DEBUG -ggdb' --exe --vpi --no-l2name $Self->{t_dir}/t_vpi_time_cb.cpp"],
     );
 
 execute(
-    iv_pli => 1,
-    ms_pli => 1,
     check_finished => 1,
-    all_run_flags => ['+PLUS +INT=1234 +STRSTR']
     );
 
 ok(1);
